@@ -54,6 +54,8 @@ optional<Action> fromString(string const& name)
         mapAction<actions::ResetConfig>("ResetConfig"),
         mapAction<actions::ResetFontSize>("ResetFontSize"),
         mapAction<actions::ScreenshotVT>("ScreenshotVT"),
+        mapAction<actions::SaveScreenshot>("SaveScreenshot"),
+        mapAction<actions::CopyScreenshot>("CopyScreenshot"),
         mapAction<actions::ScrollDown>("ScrollDown"),
         mapAction<actions::ScrollMarkDown>("ScrollMarkDown"),
         mapAction<actions::ScrollMarkUp>("ScrollMarkUp"),
@@ -68,6 +70,7 @@ optional<Action> fromString(string const& name)
         mapAction<actions::SendChars>("SendChars"),
         mapAction<actions::ToggleAllKeyMaps>("ToggleAllKeyMaps"),
         mapAction<actions::ToggleFullscreen>("ToggleFullscreen"),
+        mapAction<actions::ToggleInputMethodHandling>("ToggleInputMethodHandling"),
         mapAction<actions::ToggleInputProtection>("ToggleInputProtection"),
         mapAction<actions::ToggleStatusLine>("ToggleStatusLine"),
         mapAction<actions::ToggleTitleBar>("ToggleTitleBar"),
@@ -77,6 +80,16 @@ optional<Action> fromString(string const& name)
         mapAction<actions::TraceStep>("TraceStep"),
         mapAction<actions::ViNormalMode>("ViNormalMode"),
         mapAction<actions::WriteScreen>("WriteScreen"),
+        mapAction<actions::CreateNewTab>("CreateNewTab"),
+        mapAction<actions::CloseTab>("CloseTab"),
+        mapAction<actions::MoveTabTo>("MoveTabTo"),
+        mapAction<actions::MoveTabToLeft>("MoveTabToLeft"),
+        mapAction<actions::MoveTabToRight>("MoveTabToRight"),
+        mapAction<actions::SwitchToTab>("SwitchToTab"),
+        mapAction<actions::SwitchToPreviousTab>("SwitchToPreviousTab"),
+        mapAction<actions::SwitchToTabLeft>("SwitchToTabLeft"),
+        mapAction<actions::SwitchToTabRight>("SwitchToTabRight"),
+        mapAction<actions::SetTabName>("SetTabName"),
     };
 
     auto const lowerCaseName = toLower(name);
