@@ -2,10 +2,13 @@
 #pragma once
 
 #include <crispy/assert.h>
+#include <crispy/utils.h>
 
+#include <array>
 #include <format>
 #include <optional>
 #include <string>
+#include <tuple>
 #include <variant>
 
 namespace contour::actions
@@ -70,6 +73,7 @@ struct SearchReverse{};
 struct SendChars{ std::string chars; };
 struct ToggleAllKeyMaps{};
 struct ToggleFullscreen{};
+struct ToggleInputMethodHandling {};
 struct ToggleInputProtection{};
 struct ToggleStatusLine{};
 struct ToggleTitleBar{};
@@ -81,10 +85,14 @@ struct ViNormalMode{};
 struct WriteScreen{ std::string chars; }; // "\033[2J\033[3J"
 struct CreateNewTab{};
 struct CloseTab{};
+struct MoveTabTo{ int position; };
+struct MoveTabToLeft{};
+struct MoveTabToRight{};
 struct SwitchToTab{ int position; };
 struct SwitchToPreviousTab{};
 struct SwitchToTabLeft{};
 struct SwitchToTabRight{};
+struct SetTabName{};
 // clang-format on
 
 using Action = std::variant<CancelSelection,
@@ -129,6 +137,7 @@ using Action = std::variant<CancelSelection,
                             SendChars,
                             ToggleAllKeyMaps,
                             ToggleFullscreen,
+                            ToggleInputMethodHandling,
                             ToggleInputProtection,
                             ToggleStatusLine,
                             ToggleTitleBar,
@@ -140,10 +149,17 @@ using Action = std::variant<CancelSelection,
                             WriteScreen,
                             CreateNewTab,
                             CloseTab,
+                            MoveTabTo,
+                            MoveTabToLeft,
+                            MoveTabToRight,
                             SwitchToTab,
                             SwitchToPreviousTab,
                             SwitchToTabLeft,
-                            SwitchToTabRight>;
+                            SwitchToTabRight,
+                            SetTabName>;
+
+template <typename T>
+concept NonRepeatableActionConcept = crispy::one_of<T, CreateNewTab, CloseTab>;
 
 std::optional<Action> fromString(std::string const& name);
 
@@ -233,6 +249,9 @@ namespace documentation
                                                          "keybind will be preserved when disabling all "
                                                          "others)." };
     constexpr inline std::string_view ToggleFullscreen { "Enables/disables full screen mode." };
+    constexpr inline std::string_view ToggleInputMethodHandling {
+        "Enables/disables IME (input method editor) handling."
+    };
     constexpr inline std::string_view ToggleInputProtection { "Enables/disables terminal input protection." };
     constexpr inline std::string_view ToggleStatusLine {
         "Shows/hides the VT320 compatible Indicator status line."
@@ -259,15 +278,21 @@ namespace documentation
     };
     constexpr inline std::string_view CreateNewTab { "Creates a new tab in the terminal emulator." };
     constexpr inline std::string_view CloseTab { "Closes current tab." };
+    constexpr inline std::string_view MoveTabTo {
+        "Moves current tab to the given position (starting at number 1)."
+    };
+    constexpr inline std::string_view MoveTabToLeft { "Moves current tab to the left." };
+    constexpr inline std::string_view MoveTabToRight { "Moves current tab to the right." };
     constexpr inline std::string_view SwitchToTab {
         "Switch to absolute tab position (starting at number 1)"
     };
     constexpr inline std::string_view SwitchToPreviousTab { "Switch to the previously focused tab" };
     constexpr inline std::string_view SwitchToTabLeft { "Switch to tab to the left" };
     constexpr inline std::string_view SwitchToTabRight { "Switch to tab to the right" };
+    constexpr inline std::string_view SetTabName { "Set the name of the current tab" };
 } // namespace documentation
 
-inline auto getDocumentation()
+constexpr inline auto getDocumentation()
 {
     return std::array {
         std::tuple { Action { CancelSelection {} }, documentation::CancelSelection },
@@ -312,6 +337,7 @@ inline auto getDocumentation()
         std::tuple { Action { SendChars {} }, documentation::SendChars },
         std::tuple { Action { ToggleAllKeyMaps {} }, documentation::ToggleAllKeyMaps },
         std::tuple { Action { ToggleFullscreen {} }, documentation::ToggleFullscreen },
+        std::tuple { Action { ToggleInputMethodHandling {} }, documentation::ToggleInputMethodHandling },
         std::tuple { Action { ToggleInputProtection {} }, documentation::ToggleInputProtection },
         std::tuple { Action { ToggleStatusLine {} }, documentation::ToggleStatusLine },
         std::tuple { Action { ToggleTitleBar {} }, documentation::ToggleTitleBar },
@@ -323,13 +349,21 @@ inline auto getDocumentation()
         std::tuple { Action { WriteScreen {} }, documentation::WriteScreen },
         std::tuple { Action { CreateNewTab {} }, documentation::CreateNewTab },
         std::tuple { Action { CloseTab {} }, documentation::CloseTab },
+        std::tuple { Action { MoveTabTo {} }, documentation::MoveTabTo },
+        std::tuple { Action { MoveTabToLeft {} }, documentation::MoveTabToLeft },
+        std::tuple { Action { MoveTabToRight {} }, documentation::MoveTabToRight },
         std::tuple { Action { SwitchToTab {} }, documentation::SwitchToTab },
         std::tuple { Action { SwitchToPreviousTab {} }, documentation::SwitchToPreviousTab },
         std::tuple { Action { SwitchToTabLeft {} }, documentation::SwitchToTabLeft },
         std::tuple { Action { SwitchToTabRight {} }, documentation::SwitchToTabRight },
+        std::tuple { Action { SetTabName {} }, documentation::SetTabName },
     };
 }
 
+#if defined(__clang__) && __clang_major__ >= 19
+static_assert(std::tuple_size_v<decltype(getDocumentation())> == std::variant_size_v<Action>);
+#endif
+
 } // namespace contour::actions
 
 // {{{ fmtlib custom formatters
@@ -386,6 +420,7 @@ DECLARE_ACTION_FMT(SearchReverse)
 DECLARE_ACTION_FMT(SendChars)
 DECLARE_ACTION_FMT(ToggleAllKeyMaps)
 DECLARE_ACTION_FMT(ToggleFullscreen)
+DECLARE_ACTION_FMT(ToggleInputMethodHandling)
 DECLARE_ACTION_FMT(ToggleInputProtection)
 DECLARE_ACTION_FMT(ToggleStatusLine)
 DECLARE_ACTION_FMT(ToggleTitleBar)
@@ -397,12 +432,24 @@ DECLARE_ACTION_FMT(ViNormalMode)
 DECLARE_ACTION_FMT(WriteScreen)
 DECLARE_ACTION_FMT(CreateNewTab)
 DECLARE_ACTION_FMT(CloseTab)
+DECLARE_ACTION_FMT(MoveTabToLeft)
+DECLARE_ACTION_FMT(MoveTabToRight)
 DECLARE_ACTION_FMT(SwitchToPreviousTab)
 DECLARE_ACTION_FMT(SwitchToTabLeft)
 DECLARE_ACTION_FMT(SwitchToTabRight)
+DECLARE_ACTION_FMT(SetTabName)
 // }}}
 #undef DECLARE_ACTION_FMT
 
+template <>
+struct std::formatter<contour::actions::MoveTabTo>: std::formatter<std::string>
+{
+    auto format(contour::actions::MoveTabTo const& value, auto& ctx) const
+    {
+        return formatter<string>::format(std::format("MoveTabTo {{ position: {} }}", value.position), ctx);
+    }
+};
+
 template <>
 struct std::formatter<contour::actions::SwitchToTab>: std::formatter<std::string>
 {
@@ -421,6 +468,7 @@ struct std::formatter<contour::actions::SwitchToTab>: std::formatter<std::string
 template <>
 struct std::formatter<contour::actions::Action>: std::formatter<std::string>
 {
+    // NOLINTNEXTLINE(readability-function-cognitive-complexity)
     auto format(contour::actions::Action const& _action, auto& ctx) const
     {
         std::string name = "Unknown action";
@@ -466,6 +514,7 @@ struct std::formatter<contour::actions::Action>: std::formatter<std::string>
         HANDLE_ACTION(SendChars);
         HANDLE_ACTION(ToggleAllKeyMaps);
         HANDLE_ACTION(ToggleFullscreen);
+        HANDLE_ACTION(ToggleInputMethodHandling);
         HANDLE_ACTION(ToggleInputProtection);
         HANDLE_ACTION(ToggleStatusLine);
         HANDLE_ACTION(ToggleTitleBar);
@@ -476,9 +525,17 @@ struct std::formatter<contour::actions::Action>: std::formatter<std::string>
         HANDLE_ACTION(ViNormalMode);
         HANDLE_ACTION(CreateNewTab);
         HANDLE_ACTION(CloseTab);
+        HANDLE_ACTION(MoveTabToLeft);
+        HANDLE_ACTION(MoveTabToRight);
         HANDLE_ACTION(SwitchToPreviousTab);
         HANDLE_ACTION(SwitchToTabLeft);
         HANDLE_ACTION(SwitchToTabRight);
+        HANDLE_ACTION(SetTabName);
+        if (std::holds_alternative<contour::actions::MoveTabTo>(_action))
+        {
+            const auto action = std::get<contour::actions::MoveTabTo>(_action);
+            name = std::format("MoveTabTo, position: {}", action.position);
+        }
         if (std::holds_alternative<contour::actions::SwitchToTab>(_action))
         {
             const auto action = std::get<contour::actions::SwitchToTab>(_action);
