@@ -5,12 +5,9 @@
 #include <crispy/logstore.h>
 #include <crispy/times.h>
 
-#include <qbuffer.h>
-#include <qthread.h>
+#include <QtMultimedia/QMediaDevices>
 
-#if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
-    #include <QtMultimedia/QMediaDevices>
-#endif
+#include <cmath>
 
 using namespace contour;
 
@@ -18,8 +15,14 @@ namespace
 {
 constexpr double SampleRate = 44100;
 
-// TODO make this function constexpr when we switch to c++23
-double square_wave(double x) noexcept
+// On Windows, this cannot be constexpr just yet, because std::fmod is not constexpr in MSVC (yet).
+#if !defined(_WIN32)
+    #define CONSTEXPR_UNLESS_MSVC constexpr
+#else
+    #define CONSTEXPR_UNLESS_MSVC
+#endif
+
+CONSTEXPR_UNLESS_MSVC double square_wave(double x) noexcept
 {
     x = std::fmod(x, 2);
     return std::isgreater(x, 1) ? -1 : 1;
@@ -44,16 +47,8 @@ Audio::Audio()
     f.setSampleRate(44100);
     f.setChannelCount(1);
 
-#if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
     f.setSampleFormat(QAudioFormat::Int16);
     QAudioDevice const info(QMediaDevices::defaultAudioOutput());
-#else
-    f.setSampleSize(16);
-    f.setCodec("audio/pcm");
-    f.setByteOrder(QAudioFormat::LittleEndian);
-    f.setSampleType(QAudioFormat::SignedInt);
-    QAudioDeviceInfo const info(QAudioDeviceInfo::defaultOutputDevice());
-#endif
 
     if (!info.isFormatSupported(f))
     {
@@ -61,11 +56,11 @@ Audio::Audio()
         return;
     }
 
-    _audioSink = std::make_unique<QtAudioSink>(f);
+    _audioSink = std::make_unique<QAudioSink>(f);
 
     _audioSink->moveToThread(&_soundThread);
 
-    connect(_audioSink.get(), &QtAudioSink::stateChanged, this, &Audio::handleStateChanged);
+    connect(_audioSink.get(), &QAudioSink::stateChanged, this, &Audio::handleStateChanged);
     qRegisterMetaType<std::vector<int>>();
     connect(this, &Audio::play, this, &Audio::handlePlayback);
     _soundThread.start();
