@@ -17,11 +17,19 @@
 #endif
 
 #if defined(CONTOUR_FRONTEND_XCB)
-    #if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
-        #include <QtGui/QGuiApplication>
-    #else
-        #include <QtX11Extras/QX11Info>
-    #endif
+    #include <QtGui/QGuiApplication>
+#endif
+
+#if defined(CONTOUR_WAYLAND) && QT_VERSION >= QT_VERSION_CHECK(6, 5, 0)
+    #include <QtGui/QGuiApplication>
+    #include <QtWaylandClient/QWaylandClientExtension>
+    #include <QtWaylandClient/private/qwaylandwindow_p.h>
+
+    #include <map>
+    #include <memory>
+
+    #include "qwayland-blur.h"
+    #include <qpa/qplatformwindow.h>
 #endif
 
 namespace BlurBehind
@@ -43,7 +51,6 @@ namespace
 
     xcb_connection_t* x11Connection()
     {
-    #if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
         if (!qApp)
             return nullptr;
 
@@ -52,9 +59,6 @@ namespace
             return nullptr;
 
         return native->connection();
-    #else
-        return QX11Info::connection();
-    #endif
     }
 
     optional<XcbPropertyInfo> queryXcbPropertyInfo(QWindow* window, string const& name)
@@ -124,27 +128,97 @@ namespace
 } // namespace
 #endif
 
+#if defined(CONTOUR_WAYLAND) && QT_VERSION >= QT_VERSION_CHECK(6, 5, 0)
+namespace
+{
+    class KWinBlurManager:
+        public QWaylandClientExtensionTemplate<KWinBlurManager>,
+        public QtWayland::org_kde_kwin_blur_manager
+    {
+      public:
+        KWinBlurManager(): QWaylandClientExtensionTemplate<KWinBlurManager>(1) { initialize(); }
+    };
+    Q_GLOBAL_STATIC(KWinBlurManager, kwinBlurManager)
+} // namespace
+#endif
+
 void setEnabled(QWindow* window, bool enable, QRegion const& region)
 {
     crispy::ignore_unused(region);
 
+#if defined(CONTOUR_WAYLAND) && QT_VERSION >= QT_VERSION_CHECK(6, 5, 0)
+    if (QGuiApplication::platformName() == "wayland")
+    {
+        static auto activeBlurs = std::map<QWindow*, std::unique_ptr<QtWayland::org_kde_kwin_blur>> {};
+
+        if (auto* blurManager = kwinBlurManager(); blurManager && blurManager->isActive())
+        {
+            if (auto* platformWindow = window->handle())
+            {
+                if (auto* waylandWindow = static_cast<QtWaylandClient::QWaylandWindow*>(platformWindow))
+                {
+                    if (auto* surface = waylandWindow->surface())
+                    {
+                        if (enable)
+                        {
+                            if (activeBlurs.find(window) == activeBlurs.end())
+                            {
+                                auto* rawBlur = blurManager->create(surface);
+                                activeBlurs[window] = std::make_unique<QtWayland::org_kde_kwin_blur>(rawBlur);
+                                QObject::connect(window, &QWindow::destroyed, [window]() {
+                                    auto it = activeBlurs.find(window);
+                                    if (it != activeBlurs.end())
+                                    {
+                                        it->second->release();
+                                        activeBlurs.erase(it);
+                                    }
+                                });
+                            }
+                            auto& blur = *activeBlurs[window];
+                            blur.set_region(nullptr);
+                            blur.commit();
+                        }
+                        else
+                        {
+                            auto it = activeBlurs.find(window);
+                            if (it != activeBlurs.end())
+                            {
+                                it->second->release();
+                                activeBlurs.erase(it);
+                            }
+                            blurManager->unset(surface);
+                        }
+                    }
+                }
+            }
+        }
+        return;
+    }
+#endif
+
 #if !defined(Q_OS_WINDOWS) && !defined(Q_OS_DARWIN)
     // This #if should catch UNIX in general but not Mac, so we have not just Linux but also the BSDs and
     // maybe others if one wants to.
     //
     // I was looking into the kwin source code and it's all in fact just a one-liner, so easy to get rid of
     // the dependency and still support nice looking semi transparent blurred backgrounds.
-    if (enable)
+    if (QGuiApplication::platformName() == "xcb")
     {
-        setPropertyX11(window, "_KDE_NET_WM_BLUR_BEHIND_REGION", uint32_t { 0 });
-        setPropertyX11(window, "_MUTTER_HINTS", "blur-provider=sigma:60,brightness:0.9");
+        if (enable)
+        {
+            setPropertyX11(window, "_KDE_NET_WM_BLUR_BEHIND_REGION", uint32_t { 0 });
+            setPropertyX11(window, "_MUTTER_HINTS", "blur-provider=sigma:60,brightness:0.9");
+        }
+        else
+        {
+            unsetPropertyX11(window, "kwin_blur");
+            unsetPropertyX11(window, "_MUTTER_HINTS");
+        }
+        return;
     }
-    else
-    {
-        unsetPropertyX11(window, "kwin_blur");
-        unsetPropertyX11(window, "_MUTTER_HINTS");
-    }
-#elif defined(_WIN32) // {{{
+#endif
+
+#if defined(_WIN32) // {{{
     // Awesome hack with the noteworty links:
     // * https://gist.github.com/ethanhs/0e157e4003812e99bf5bc7cb6f73459f (used as code template)
     // * https://github.com/riverar/sample-win32-acrylicblur/blob/master/MainWindow.xaml.cs
@@ -153,6 +227,37 @@ void setEnabled(QWindow* window, bool enable, QRegion const& region)
 
     if (HWND hwnd = (HWND) window->winId(); hwnd != nullptr)
     {
+        // 1. Attempt DWM-based blur (Windows 11+)
+        const HMODULE hDwm = LoadLibrary(TEXT("dwmapi.dll"));
+        if (hDwm)
+        {
+            typedef HRESULT(WINAPI * P_DwmSetWindowAttribute)(HWND, DWORD, LPCVOID, DWORD);
+            const auto pDwmSetWindowAttribute =
+                (P_DwmSetWindowAttribute) GetProcAddress(hDwm, "DwmSetWindowAttribute");
+
+            if (pDwmSetWindowAttribute)
+            {
+                const DWORD DWMWA_SYSTEMBACKDROP_TYPE = 38;
+                const DWORD DWMSBT_NONE = 1;
+                const DWORD DWMSBT_TRANSIENTWINDOW = 3; // Acrylic
+                const DWORD DWMWA_USE_IMMERSIVE_DARK_MODE = 20;
+
+                BOOL dark = TRUE;
+                pDwmSetWindowAttribute(hwnd, DWMWA_USE_IMMERSIVE_DARK_MODE, &dark, sizeof(dark));
+
+                DWORD backdropType = enable ? DWMSBT_TRANSIENTWINDOW : DWMSBT_NONE;
+                HRESULT hr = pDwmSetWindowAttribute(
+                    hwnd, DWMWA_SYSTEMBACKDROP_TYPE, &backdropType, sizeof(backdropType));
+                if (SUCCEEDED(hr))
+                {
+                    FreeLibrary(hDwm);
+                    return;
+                }
+            }
+            FreeLibrary(hDwm);
+        }
+
+        // 2. Fallback to undocumented SetWindowCompositionAttribute (Windows 10)
         const HINSTANCE hModule = LoadLibrary(TEXT("user32.dll"));
         if (hModule)
         {
@@ -174,9 +279,9 @@ void setEnabled(QWindow* window, bool enable, QRegion const& region)
             struct ACCENTPOLICY
             {
                 AcceptState nAccentState;
-                int nFlags;
-                int nColor;
-                int nAnimationId;
+                UINT nFlags;
+                COLORREF nColor;
+                LONG nAnimationId;
             };
             struct WINCOMPATTRDATA
             {
@@ -189,8 +294,10 @@ void setEnabled(QWindow* window, bool enable, QRegion const& region)
                 (pSetWindowCompositionAttribute) GetProcAddress(hModule, "SetWindowCompositionAttribute");
             if (SetWindowCompositionAttribute)
             {
-                auto const policy = enable ? ACCENTPOLICY { ACCENT_ENABLE_BLURBEHIND, 0, 0, 0 }
-                                           : ACCENTPOLICY { ACCENT_DISABLED, 0, 0, 0 };
+                // Use a dark semi-transparent tint (0x99000000) for Acrylic to be visible.
+                auto const policy = enable
+                                        ? ACCENTPOLICY { ACCENT_ENABLE_ACRYLICBLURBEHIND, 0, 0x99000000, 0 }
+                                        : ACCENTPOLICY { ACCENT_DISABLED, 0, 0, 0 };
                 auto const data = WINCOMPATTRDATA { WCA_ACCENT_POLICY, &policy, sizeof(ACCENTPOLICY) };
                 BOOL rs = SetWindowCompositionAttribute(hwnd, &data);
                 if (!rs)
@@ -199,11 +306,10 @@ void setEnabled(QWindow* window, bool enable, QRegion const& region)
             FreeLibrary(hModule);
         }
     }
-    // }}}
-#else
+#endif // }}}
+
     // Get me working on other platforms/compositors (such as OSX, Gnome, ...), please.
     crispy::ignore_unused(window, enable);
-#endif
 }
 
 } // namespace BlurBehind
