@@ -108,9 +108,9 @@ namespace
                     "Could not create directory {}. {}", path.parent_path().string(), ec.message()) };
     }
 
-    vector<fs::path> getTermInfoDirs(optional<fs::path> const& appTerminfoDir)
+    std::vector<fs::path> getTermInfoDirs(optional<fs::path> const& appTerminfoDir)
     {
-        auto locations = vector<fs::path>();
+        auto locations = std::vector<fs::path>();
 
         if (appTerminfoDir.has_value())
             locations.emplace_back(appTerminfoDir.value().string());
@@ -140,7 +140,7 @@ namespace
             return "contour";
 
         auto locations = getTermInfoDirs(appTerminfoDir);
-        auto const terms = vector<string> {
+        auto const terms = std::vector<string> {
             "contour", "xterm-256color", "xterm", "vt340", "vt220",
         };
 
@@ -360,6 +360,15 @@ void YAMLConfigReader::load(Config& c)
         loadFromEntry("on_mouse_select", c.onMouseSelection);
         loadFromEntry("mouse_block_selection_modifier", c.mouseBlockSelectionModifiers);
         loadFromEntry("profiles", c.profiles, c.defaultProfileName.value());
+        loadFromEntry("git_drawings", c.gitDrawings);
+#if defined(CONTOUR_FRONTEND_GUI)
+        vtrasterizer::BoxDrawingRenderer::setGitDrawingsStyle(c.gitDrawings.value());
+        loadFromEntry("box_arc_style", c.boxArcStyle);
+        vtrasterizer::BoxDrawingRenderer::setArcStyle(c.boxArcStyle.value());
+        loadFromEntry("braille_style", c.brailleStyle);
+        vtrasterizer::BoxDrawingRenderer::setBrailleStyle(c.brailleStyle.value());
+#endif
+
         // loadFromEntry("color_schemes", c.colorschemes); // NB: This is always loaded lazily
         loadFromEntry("input_mapping", c.inputMappings);
     }
@@ -398,6 +407,7 @@ void YAMLConfigReader::loadFromEntry(YAML::Node const& node, std::string const&
         loadFromEntry(child, "insert_after_yank", where.insertAfterYank);
         loadFromEntry(child, "bell", where.bell);
         loadFromEntry(child, "wm_class", where.wmClass);
+        loadFromEntry(child, "option_as_alt", where.optionKeyAsAlt);
         loadFromEntry(child, "margins", where.margins);
         loadFromEntry(child, "terminal_id", where.terminalId);
         loadFromEntry(child, "frozen_dec_modes", where.frozenModes);
@@ -407,6 +417,7 @@ void YAMLConfigReader::loadFromEntry(YAML::Node const& node, std::string const&
         loadFromEntry(child, "scrollbar", where.scrollbar);
         loadFromEntry(child, "mouse", where.mouse);
         loadFromEntry(child, "permissions", where.permissions);
+        loadFromEntry(child, "input_method_editor", where.inputMethodEditor);
         loadFromEntry(child, "highlight_word_and_matches_on_double_click", where.highlightDoubleClickedWord);
         loadFromEntry(child, "font", where.fonts);
         loadFromEntry(child, "draw_bold_text_with_bright_colors", where.drawBoldTextWithBrightColors);
@@ -470,7 +481,6 @@ void YAMLConfigReader::loadFromEntry(YAML::Node const& node,
     if (vtbackend::defaultColorPalettes(entry, where))
     {
         logger()("Loaded predefined color palette {}", entry);
-        return;
     }
 
     if (!child) // can not load directly from config file
@@ -550,11 +560,13 @@ void YAMLConfigReader::loadFromEntry(YAML::Node const& node, vtbackend::ColorPal
             vtbackend::RGBColorPair defaultIndicatorStatusLine;
             loadFromEntry(child["indicator_statusline"], "default", defaultIndicatorStatusLine);
             where.indicatorStatusLineInactive = defaultIndicatorStatusLine;
+            where.indicatorStatusLineInsertMode = defaultIndicatorStatusLine;
             where.indicatorStatusLineNormalMode = defaultIndicatorStatusLine;
             where.indicatorStatusLineVisualMode = defaultIndicatorStatusLine;
         }
         for (auto const& [name, defaultColor]:
              { pair { "inactive", &where.indicatorStatusLineInactive },
+               pair { "insert_mode", &where.indicatorStatusLineInsertMode },
                pair { "normal_mode", &where.indicatorStatusLineNormalMode },
                pair { "visual_mode", &where.indicatorStatusLineVisualMode } })
         {
@@ -825,7 +837,7 @@ void YAMLConfigReader::loadFromEntry(YAML::Node const& node,
         loadFromEntry(child, "user", where.username);
         loadFromEntry(child, "private_key", where.privateKeyFile);
         loadFromEntry(child, "public_key", where.publicKeyFile);
-        loadFromEntry(child, "known_hosts", where.publicKeyFile);
+        loadFromEntry(child, "known_hosts", where.knownHostsFile);
         loadFromEntry(child, "forward_agent", where.forwardAgent);
     }
 }
@@ -1461,6 +1473,107 @@ void YAMLConfigReader::loadFromEntry(YAML::Node const& node, std::string const&
         }
     }
 }
+void YAMLConfigReader::loadFromEntry(YAML::Node const& node,
+                                     std::string const& entry,
+                                     vtrasterizer::BoxDrawingRenderer::ArcStyle& where)
+{
+    using ArcStyle = vtrasterizer::BoxDrawingRenderer::ArcStyle;
+    if (auto const child = node[entry])
+    {
+        auto const styleName = crispy::toLower(child.as<std::string>());
+        auto constexpr static Mappings = std::array {
+            std::pair { "", ArcStyle::Round },
+            std::pair { "round", ArcStyle::Round },
+            std::pair { "elliptic", ArcStyle::Elliptic },
+        };
+        for (auto const& mapping: Mappings)
+            if (mapping.first == styleName)
+            {
+                logger()("Loading entry: arc_style, value {}", styleName);
+                where = mapping.second;
+                return;
+            }
+    }
+}
+void YAMLConfigReader::loadFromEntry(YAML::Node const& node,
+                                     std::string const& entry,
+                                     vtrasterizer::BoxDrawingRenderer::GitDrawingsStyle& where)
+{
+    where = {};
+    using BranchStyle = vtrasterizer::BoxDrawingRenderer::GitDrawingsStyle::BranchStyle;
+    using MergeCommitStyle = vtrasterizer::BoxDrawingRenderer::GitDrawingsStyle::MergeCommitStyle;
+
+    auto const parseBranchStyle = [&](std::string const& style) -> std::optional<BranchStyle> {
+        auto const styleName = crispy::toLower(style);
+        auto constexpr static Mappings = std::array {
+            std::pair { "", BranchStyle::None },         //
+            std::pair { "none", BranchStyle::None },     //
+            std::pair { "thin", BranchStyle::Thin },     //
+            std::pair { "double", BranchStyle::Double }, //
+            std::pair { "thick", BranchStyle::Thick },
+        };
+        for (auto const& mapping: Mappings)
+            if (mapping.first == styleName)
+            {
+                logger()("Loading entry: branch_style, value {}", styleName);
+                return mapping.second;
+            }
+        return std::nullopt;
+    };
+
+    auto const parseMCStyle = [&](std::string const& style) -> std::optional<MergeCommitStyle> {
+        auto const styleName = crispy::toLower(style);
+        auto constexpr static Mappings = std::array {
+            std::pair { "", MergeCommitStyle::Bullet },       //
+            std::pair { "bullet", MergeCommitStyle::Bullet }, //
+            std::pair { "solid", MergeCommitStyle::Solid },   //
+        };
+        for (auto const& mapping: Mappings)
+            if (mapping.first == styleName)
+            {
+                logger()("Loading entry: merge_commit_style, value {}", styleName);
+                return mapping.second;
+            }
+        return std::nullopt;
+    };
+
+    if (auto const child = node[entry])
+    {
+        loadFromEntry(child, "arc_style", where.arcStyle);
+        if (auto const branchStyle = parseBranchStyle(child["branch_style"].as<std::string>()))
+            where.branchStyle = *branchStyle;
+        if (auto const mcStyle = parseMCStyle(child["merge_commit_style"].as<std::string>()))
+            where.mergeCommitStyle = *mcStyle;
+    }
+}
+void YAMLConfigReader::loadFromEntry(YAML::Node const& node,
+                                     std::string const& entry,
+                                     vtrasterizer::BoxDrawingRenderer::BrailleStyle& where)
+{
+    using BrailleStyle = vtrasterizer::BoxDrawingRenderer::BrailleStyle;
+    if (auto const child = node[entry])
+    {
+        auto const styleName = crispy::toLower(child.as<std::string>());
+        auto constexpr static Mappings = std::array {
+            std::pair { "font", BrailleStyle::Font },                     //
+            std::pair { "solid", BrailleStyle::Solid },                   //
+            std::pair { "", BrailleStyle::Circle },                       //
+            std::pair { "circle", BrailleStyle::Circle },                 //
+            std::pair { "circle_empty", BrailleStyle::CircleEmpty },      //
+            std::pair { "square", BrailleStyle::SquareEmpty },            //
+            std::pair { "square_empty", BrailleStyle::SquareEmpty },      //
+            std::pair { "aa_square", BrailleStyle::AASquare },            //
+            std::pair { "aa_square_empty", BrailleStyle::AASquareEmpty }, //
+        };
+        for (auto const& mapping: Mappings)
+            if (mapping.first == styleName)
+            {
+                logger()("Loading entry: {}, value {}", entry, styleName);
+                where = mapping.second;
+                return;
+            }
+    }
+}
 
 bool YAMLConfigReader::tryAddMouse(std::vector<MouseInputMapping>& bindings,
                                    vtbackend::MatchModes modes,
@@ -1827,6 +1940,14 @@ std::optional<actions::Action> YAMLConfigReader::parseAction(YAML::Node const& n
                 return std::nullopt;
         }
 
+        if (holds_alternative<actions::MoveTabTo>(action))
+        {
+            if (auto position = node["position"]; position.IsScalar())
+                return actions::MoveTabTo { position.as<int>() };
+            else
+                return std::nullopt;
+        }
+
         if (holds_alternative<actions::SwitchToTab>(action))
         {
             if (auto position = node["position"]; position.IsScalar())
@@ -2141,7 +2262,7 @@ std::string createForProfile(Config const& c)
                 contour::config::documentation::DocumentationEntry<ConfigDoc, WebDoc>> const& vEntry) {
             auto v = vEntry.value();
             auto args = std::string { "[" };
-            args.append(v.arguments | ::ranges::views::join(", ") | ::ranges::to<std::string>);
+            args.append(v.arguments | crispy::views::join_with(", "sv));
             args.append("]");
 
             doc.append(writer.process(writer.whichDoc(vEntry), name, v.program, args, [&]() -> std::string {
