@@ -3,7 +3,10 @@
 
 #include <contour/Actions.h>
 #include <contour/ConfigDocumentation.h>
-#include <contour/display/ShaderConfig.h>
+
+#if defined(CONTOUR_FRONTEND_GUI)
+    #include <contour/display/ShaderConfig.h>
+#endif
 
 #include <vtbackend/Color.h>
 #include <vtbackend/ColorPalette.h>
@@ -20,6 +23,7 @@
 #include <vtpty/Process.h>
 #include <vtpty/SshSession.h>
 
+#include <vtrasterizer/BoxDrawingRenderer.h>
 #include <vtrasterizer/Decorator.h>
 #include <vtrasterizer/FontDescriptions.h>
 
@@ -38,11 +42,6 @@
 #include <yaml-cpp/ostream_wrapper.h>
 #include <yaml-cpp/yaml.h>
 
-#include <range/v3/range/conversion.hpp>
-#include <range/v3/view/concat.hpp>
-#include <range/v3/view/join.hpp>
-#include <range/v3/view/transform.hpp>
-
 #include <chrono>
 #include <cstdint>
 #include <exception>
@@ -51,6 +50,7 @@
 #include <limits>
 #include <memory>
 #include <optional>
+#include <ranges>
 #include <regex>
 #include <set>
 #include <string>
@@ -180,11 +180,11 @@ struct MouseConfig
 struct IndicatorConfig
 {
     std::string left { " {InputMode:Bold,Color=#FFFF00}"
-                       "{Tabs:ActiveColor=#FFFF00,Left= │ }"
-                       "{SearchPrompt:Left= │ }"
                        "{TraceMode:Bold,Color=#FFFF00,Left= │ }"
-                       "{ProtectedMode:Bold,Left= │ }" };
-    std::string middle { "{Title:Left= « ,Right= » }" };
+                       "{Tabs:ActiveColor=#FFFF00,Left= │ }"
+                       "{ProtectedMode:Bold,Left= │ }"
+                       "{SearchPrompt:Left= │ }" };
+    std::string middle { "« {Title} »" };
     std::string right { "{HistoryLineCount:Faint,Color=#c0c0c0} │ {Clock:Bold}" };
 };
 
@@ -387,14 +387,12 @@ struct TerminalProfile
         .program =
             []() {
                 auto const program = vtpty::Process::loginShell(true);
-                return program                    // NB: sadly can't be a temporary
-                       | ranges::views::join(' ') //
-                       | ranges::to<std::string>();
+                return program | crispy::views::join_with(std::string_view(" "));
             }(),
         .arguments = {},
         .workingDirectory = "",
         .env = {},
-    } };
+    } }; // namespace contour::config
     ConfigEntry<vtpty::SshHostConfig, documentation::SshHostConfig> ssh {};
     ConfigEntry<bool, documentation::EscapeSandbox> escapeSandbox { true };
     ConfigEntry<vtbackend::LineOffset, documentation::CopyLastMarkRangeOffset> copyLastMarkRangeOffset { 0 };
@@ -409,29 +407,32 @@ struct TerminalProfile
     ConfigEntry<std::map<vtbackend::DECMode, bool>, documentation::FrozenDecMode> frozenModes {};
     ConfigEntry<std::chrono::milliseconds, documentation::SmoothLineScrolling> smoothLineScrolling { 100 };
     ConfigEntry<vtbackend::PageSize, documentation::TerminalSize> terminalSize { {
-        vtbackend::LineCount(25),
-        vtbackend::ColumnCount(80),
+        .lines = vtbackend::LineCount(25),
+        .columns = vtbackend::ColumnCount(80),
     } };
-    ConfigEntry<WindowMargins, documentation::Margins> margins { { HorizontalMargin { 0u },
-                                                                   VerticalMargin { 0u } } };
+    ConfigEntry<WindowMargins, documentation::Margins> margins { { .horizontal = HorizontalMargin { 0u },
+                                                                   .vertical = VerticalMargin { 0u } } };
     ConfigEntry<HistoryConfig, documentation::History> history {};
     ConfigEntry<ScrollBarConfig, documentation::Scrollbar> scrollbar {};
     ConfigEntry<MouseConfig, documentation::Mouse> mouse { true };
     ConfigEntry<PermissionsConfig, documentation::Permissions> permissions {};
+    ConfigEntry<bool, documentation::InputMethodEditorSupport> inputMethodEditor { true };
     ConfigEntry<bool, documentation::HighlightDoubleClickerWord> highlightDoubleClickedWord { true };
     ConfigEntry<vtrasterizer::FontDescriptions, documentation::Fonts> fonts { defaultFont };
     ConfigEntry<bool, documentation::DrawBoldTextWithBrightColors> drawBoldTextWithBrightColors { false };
     ConfigEntry<InputModeConfig, documentation::ModeInsert> modeInsert { CursorConfig {
-        vtbackend::CursorShape::Bar, vtbackend::CursorDisplay::Steady, std::chrono::milliseconds { 500 } } };
+        .cursorShape = vtbackend::CursorShape::Bar,
+        .cursorDisplay = vtbackend::CursorDisplay::Steady,
+        .cursorBlinkInterval = std::chrono::milliseconds { 500 } } };
     ConfigEntry<InputModeConfig, documentation::ModeNormal> modeNormal {
-        CursorConfig { vtbackend::CursorShape::Block,
-                       vtbackend::CursorDisplay::Steady,
-                       std::chrono::milliseconds { 500 } },
+        CursorConfig { .cursorShape = vtbackend::CursorShape::Block,
+                       .cursorDisplay = vtbackend::CursorDisplay::Steady,
+                       .cursorBlinkInterval = std::chrono::milliseconds { 500 } },
     };
     ConfigEntry<InputModeConfig, documentation::ModeVisual> modeVisual {
-        CursorConfig { vtbackend::CursorShape::Block,
-                       vtbackend::CursorDisplay::Steady,
-                       std::chrono::milliseconds { 500 } },
+        CursorConfig { .cursorShape = vtbackend::CursorShape::Block,
+                       .cursorDisplay = vtbackend::CursorDisplay::Steady,
+                       .cursorBlinkInterval = std::chrono::milliseconds { 500 } },
     };
     ConfigEntry<std::chrono::milliseconds, documentation::HighlightTimeout> highlightTimeout { 100 };
     ConfigEntry<vtbackend::LineCount, documentation::ModalCursorScrollOff> modalCursorScrollOff {
@@ -452,7 +453,12 @@ const InputMappings defaultInputMappings {
                           .modifiers { vtbackend::Modifiers { vtbackend::Modifier::Alt } },
                           .input = vtbackend::Key::Enter,
                           .binding = { { actions::ToggleFullscreen {} } } },
-        KeyInputMapping { .modes {},
+        KeyInputMapping { .modes = []() -> vtbackend::MatchModes {
+                             auto mods = vtbackend::MatchModes();
+                             mods.enable(vtbackend::MatchModes::Select);
+                             mods.enable(vtbackend::MatchModes::Insert);
+                             return mods;
+                         }(),
                           .modifiers { vtbackend::Modifiers {} },
                           .input = vtbackend::Key::Escape,
                           .binding = { { actions::CancelSelection {} } } },
@@ -506,6 +512,12 @@ const InputMappings defaultInputMappings {
         //                       .binding = { { actions::IncreaseFontSize {} } } },
     },
     .charMappings {
+        CharInputMapping {
+            .modes { vtbackend::MatchModes {} },
+            .modifiers { vtbackend::Modifiers { vtbackend::Modifiers { vtbackend::Modifier::Alt }
+                                                | vtbackend::Modifiers { vtbackend::Modifier::Control } } },
+            .input = ',',
+            .binding = { { actions::ToggleInputMethodHandling {} } } },
         CharInputMapping {
             .modes { vtbackend::MatchModes {} },
             .modifiers { vtbackend::Modifiers { vtbackend::Modifiers { vtbackend::Modifier::Shift }
@@ -773,6 +785,14 @@ struct Config
         colorschemes { { { "default", vtbackend::ColorPalette {} } } };
 
     ConfigEntry<InputMappings, documentation::InputMappings> inputMappings { defaultInputMappings };
+    ConfigEntry<vtrasterizer::BoxDrawingRenderer::GitDrawingsStyle, documentation::GitDrawings>
+        gitDrawings {};
+    ConfigEntry<vtrasterizer::BoxDrawingRenderer::ArcStyle, documentation::BoxArcStyle> boxArcStyle {
+        vtrasterizer::BoxDrawingRenderer::ArcStyle::Round
+    };
+    ConfigEntry<vtrasterizer::BoxDrawingRenderer::BrailleStyle, documentation::BrailleStyle> brailleStyle {
+        vtrasterizer::BoxDrawingRenderer::BrailleStyle::Circle
+    };
 
     TerminalProfile* profile(std::string const& name) noexcept
     {
@@ -994,6 +1014,9 @@ struct YAMLConfigReader
     void loadFromEntry(YAML::Node const& node, std::string const& entry, BackgroundConfig& where);
     void loadFromEntry(YAML::Node const& node, std::string const& entry, HyperlinkDecorationConfig& where);
     void loadFromEntry(YAML::Node const& node, std::string const& entry, PermissionsConfig& where);
+    void loadFromEntry(YAML::Node const& node, std::string const& entry, vtrasterizer::BoxDrawingRenderer::ArcStyle& where);
+    void loadFromEntry(YAML::Node const& node, std::string const& entry, vtrasterizer::BoxDrawingRenderer::GitDrawingsStyle& where);
+    void loadFromEntry(YAML::Node const& node, std::string const& entry, vtrasterizer::BoxDrawingRenderer::BrailleStyle& where);
 
 
     void defaultSettings(vtpty::Process::ExecInfo& shell);
@@ -1049,10 +1072,15 @@ struct Writer
 
     [[nodiscard]] std::string format(KeyInputMapping v)
     {
+        auto actionAndModes = format(" action: {} }}", v.binding[0]);
+        if (v.modes.any())
+        {
+            actionAndModes = format(" action: {}, mode: '{}' }}", v.binding[0], v.modes);
+        }
         return format("{:<30},{:<30},{:<30}\n",
                       format("- {{ mods: [{}]", format(v.modifiers)),
                       format(" key: '{}'", v.input),
-                      format(" action: {} }}", v.binding[0]));
+                      actionAndModes);
     }
 
     [[nodiscard]] std::string format(CharInputMapping v)
@@ -1081,8 +1109,8 @@ struct Writer
     {
 
         auto result = std::string { "[" };
-        result.append(v | ranges::views::transform([](auto f) { return std::format("{}", f); })
-                      | ranges::views::join(", ") | ranges::to<std::string>);
+        result.append(v | std::views::transform([](auto f) { return std::format("{}", f); })
+                      | crispy::views::join_with(std::string_view(", ")));
         result.append("]");
         return result;
     }
@@ -1134,7 +1162,7 @@ struct Writer
     [[nodiscard]] std::string format(std::string_view doc, vtpty::Process::ExecInfo const& v)
     {
         auto args = std::string { "[" };
-        args.append(v.arguments | ranges::views::join(", ") | ranges::to<std::string>);
+        args.append(v.arguments | crispy::views::join_with(std::string_view(", ")));
         args.append("]");
         return format(doc, v.program, args, [&]() -> std::string {
             auto fromConfig = v.workingDirectory.string();
@@ -1294,6 +1322,68 @@ struct Writer
         auto const blinkingInterval = v.cursor.cursorBlinkInterval.count();
         return format(doc, shape, blinking, blinkingInterval);
     }
+
+    [[nodiscard]] std::string format(std::string_view doc, vtrasterizer::BoxDrawingRenderer::ArcStyle& v)
+    {
+        using ArcStyle = vtrasterizer::BoxDrawingRenderer::ArcStyle;
+        switch (v)
+        {
+            case ArcStyle::Elliptic: return format(doc, "elliptic");
+            case ArcStyle::Round: return format(doc, "round");
+            default: assert(false); return format(doc, "round");
+        }
+    }
+
+    [[nodiscard]] std::string format(std::string_view doc, vtrasterizer::BoxDrawingRenderer::BrailleStyle& v)
+    {
+        using BrailleStyle = vtrasterizer::BoxDrawingRenderer::BrailleStyle;
+        switch (v)
+        {
+            case BrailleStyle::Font: return format(doc, "font");
+            case BrailleStyle::Solid: return format(doc, "solid");
+            case BrailleStyle::Circle: return format(doc, "circle");
+            case BrailleStyle::CircleEmpty: return format(doc, "circle_empty");
+            case BrailleStyle::Square: return format(doc, "square");
+            case BrailleStyle::SquareEmpty: return format(doc, "square_empty");
+            case BrailleStyle::AASquare: return format(doc, "aa_square");
+            case BrailleStyle::AASquareEmpty: return format(doc, "aa_square_empty");
+            default: assert(false); return format(doc, "circle");
+        }
+    }
+
+    [[nodiscard]] std::string format(std::string_view doc,
+                                     vtrasterizer::BoxDrawingRenderer::GitDrawingsStyle& v)
+    {
+        using ArcStyle = vtrasterizer::BoxDrawingRenderer::ArcStyle;
+        using DrawingStyle = vtrasterizer::BoxDrawingRenderer::GitDrawingsStyle;
+        auto const arc = [&]() -> std::string_view {
+            switch (v.arcStyle)
+            {
+                case ArcStyle::Elliptic: return "elliptic";
+                case ArcStyle::Round: return "round";
+                default: assert(false); return "round";
+            }
+        }();
+        auto const branch = [&]() -> std::string_view {
+            switch (v.branchStyle)
+            {
+                case DrawingStyle::BranchStyle::None: return "none";
+                case DrawingStyle::BranchStyle::Thin: return "thin";
+                case DrawingStyle::BranchStyle::Double: return "double";
+                case DrawingStyle::BranchStyle::Thick: return "thick";
+                default: assert(false); return "thin";
+            }
+        }();
+        auto const mergeCommit = [&]() -> std::string_view {
+            switch (v.mergeCommitStyle)
+            {
+                case DrawingStyle::MergeCommitStyle::Solid: return "solid";
+                case DrawingStyle::MergeCommitStyle::Bullet: return "bullet";
+                default: assert(false); return "solid";
+            }
+        }();
+        return format(doc, branch, arc, mergeCommit);
+    }
 };
 
 template <typename T>
@@ -1481,11 +1571,11 @@ struct std::formatter<vtbackend::Opacity>: formatter<float>
 };
 
 template <>
-struct std::formatter<crispy::strong_hashtable_size>: formatter<uint>
+struct std::formatter<crispy::strong_hashtable_size>: formatter<unsigned>
 {
     auto format(crispy::strong_hashtable_size value, auto& ctx) const
     {
-        return formatter<uint>::format(value.value, ctx);
+        return formatter<unsigned>::format(value.value, ctx);
     }
 };
 
@@ -1532,11 +1622,11 @@ struct std::formatter<vtbackend::StatusDisplayType>: formatter<std::string_view>
 };
 
 template <>
-struct std::formatter<crispy::lru_capacity>: formatter<uint>
+struct std::formatter<crispy::lru_capacity>: formatter<unsigned>
 {
     auto format(crispy::lru_capacity value, auto& ctx) const
     {
-        return formatter<uint>::format(value.value, ctx);
+        return formatter<unsigned>::format(value.value, ctx);
     }
 };
 
@@ -1546,7 +1636,7 @@ struct std::formatter<std::set<std::basic_string<char>>>: formatter<std::string_
     auto format(std::set<std::basic_string<char>> value, auto& ctx) const
     {
         auto result = std::string {};
-        result.append(value | ::ranges::views::join(", ") | ::ranges::to<std::string>);
+        result.append(value | crispy::views::join_with(std::string_view(", ")));
         return formatter<std::string_view>::format(result, ctx);
     }
 };
