@@ -24,10 +24,10 @@ struct DocumentationEntry
 {
 };
 
-constexpr StringLiteral Dummy { "{comment} fmt formatted doc {} \n" };
+constexpr StringLiteral Dummy { "{comment} fmt formatted doc {}\n" };
 
 constexpr StringLiteral ShellConfig {
-    "{comment} You can override the process to be started inside the terminal."
+    "{comment} You can override the process to be started inside the terminal.\n"
     "{comment} If nothing is specified, the users' default login shell will be used.\n"
     "{comment} But you may as well log in to a remote host.\n"
     "shell: {}\n"
@@ -142,9 +142,9 @@ constexpr StringLiteral InsertAfterYankConfig {
 };
 
 constexpr StringLiteral CopyLastMarkRangeOffsetConfig {
-    "{comment} Advanced value that is useful when CopyPreviousMarkRange is used \n"
-    "{comment} with multiline-prompts. This offset value is being added to the \n"
-    "{comment} current cursor's line number minus 1 (i.e. the line above the current cursor). \n"
+    "{comment} Advanced value that is useful when CopyPreviousMarkRange is used\n"
+    "{comment} with multiline-prompts. This offset value is being added to the\n"
+    "{comment} current cursor's line number minus 1 (i.e. the line above the current cursor).\n"
     "copy_last_mark_range_offset: {}\n"
     "\n"
 };
@@ -168,8 +168,8 @@ constexpr StringLiteral MarginsConfig {
 
 constexpr StringLiteral TerminalSizeConfig { "{comment}Determines the initial terminal size in  characters\n"
                                              "terminal_size:\n"
-                                             "    columns: {} \n"
-                                             "    lines: {} \n"
+                                             "    columns: {}\n"
+                                             "    lines: {}\n"
                                              "\n" };
 
 constexpr StringLiteral TerminalIdConfig {
@@ -325,7 +325,7 @@ constexpr StringLiteral PermissionsConfig {
 
     "\n"
     "{comment} Some VT sequences should need access permissions.\n"
-    "{comment} \n"
+    "{comment}\n"
     "{comment}  These can be to:\n"
     "{comment}  - allow     Allows the given functionality\n"
     "{comment}  - deny      Denies the given functionality\n"
@@ -421,6 +421,13 @@ constexpr StringLiteral HighlightTimeoutConfig {
     "\n"
 };
 
+constexpr StringLiteral InputMethodEditorSupportConfig {
+    "{comment} Enables Input Method Editor (IME) support.\n"
+    "{comment} This is enabled by default.\n"
+    "input_method_editor: {}\n"
+    "\n"
+};
+
 constexpr StringLiteral HighlightDoubleClickerWordConfig {
     "{comment} If enabled, and you double-click on a word in the primary screen,\n"
     "{comment} all other words matching this word will be highlighted as well.\n"
@@ -476,7 +483,6 @@ constexpr StringLiteral BellConfig {
     "    sound: {}\n"
     "\n"
     "    {comment} Bell volume, a normalized value between 0.0 (silent) and 1.0 (loudest).\n"
-    "    {comment} Default: 1.0\n"
     "    volume: {}\n"
     "\n"
     "    {comment} If this boolean is true, a window alert will "
@@ -501,82 +507,81 @@ constexpr StringLiteral FrozenDecModeConfig {
     "{comment}         2026: false\n"
     "{comment}         2027: true\n"
     "{comment}\n"
-    "{comment} Default: (empty object)\n"
     "{comment}frozen_dec_modes:\n"
     "\n"
 };
 
 constexpr StringLiteral LiveConfig {
     "{comment} Determines whether the instance is reloading the configuration files "
-    "whenever it is changing or not. \n"
-    "live_config: {} \n"
+    "whenever it is changing or not.\n"
+    "live_config: {}\n"
     "\n"
 };
 
 constexpr StringLiteral PlatformPluginConfig {
-    "{comment} Overrides the auto-detected platform plugin to be loaded. \n"
-    "{comment} \n"
+    "{comment} Overrides the auto-detected platform plugin to be loaded.\n"
+    "{comment}\n"
     "{comment} Possible (incomplete list of) values are:\n"
     "{comment} - auto        The platform will be auto-detected.\n"
     "{comment} - xcb         Uses XCB plugin (for X11 environment).\n"
     "{comment} - cocoa       Used to be run on Mac OS/X.\n"
     "{comment} - direct2d    Windows platform plugin using Direct2D.\n"
     "{comment} - winrt       Windows platform plugin using WinRT.\n"
-    "platform_plugin: {} \n"
+    "platform_plugin: {}\n"
     "\n"
 };
 
 constexpr StringLiteral RendererConfig {
     "renderer:\n"
-    "    {comment} Backend to use for rendering the terminal onto the screen \n"
-    "    {comment} Possible values are: \n"
-    "    {comment} - default     Uses the default rendering option as decided by the terminal. \n"
-    "    {comment} - software    Uses software-based rendering. \n"
-    "    {comment} - OpenGL      Use (possibly) hardware accelerated OpenGL \n"
-    "    backend: {} \n"
+    "    {comment} Backend to use for rendering the terminal onto the screen\n"
+    "    {comment} Possible values are:\n"
+    "    {comment} - default     Uses the default rendering option as decided by the terminal.\n"
+    "    {comment} - software    Uses software-based rendering.\n"
+    "    {comment} - OpenGL      Use (possibly) hardware accelerated OpenGL\n"
+    "    backend: {}\n"
     "\n"
-    "    {comment} Enables/disables the use of direct-mapped texture atlas tiles for \n"
-    "    {comment} the most often used ones (US-ASCII, cursor shapes, underline styles) \n"
-    "    {comment} You most likely do not want to touch this. \n"
-    "    {comment} \n"
-    "    tile_direct_mapping: {} \n"
+    "    {comment} Enables/disables the use of direct-mapped texture atlas tiles for\n"
+    "    {comment} the most often used ones (US-ASCII, cursor shapes, underline styles)\n"
+    "    {comment} You most likely do not want to touch this.\n"
+    "    {comment}\n"
+    "    tile_direct_mapping: {}\n"
     "\n"
-    "    {comment} Number of hashtable slots to map to the texture tiles. \n"
-    "    {comment} Larger values may increase performance, but too large may also decrease. \n"
-    "    {comment} This value is rounded up to a value equal to the power of two. \n"
-    "    {comment} \n"
-    "    tile_hashtable_slots: {} \n"
+    "    {comment} Number of hashtable slots to map to the texture tiles.\n"
+    "    {comment} Larger values may increase performance, but too large may also decrease.\n"
+    "    {comment} This value is rounded up to a value equal to the power of two.\n"
+    "    {comment}\n"
+    "    tile_hashtable_slots: {}\n"
     "\n"
-    "    {comment} Number of tiles that must fit at lest into the texture atlas. \n"
-    "    {comment} \n"
-    "    {comment} This does not include direct mapped tiles (US-ASCII glyphs, \n"
-    "    {comment} cursor shapes and decorations), if tile_direct_mapping is set to true). \n"
-    "    {comment} \n"
-    "    {comment} Value must be at least as large as grid cells available in the terminal view. \n"
-    "    {comment} This value is automatically adjusted if too small. \n"
-    "    {comment} \n"
-    "    tile_cache_count: {} \n"
+    "    {comment} Number of tiles that must fit at lest into the texture atlas.\n"
+    "    {comment}\n"
+    "    {comment} This does not include direct mapped tiles (US-ASCII glyphs,\n"
+    "    {comment} cursor shapes and decorations), if tile_direct_mapping is set to true).\n"
+    "    {comment}\n"
+    "    {comment} Value must be at least as large as grid cells available in the terminal view.\n"
+    "    {comment} This value is automatically adjusted if too small.\n"
+    "    {comment}\n"
+    "    tile_cache_count: {}\n"
     "\n"
 };
 
-constexpr StringLiteral PTYReadBufferSizeConfig { "{comment} Default PTY read buffer size. \n"
-                                                  "{comment} \n"
-                                                  "{comment} This is an advance option. Use with care! \n"
-                                                  "read_buffer_size: {} \n"
+constexpr StringLiteral PTYReadBufferSizeConfig { "{comment} Default PTY read buffer size.\n"
+                                                  "{comment}\n"
+                                                  "{comment} This is an advance option. Use with care!\n"
+                                                  "read_buffer_size: {}\n"
                                                   "\n" };
 
 constexpr StringLiteral PTYBufferObjectSizeConfig {
-    "{comment} Size in bytes per PTY Buffer Object. \n "
-    "{comment} \n"
-    "{comment} This is an advanced option of an internal storage. Only change with care! \n"
-    "pty_buffer_size: {} \n"
+    "{comment} Size in bytes per PTY Buffer Object.\n"
+    "{comment}\n"
+    "{comment} This is an advanced option of an internal storage. Only change with care!\n"
+    "pty_buffer_size: {}\n"
     "\n"
 };
 
 constexpr StringLiteral ReflowOnResizeConfig {
     "\n"
-    "{comment} Whether or not to reflow the lines on terminal resize events. \n"
-    "reflow_on_resize: {} \n"
+    "{comment} Whether or not to reflow the lines on terminal resize events.\n"
+    "reflow_on_resize: {}\n"
 };
 
 constexpr StringLiteral ColorSchemesConfig {
@@ -603,56 +608,56 @@ constexpr StringLiteral ProfilesConfig {
     "\n"
 };
 
-constexpr StringLiteral WordDelimitersConfig { "{comment} Word delimiters when selecting word-wise. \n"
-                                               "word_delimiters: \"{}\" \n"
+constexpr StringLiteral WordDelimitersConfig { "{comment} Word delimiters when selecting word-wise.\n"
+                                               "word_delimiters: \"{}\"\n"
                                                "\n" };
 
 constexpr StringLiteral ExtendedWordDelimitersConfig {
-    "{comment} Word delimiters for second selection when selecting word-wise. \n"
-    "{comment} Setting allows you to set less strict boundaried between words, for example \n"
-    "{comment} if you want to select whole ip address during selection set delimieters to \" \" (space) \n"
-    "extended_word_delimiters: \"{}\" \n"
+    "{comment} Word delimiters for second selection when selecting word-wise.\n"
+    "{comment} Setting allows you to set less strict boundaried between words, for example\n"
+    "{comment} if you want to select whole ip address during selection set delimieters to \" \" (space)\n"
+    "extended_word_delimiters: \"{}\"\n"
     "\n"
 };
 
 constexpr StringLiteral BypassMouseProtocolModifiersConfig {
-    "{comment} This keyboard modifier can be used to bypass the terminal's mouse protocol, \n"
-    "{comment} which can be used to select screen content even if the an application \n"
-    "{comment} mouse protocol has been activated (Default: Shift). \n"
-    "{comment} \n"
-    "{comment} The same modifier values apply as with input modifiers (see below). \n"
-    "bypass_mouse_protocol_modifier: {} \n"
+    "{comment} This keyboard modifier can be used to bypass the terminal's mouse protocol,\n"
+    "{comment} which can be used to select screen content even if the an application\n"
+    "{comment} mouse protocol has been activated (Default: Shift).\n"
+    "{comment}\n"
+    "{comment} The same modifier values apply as with input modifiers (see below).\n"
+    "bypass_mouse_protocol_modifier: {}\n"
     "\n"
 };
 
 constexpr StringLiteral OnMouseSelectionConfig {
-    "{comment} Selects an action to perform when a text selection has been made. \n"
-    "{comment} \n"
-    "{comment} Possible values are: \n"
-    "{comment} \n"
-    "{comment} - None                        Does nothing \n"
-    "{comment} - CopyToClipboard             Copies the selection to the primary clipboard. \n"
-    "{comment} - CopyToSelectionClipboard    Copies the selection to the selection clipboard. \n"
-    "{comment}This is not supported on all platforms. \n"
-    "{comment} \n"
-    "on_mouse_select: {} \n"
+    "{comment} Selects an action to perform when a text selection has been made.\n"
+    "{comment}\n"
+    "{comment} Possible values are:\n"
+    "{comment}\n"
+    "{comment} - None                        Does nothing\n"
+    "{comment} - CopyToClipboard             Copies the selection to the primary clipboard.\n"
+    "{comment} - CopyToSelectionClipboard    Copies the selection to the selection clipboard.\n"
+    "{comment}This is not supported on all platforms.\n"
+    "{comment}\n"
+    "on_mouse_select: {}\n"
     "\n"
 };
 
 constexpr StringLiteral MouseBlockSelectionModifiersConfig {
-    "{comment} Modifier to be pressed in order to initiate block-selection \n"
-    "{comment} using the left mouse button. \n"
-    "{comment} \n"
-    "{comment} This is usually the Control modifier, but on OS/X that is not possible, \n"
-    "{comment} so Alt or Meta would be recommended instead. \n"
-    "{comment} \n"
-    "{comment} Supported modifiers: \n"
-    "{comment} - Alt \n"
-    "{comment} - Control \n"
-    "{comment} - Shift \n"
-    "{comment} - Meta \n"
-    "{comment} \n"
-    "mouse_block_selection_modifier: {} \n"
+    "{comment} Modifier to be pressed in order to initiate block-selection\n"
+    "{comment} using the left mouse button.\n"
+    "{comment}\n"
+    "{comment} This is usually the Control modifier, but on OS/X that is not possible,\n"
+    "{comment} so Alt or Meta would be recommended instead.\n"
+    "{comment}\n"
+    "{comment} Supported modifiers:\n"
+    "{comment} - Alt\n"
+    "{comment} - Control\n"
+    "{comment} - Shift\n"
+    "{comment} - Meta\n"
+    "{comment}\n"
+    "mouse_block_selection_modifier: {}\n"
     "\n"
 };
 
@@ -741,6 +746,8 @@ constexpr StringLiteral InputMappingsConfig {
     "position.\n"
     "{comment} - IncreaseFontSize  Increases the font size by 1 pixel.\n"
     "{comment} - IncreaseOpacity   Increases the default-background opacity by 5%.\n"
+    "{comment} - MoveTabToLeft     Moves the current tab to the left.\n"
+    "{comment} - MoveTabToRight    Moves the current tab to the right.\n"
     "{comment} - NewTerminal       Spawns a new terminal at the current terminals current working "
     "directory.\n"
     "{comment} - NoSearchHighlight Disables current search highlighting, if anything is still "
@@ -783,6 +790,7 @@ constexpr StringLiteral InputMappingsConfig {
     "{comment} - SendChars         Writes given characters in `chars` member to the applications input.\n"
     "{comment} - SwitchToTab       Switches to the tab position, given by extra parameter \"position\".\n"
     "{comment}                     The positions start at number 1.\n"
+    "{comment} - SwitchToPreviousTab Switches to the previously active tab.\n"
     "{comment} - SwitchToTabLeft   Switches to the tab left of the current tab.\n"
     "{comment} - SwitchToTabRight  Switches to the tab right of the current tab.\n"
     "{comment} - CreateNewTab      Creates a new tab.\n"
@@ -791,6 +799,7 @@ constexpr StringLiteral InputMappingsConfig {
     "preserved "
     "when disabling all others).\n"
     "{comment} - ToggleFullScreen  Enables/disables full screen mode.\n"
+    "{comment} - ToggleInputMethodHandling Enables/disables IME (input method editor) handling.\n"
     "{comment} - ToggleInputProtection Enables/disables terminal input protection.\n"
     "{comment} - ToggleStatusLine  Shows/hides the VT320 compatible Indicator status line.\n"
     "{comment} - ToggleTitleBar    Shows/Hides titlebar\n"
@@ -810,36 +819,38 @@ constexpr StringLiteral InputMappingsConfig {
     "p.\n"
     "{comment} - WriteScreen       Writes VT sequence in `chars` member to the screen (bypassing the "
     "application).\n"
+    "{comment} - SetTabName        Ask the user to assign a name to the active tab.\n"
+    "\n"
     "input_mapping:\n"
 };
 
 constexpr StringLiteral SpawnNewProcessConfig {
     "\n"
-    "{comment} Flag to determine whether to spawn new process or not when creating new terminal \n"
-    "spawn_new_process: {} \n"
+    "{comment} Flag to determine whether to spawn new process or not when creating new terminal\n"
+    "spawn_new_process: {}\n"
 };
 
 constexpr unsigned DefaultEarlyExitThreshold = 5u;
 constexpr StringLiteral EarlyExitThresholdConfig { "\n"
-                                                   "{comment} Time in seconds to check for early threshold \n"
-                                                   "early_exit_threshold: {} \n" };
+                                                   "{comment} Time in seconds to check for early threshold\n"
+                                                   "early_exit_threshold: {}\n" };
 
 constexpr StringLiteral ImagesConfig {
     "images:\n"
-    "    {comment} Enable or disable sixel scrolling (SM/RM ?80 default) \n"
-    "    sixel_scrolling: {} \n"
+    "    {comment} Enable or disable sixel scrolling (SM/RM ?80 default)\n"
+    "    sixel_scrolling: {}\n"
     "\n"
     "    {comment} Configures the maximum number of color registers available when rendering Sixel "
-    "graphics. \n"
-    "    sixel_register_count: {} \n"
+    "graphics.\n"
+    "    sixel_register_count: {}\n"
     "\n"
     "    {comment} maximum width in pixels of an image to be accepted (0 defaults to system screen pixel "
     "width) "
     "\n"
-    "    max_width: {} \n"
+    "    max_width: {}\n"
     "    {comment} maximum height in pixels of an image to be accepted (0 defaults to system screen pixel "
-    "height) \n"
-    "    max_height: {} \n"
+    "height)\n"
+    "    max_height: {}\n"
 };
 
 constexpr StringLiteral ExperimentalFeaturesConfig {
@@ -1044,6 +1055,43 @@ constexpr StringLiteral DimColorsConfig {
     "{comment}     white:   {}\n"
 };
 
+constexpr StringLiteral BoxArcStyleConfig {
+    "\n"
+    "{comment} Controls how arcs in box drawing characters are drawn. "
+    "May be on of `round`, `ellips`. The default value is `round`.\n"
+    "box_arc_style: {}\n"
+};
+
+constexpr StringLiteral BrailleStyleConfig {
+    "\n"
+    "{comment} Controls how Unicode Braille characters are rendered.\n "
+    "{comment} May be one of `font`, `solid`, `circle`, `circle_empty`, "
+    "`square`, `square_empty, `aa_square`, `aa_square_emtpy`.\n"
+    "{comment} The default value is `circle`.\n"
+    "{comment} `font` disables special rendering and uses font characters.\n"
+    "{comment} `solid` splits character box into 8 rectangles without space between them."
+    "No antialiasing is used, some font sizes may result in differently sized \"dots\".\n"
+    "{comment} `circle<..>` and `<..>square<..>` groups use circle and square \"dots\" respectively.\n"
+    "{comment} `<..>_empty` draw non-filled \"dots\".\n"
+    "{comment} `square<..>` does not use antialiasing, which may result in differently sized \"dots\". "
+    "`aa_square<..>` uses antialiasing.\n"
+    "braille_style: {}\n"
+};
+constexpr StringLiteral GitDrawingsConfig {
+    "\n"
+    "{comment} Controls how git branch drawings are rendered.\n"
+    "{comment} `branch_style` controls style of connecting branches. "
+    "May be on of `none`,`thin`,`thick`,`double. The default value is `thin`.\n"
+    "{comment} `arc_style` controls style of arcs. May be one of `round`,`ellips`. "
+    "`arc_style` has effect only if `branch_style` is `thin`. The default value is `round`.\n"
+    "{comment} `merge_commit_style` controls style of merge commits. May be one of `solid`,`bullet`. The "
+    "default value is `bullet`.\n"
+    "git_drawings: \n"
+    "    branch_style:       {}\n"
+    "    arc_style:          {}\n"
+    "    merge_commit_style: {}\n"
+};
+
 constexpr StringLiteral PlatformPluginWeb {
     "option allows you to override the auto-detected platform plugin to be loaded. You can specify values "
     "like `auto`, `xcb`, `cocoa`, `direct2d`, or `winrt` to determine the platform plugin. The default value "
@@ -1580,7 +1628,7 @@ constexpr StringLiteral BackgroundWeb {
 constexpr StringLiteral ColorsWeb {
 
     "section in the configuration file allows you to specify the colorscheme to use for the terminal. You "
-    "can use one of the predefined color palettes as a setting for colors entry. \n"
+    "can use one of the predefined color palettes as a setting for colors entry.\n"
     "List of predefined colorschemes: `contour`(default colors), `monokai`, `one-light`, `one-dark`, "
     "`gruvbox-light`, `gruvbox-dark`, `solarized-light`, `solarized-dark`, `papercolor-light`, "
     "`papercolor-dark`.\n"
@@ -1799,8 +1847,12 @@ using WordHighlightCurrent = DocumentationEntry<WordHighlightCurrentConfig, Dumm
 using WordHighlight = DocumentationEntry<WordHighlightConfig, Dummy>;
 using IndicatorStatusLine = DocumentationEntry<IndicatorStatusLineConfig, Dummy>;
 using InputMethodEditor = DocumentationEntry<InputMethodEditorConfig, Dummy>;
+using InputMethodEditorSupport = DocumentationEntry<InputMethodEditorSupportConfig, Dummy>;
 using NormalColors = DocumentationEntry<NormalColorsConfig, Dummy>;
 using BrightColors = DocumentationEntry<BrightColorsConfig, Dummy>;
 using DimColors = DocumentationEntry<DimColorsConfig, Dummy>;
+using GitDrawings = DocumentationEntry<GitDrawingsConfig, Dummy>;
+using BoxArcStyle = DocumentationEntry<BoxArcStyleConfig, Dummy>;
+using BrailleStyle = DocumentationEntry<BrailleStyleConfig, Dummy>;
 
 } // namespace contour::config::documentation
