@@ -315,7 +315,13 @@ bool ContourGuiApp::loadConfig(string const& target)
     }
 
     if (auto const wmClass = flags.get<string>("contour.terminal.class"); !wmClass.empty())
-        _config.profile(profileName())->wmClass = wmClass;
+    {
+        auto profile = _config.profile(profileName());
+        if (profile)
+        {
+            profile->wmClass = wmClass;
+        }
+    }
 
     return true;
 }
@@ -431,12 +437,10 @@ int ContourGuiApp::terminalGuiAction()
     });
 #endif
 
-#if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
     // Enforce OpenGL over any other. As much as I'd love to provide other backends, too.
     // We currently only support OpenGL.
     // If anyone feels happy about it, I'd love to at least provide Vulkan. ;-)
     QQuickWindow::setGraphicsApi(QSGRendererInterface::OpenGL);
-#endif
 
     QGuiApplication::setWindowIcon(QIcon(":/contour/logo-256.png"));
 
@@ -561,7 +565,6 @@ void ContourGuiApp::ensureTermInfoFile()
 void ContourGuiApp::newWindow()
 {
     _qmlEngine->load(resolveResource("ui/main.qml"));
-    _sessionManager.display = _sessionManager.getSession()->display();
 }
 
 void ContourGuiApp::showNotification(std::string_view title, std::string_view content)
