@@ -34,6 +34,7 @@
 
 #include <algorithm>
 #include <filesystem>
+#include <format>
 #include <fstream>
 #include <limits>
 
@@ -148,6 +149,43 @@ namespace
         settings.indicatorStatusLine.left = profile.statusLine.value().indicator.left;
         settings.indicatorStatusLine.middle = profile.statusLine.value().indicator.middle;
         settings.indicatorStatusLine.right = profile.statusLine.value().indicator.right;
+        settings.tabNamingMode = [&]() {
+            // try to find Tab section in one of the status line segments
+
+            std::string segment;
+            if (profile.statusLine.value().indicator.left.find("Tabs") != std::string::npos)
+            {
+                segment = profile.statusLine.value().indicator.left;
+            }
+            else if (profile.statusLine.value().indicator.middle.find("Tabs") != std::string::npos)
+            {
+                segment = profile.statusLine.value().indicator.middle;
+            }
+            else if (profile.statusLine.value().indicator.right.find("Tabs") != std::string::npos)
+            {
+                segment = profile.statusLine.value().indicator.right;
+            }
+
+            // check if indexing is defined
+            if (segment.find("Indexing=") != std::string::npos)
+            {
+                // cut the string after indexing=
+                std::string indexing = segment.substr(segment.find("Indexing=") + 9);
+                // cut right part of the string
+                indexing = indexing.substr(0, indexing.find(','));
+                indexing = indexing.substr(0, indexing.find('}'));
+
+                std::ranges::transform(
+                    indexing, indexing.begin(), [](unsigned char c) { return std::tolower(c); });
+
+                if (indexing == "title")
+                {
+                    return vtbackend::TabsNamingMode::Title;
+                }
+            }
+            return vtbackend::TabsNamingMode::Indexing;
+        }();
+
         settings.syncWindowTitleWithHostWritableStatusDisplay =
             profile.statusLine.value().syncWindowTitleWithHostWritableStatusDisplay;
         if (auto const* p = preferredColorPalette(profile.colors.value(), colorPreference))
@@ -185,7 +223,10 @@ namespace
 
 } // namespace
 
-TerminalSession::TerminalSession(unique_ptr<vtpty::Pty> pty, ContourGuiApp& app):
+TerminalSession::TerminalSession(TerminalSessionManager* manager,
+                                 unique_ptr<vtpty::Pty> pty,
+                                 ContourGuiApp& app):
+    _manager { manager },
     _id { createSessionId() },
     _startTime { steady_clock::now() },
     _config { app.config() },
@@ -193,8 +234,8 @@ TerminalSession::TerminalSession(unique_ptr<vtpty::Pty> pty, ContourGuiApp& app)
     _profile { *_config.profile(_profileName) },
     _app { app },
     _currentColorPreference { app.colorPreference() },
-    _accumulatedScrollX { 0 },
-    _accumulatedScrollY { 0 },
+    _accumulatedPixelScroll {},
+    _accumulatedAngleScroll {},
     _terminal { *this,
                 std::move(pty),
                 createSettingsFromConfig(_config, _profile, _currentColorPreference),
@@ -258,6 +299,7 @@ void TerminalSession::attachDisplay(display::TerminalDisplay& newDisplay)
 void TerminalSession::scheduleRedraw()
 {
     _terminal.markScreenDirty();
+    _manager->update();
     if (_display)
         _display->scheduleRedraw();
 }
@@ -564,8 +606,9 @@ void TerminalSession::inspect()
     // Deferred termination? Then close display now.
     if (_terminal.device().isClosed() && !_app.dumpStateAtExit().has_value())
     {
-        sessionLog()("Terminal device is closed. Closing display.");
-        _display->closeDisplay();
+        sessionLog()("Terminal device is closed. Notify session manager.");
+        _manager->currentSessionIsTerminated();
+        //_display->closeDisplay(); // TODO MOVE LOGIC
     }
 }
 
@@ -638,8 +681,8 @@ void TerminalSession::onClosed()
         inspect();
     else if (_display)
     {
-        sessionLog()("Terminal device is closed. Closing display.");
-        _display->closeDisplay();
+        sessionLog()("Terminal device is closed. Notify manager.");
+        _manager->currentSessionIsTerminated();
     }
     else
         sessionLog()("Terminal device is closed. But no display available (yet).");
@@ -761,6 +804,50 @@ void TerminalSession::requestWindowResize(Width width, Height height)
     _display->post([this, width, height]() { _display->resizeWindow(width, height); });
 }
 
+void TerminalSession::addToAccumulatedScroll(crispy::point pixelDelta, crispy::point angleDelta) noexcept
+{
+    if (angleDelta && !pixelDelta)
+        _accumulatedPixelScroll = {};
+    else
+        _accumulatedPixelScroll += pixelDelta;
+
+    _accumulatedAngleScroll += angleDelta;
+}
+
+std::tuple<LineOffset, ColumnOffset> TerminalSession::consumeScroll() noexcept
+{
+    if (_accumulatedPixelScroll)
+    {
+        auto const pixelStepSize = crispy::point {
+            .x = _display->cellSize().width.as<int>(),
+            .y = _display->cellSize().height.as<int>(),
+        };
+        auto const pixelSteps = _accumulatedPixelScroll / pixelStepSize;
+
+        if (pixelSteps)
+        {
+            _accumulatedPixelScroll -= pixelSteps * pixelStepSize;
+            _accumulatedAngleScroll = {};
+
+            return {
+                LineOffset::cast_from(pixelSteps.y),
+                ColumnOffset::cast_from(pixelSteps.x),
+            };
+        }
+    }
+
+    auto const angleStepSize = double { 8 * 5 };
+    auto const angleSteps = _accumulatedAngleScroll / angleStepSize;
+
+    _accumulatedAngleScroll -= angleSteps * angleStepSize;
+    _accumulatedPixelScroll = {};
+
+    return {
+        LineOffset::cast_from(angleSteps.y),
+        ColumnOffset::cast_from(angleSteps.x),
+    };
+}
+
 QString TerminalSession::title() const
 {
 #if !defined(NDEBUG)
@@ -810,6 +897,28 @@ void TerminalSession::onScrollOffsetChanged(vtbackend::ScrollOffset value)
 }
 // }}}
 // {{{ Input Events
+
+void handleAction(auto const& actions, auto eventType, auto callback)
+{
+    if (eventType == KeyboardEventType::Press)
+        callback(*actions);
+    else if (eventType == KeyboardEventType::Repeat)
+    {
+        // filter out actions that are not repeatable
+        std::vector<actions::Action> tmpActions;
+        auto set = crispy::overloaded {
+            [&]([[maybe_unused]] actions::NonRepeatableActionConcept auto const& action) {},
+            [&](auto const& action) { tmpActions.emplace_back(action); },
+        };
+
+        for (auto const& action: *actions)
+        {
+            std::visit(set, action);
+        }
+        callback(tmpActions);
+    }
+}
+
 void TerminalSession::sendKeyEvent(Key key, Modifiers modifiers, KeyboardEventType eventType, Timestamp now)
 {
     inputLog()("Key {} event received: {} {}", eventType, modifiers, key);
@@ -829,7 +938,7 @@ void TerminalSession::sendKeyEvent(Key key, Modifiers modifiers, KeyboardEventTy
         if (auto const* actions =
                 config::apply(_config.inputMappings.value().keyMappings, key, modifiers, matchModeFlags()))
         {
-            executeAllActions(*actions);
+            handleAction(actions, eventType, [&](auto const& actions) { executeAllActions(actions); });
             return;
         }
     }
@@ -864,7 +973,7 @@ void TerminalSession::sendCharEvent(
                 config::apply(_config.inputMappings.value().charMappings, value, modifiers, matchModeFlags());
             actions && !_terminal.inputHandler().isEditingSearch())
         {
-            executeAllActions(*actions);
+            handleAction(actions, eventType, [&](auto const& actions) { executeAllActions(actions); });
             return;
         }
     }
@@ -1357,6 +1466,13 @@ bool TerminalSession::operator()(actions::ToggleFullscreen)
     return true;
 }
 
+bool TerminalSession::operator()(actions::ToggleInputMethodHandling)
+{
+    if (_display)
+        _display->toggleInputMethodEditorHandling();
+    return true;
+}
+
 bool TerminalSession::operator()(actions::ToggleInputProtection)
 {
     terminal().setAllowInput(!terminal().allowInput());
@@ -1430,37 +1546,62 @@ bool TerminalSession::operator()(actions::WriteScreen const& event)
 
 bool TerminalSession::operator()(actions::CreateNewTab)
 {
-    emit createNewTab();
+    _manager->allowCreation();
+    _manager->createSession();
     return true;
 }
 
 bool TerminalSession::operator()(actions::CloseTab)
 {
-    emit closeTab();
+    _manager->closeTab();
+    return true;
+}
+
+bool TerminalSession::operator()(actions::MoveTabTo event)
+{
+    _manager->moveTabTo(event.position);
+    return true;
+}
+
+bool TerminalSession::operator()(actions::MoveTabToLeft)
+{
+    _manager->moveTabToLeft(this);
+    return true;
+}
+
+bool TerminalSession::operator()(actions::MoveTabToRight)
+{
+    _manager->moveTabToRight(this);
     return true;
 }
 
 bool TerminalSession::operator()(actions::SwitchToTab const& event)
 {
-    emit switchToTab(event.position);
+    _manager->switchToTab(event.position);
     return true;
 }
 
 bool TerminalSession::operator()(actions::SwitchToPreviousTab)
 {
-    emit switchToPreviousTab();
+    _manager->switchToPreviousTab();
     return true;
 }
 
 bool TerminalSession::operator()(actions::SwitchToTabLeft)
 {
-    emit switchToTabLeft();
+    _manager->switchToTabLeft();
     return true;
 }
 
 bool TerminalSession::operator()(actions::SwitchToTabRight)
 {
-    emit switchToTabRight();
+    _manager->switchToTabRight();
+    return true;
+}
+
+bool TerminalSession::operator()(actions::SetTabName)
+{
+    terminal().requestTabName();
     return true;
 }
 
@@ -1556,6 +1697,8 @@ void TerminalSession::spawnNewTerminal(string const& profileName)
     {
         sessionLog()("spawning new in-process window");
         _app.config().profile(_profileName)->shell.value().workingDirectory = fs::path(wd);
+        _manager->doNotSwitchToNewSession();
+        _manager->allowCreation();
         _app.newWindow();
     }
 }
