@@ -32,6 +32,7 @@ namespace display
 }
 
 class ContourGuiApp;
+class TerminalSessionManager;
 
 /**
  * A set of user-facing activities that are guarded behind a permission-check wall.
@@ -172,13 +173,8 @@ class TerminalSession: public QAbstractItemModel, public vtbackend::Terminal::Ev
         return true;
     }
 
-    int getScrollX() const noexcept { return _accumulatedScrollX; }
-    void addScrollX(int v) noexcept { _accumulatedScrollX += v; }
-    void resetScrollX() noexcept { _accumulatedScrollX = 0; }
-
-    int getScrollY() const noexcept { return _accumulatedScrollY; }
-    void addScrollY(int v) noexcept { _accumulatedScrollY += v; }
-    void resetScrollY() noexcept { _accumulatedScrollY = 0; }
+    void addToAccumulatedScroll(crispy::point pixelDelta, crispy::point angleDelta) noexcept;
+    std::tuple<vtbackend::LineOffset, vtbackend::ColumnOffset> consumeScroll() noexcept;
 
     QString title() const;
     void setTitle(QString const& value) { terminal().setWindowTitle(value.toStdString()); }
@@ -215,10 +211,18 @@ class TerminalSession: public QAbstractItemModel, public vtbackend::Terminal::Ev
      * @param pty a PTY object (can be process, networked, mockup, ...)
      * @param display fronend display to render the terminal.
      */
-    TerminalSession(std::unique_ptr<vtpty::Pty> pty, ContourGuiApp& app);
+    TerminalSession(TerminalSessionManager* manager, std::unique_ptr<vtpty::Pty> pty, ContourGuiApp& app);
     ~TerminalSession() override;
 
     int id() const noexcept { return _id; }
+    std::optional<std::string> name() const noexcept
+    {
+        if (terminal().tabName())
+            return terminal().tabName();
+        if (terminal().getTabsNamingMode() == vtbackend::TabsNamingMode::Title)
+            return terminal().windowTitle();
+        return std::nullopt;
+    }
 
     /// Starts the VT background thread.
     void start();
@@ -240,6 +244,8 @@ class TerminalSession: public QAbstractItemModel, public vtbackend::Terminal::Ev
     void attachDisplay(display::TerminalDisplay& display);
     void detachDisplay(display::TerminalDisplay& display);
 
+    TerminalSessionManager* getTerminalManager() const noexcept { return _manager; }
+
     Q_INVOKABLE void applyPendingFontChange(bool allow, bool remember);
     Q_INVOKABLE void applyPendingPaste(bool allow, bool remember);
     Q_INVOKABLE void executePendingBufferCapture(bool allow, bool remember);
@@ -345,6 +351,7 @@ class TerminalSession: public QAbstractItemModel, public vtbackend::Terminal::Ev
     bool operator()(actions::SendChars const& event);
     bool operator()(actions::ToggleAllKeyMaps);
     bool operator()(actions::ToggleFullscreen);
+    bool operator()(actions::ToggleInputMethodHandling);
     bool operator()(actions::ToggleInputProtection);
     bool operator()(actions::ToggleStatusLine);
     bool operator()(actions::ToggleTitleBar);
@@ -356,10 +363,14 @@ class TerminalSession: public QAbstractItemModel, public vtbackend::Terminal::Ev
     bool operator()(actions::WriteScreen const& event);
     bool operator()(actions::CreateNewTab);
     bool operator()(actions::CloseTab);
+    bool operator()(actions::MoveTabTo);
+    bool operator()(actions::MoveTabToLeft);
+    bool operator()(actions::MoveTabToRight);
     bool operator()(actions::SwitchToTab const& event);
     bool operator()(actions::SwitchToPreviousTab);
     bool operator()(actions::SwitchToTabLeft);
     bool operator()(actions::SwitchToTabRight);
+    bool operator()(actions::SetTabName);
 
     void scheduleRedraw();
 
@@ -437,6 +448,7 @@ class TerminalSession: public QAbstractItemModel, public vtbackend::Terminal::Ev
 
     // private data
     //
+    TerminalSessionManager* _manager;
     int _id;
     std::chrono::steady_clock::time_point _startTime;
     config::Config _config;
@@ -445,8 +457,8 @@ class TerminalSession: public QAbstractItemModel, public vtbackend::Terminal::Ev
     ContourGuiApp& _app;
     vtbackend::ColorPreference _currentColorPreference;
 
-    int _accumulatedScrollX;
-    int _accumulatedScrollY;
+    crispy::point _accumulatedPixelScroll;
+    crispy::point _accumulatedAngleScroll;
 
     vtbackend::Terminal _terminal;
     bool _terminatedAndWaitingForKeyPress = false;
