@@ -68,23 +68,23 @@ ShaderConfig builtinShaderConfig(ShaderClass shaderClass)
     auto const makeConfig = [](ShaderClass shaderClass) -> ShaderConfig {
         auto const makeSource = [](QString const& filename) -> ShaderSource {
             QFile sharedDefinesFile(":/contour/vtrasterizer/shared_defines.h");
-            sharedDefinesFile.open(QFile::ReadOnly);
-            Require(sharedDefinesFile.isOpen());
+            Require(sharedDefinesFile.open(QFile::ReadOnly));
             auto const sharedDefines = sharedDefinesFile.readAll().toStdString() + "\n#line 1\n";
             auto const versionHeader = std::format("#version {}\n", useOpenGLES() ? "300 es" : "330");
 
             auto const shaderFilePath = ":/contour/display/shaders/" + filename;
 
             QFile file(shaderFilePath);
-            file.open(QFile::ReadOnly);
-            Require(file.isOpen());
+            Require(file.open(QFile::ReadOnly));
             auto const fileContents = file.readAll().toStdString();
 
             auto const fileHeader = versionHeader + sharedDefines;
-            return ShaderSource { shaderFilePath, QString::fromStdString(fileHeader + fileContents) };
+            return ShaderSource { .location = shaderFilePath,
+                                  .contents = QString::fromStdString(fileHeader + fileContents) };
         };
         QString const basename = QString::fromStdString(to_string(shaderClass));
-        return ShaderConfig { makeSource(basename + ".vert"), makeSource(basename + ".frag") };
+        return ShaderConfig { .vertexShader = makeSource(basename + ".vert"),
+                              .fragmentShader = makeSource(basename + ".frag") };
     };
     return makeConfig(shaderClass);
 }
