@@ -53,7 +53,7 @@ namespace fs = std::filesystem;
 #endif
 
 // Must be in global namespace
-// NB: must be publically visible, and due to -Wmissing-declarations, we better tell the compiler.
+// NB: must be publicly visible, and due to -Wmissing-declarations, we better tell the compiler.
 void initializeResourcesForContourFrontendOpenGL();
 
 void initializeResourcesForContourFrontendOpenGL()
@@ -306,6 +306,10 @@ void TerminalDisplay::setSession(TerminalSession* newSession)
 
     QObject::connect(newSession, &TerminalSession::titleChanged, this, &TerminalDisplay::titleChanged);
 
+    auto const imeEnabled = profile().inputMethodEditor.value();
+    setFlag(Flag::ItemAcceptsInputMethod, imeEnabled);
+    displayLog()("IME enabled: {}", imeEnabled);
+
     _session->start();
 
     window()->setFlag(Qt::FramelessWindowHint, !profile().showTitleBar.value());
@@ -693,10 +697,8 @@ void TerminalDisplay::paint()
 
     try
     {
-#if QT_VERSION >= QT_VERSION_CHECK(5, 14, 0)
         window()->beginExternalCommands();
         auto const _ = gsl::finally([this]() { window()->endExternalCommands(); });
-#endif
 
         [[maybe_unused]] auto const lastState = _state.fetchAndClear();
 
@@ -828,8 +830,12 @@ void TerminalDisplay::mouseReleaseEvent(QMouseEvent* event)
 void TerminalDisplay::focusInEvent(QFocusEvent* event)
 {
     QQuickItem::focusInEvent(event);
+
     if (_session)
+    {
+        _session->getTerminalManager()->FocusOnDisplay(this);
         _session->sendFocusInEvent(); // TODO: paint with "normal" colors
+    }
 }
 
 void TerminalDisplay::focusOutEvent(QFocusEvent* event)
@@ -1107,10 +1113,8 @@ void TerminalDisplay::copyToClipboard(std::string_view data)
 
 void TerminalDisplay::inspect()
 {
-// Ensure we're invoked on GUI thread when calling doDumpState().
-#if QT_VERSION >= QT_VERSION_CHECK(5, 10, 0)
+    // Ensure we're invoked on GUI thread when calling doDumpState().
     QMetaObject::invokeMethod(this, &TerminalDisplay::doDumpState, Qt::QueuedConnection);
-#endif
 }
 
 void TerminalDisplay::doDumpState()
@@ -1127,7 +1131,7 @@ QImage TerminalDisplay::screenshot()
                   size.width.as<int>(),
                   size.height.as<int>(),
                   QImage::Format_RGBA8888_Premultiplied)
-        .mirrored(false, true);
+        .transformed(QTransform().scale(1, -1));
 }
 
 void TerminalDisplay::doDumpStateInternal()
@@ -1366,6 +1370,13 @@ void TerminalDisplay::toggleTitleBar()
     window()->setFlag(Qt::FramelessWindowHint, !currentlyFrameless);
 }
 
+void TerminalDisplay::toggleInputMethodEditorHandling()
+{
+    auto const enabled = !static_cast<bool>(flags() & Flag::ItemAcceptsInputMethod);
+    displayLog()("{} IME (input method editor) handling", enabled ? "Enabling" : "Disabling");
+    setFlag(Flag::ItemAcceptsInputMethod, enabled);
+}
+
 void TerminalDisplay::setHyperlinkDecoration(vtrasterizer::Decorator normal, vtrasterizer::Decorator hover)
 {
     _renderer->setHyperlinkDecoration(normal, hover);
