@@ -78,11 +78,7 @@ namespace
                                            config::WindowMargins margins,
                                            double dpr) noexcept
     {
-#if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
         auto const position = event->position();
-#else
-        auto const position = event->pos();
-#endif
         auto const marginLeft = static_cast<int>(unbox(margins.horizontal) * dpr);
         auto const marginTop = static_cast<int>(unbox(margins.vertical) * dpr);
         return PixelCoordinate { .x = PixelCoordinate::X { int(double(position.x()) * dpr) - marginLeft },
@@ -93,11 +89,7 @@ namespace
                                            config::WindowMargins margins,
                                            double dpr) noexcept
     {
-#if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
         auto const position = event->position();
-#else
-        auto const position = QPointF { static_cast<qreal>(event->x()), static_cast<qreal>(event->y()) };
-#endif
         auto const marginLeft = static_cast<int>(unbox(margins.horizontal) * dpr);
         auto const marginTop = static_cast<int>(unbox(margins.vertical) * dpr);
         return PixelCoordinate { .x = PixelCoordinate::X { int(double(position.x()) * dpr) - marginLeft },
@@ -108,11 +100,7 @@ namespace
                                            config::WindowMargins margins,
                                            double dpr) noexcept
     {
-#if QT_VERSION >= QT_VERSION_CHECK(5, 14, 0)
         auto const position = event->position();
-#else
-        auto const position = event->posF();
-#endif
         auto const marginLeft = static_cast<int>(unbox(margins.horizontal) * dpr);
         auto const marginTop = static_cast<int>(unbox(margins.vertical) * dpr);
         return PixelCoordinate { .x = PixelCoordinate::X { int(double(position.x()) * dpr) - marginLeft },
@@ -125,36 +113,40 @@ namespace
         return QPoint { p.y(), p.x() };
     }
 
-    void sendWheelEventForDelta(QPoint const& delta,
-                                PixelCoordinate const& position,
-                                vtbackend::Modifiers modifiers,
-                                TerminalSession& session)
+    void sendWheelEvent(crispy::point const& pixelDelta,
+                        crispy::point const& angleDelta,
+                        PixelCoordinate const& currentMousePixelPosition,
+                        vtbackend::Modifiers modifiers,
+                        TerminalSession& session)
     {
         using VTMouseButton = vtbackend::MouseButton;
 
-        session.addScrollX(delta.x());
-        session.addScrollY(delta.y());
+        session.addToAccumulatedScroll(pixelDelta, angleDelta);
+        auto const [linesScroll, columnsScroll] = session.consumeScroll();
 
-        inputLog()("[{}] Accumulate scroll with current value {}",
+        inputLog()("[{}] Accumulate scroll with by value {} pixelDelta / {} angleDelta, {} lines, {} columns "
+                   "(against {})",
                    modifiers,
-                   crispy::point { .x = session.getScrollX(), .y = session.getScrollY() });
+                   pixelDelta,
+                   angleDelta,
+                   linesScroll,
+                   columnsScroll,
+                   session.terminal().cellPixelSize());
 
-        if (std::abs(session.getScrollX()) > unbox<int>(session.terminal().cellPixelSize().width))
+        auto const horizontalScrollEvent =
+            columnsScroll.as<int>() > 0 ? VTMouseButton::WheelRight : VTMouseButton::WheelLeft;
+
+        for (int i = 0; i < std::abs(columnsScroll.as<int>()); ++i)
         {
-            session.sendMousePressEvent(modifiers,
-                                        session.getScrollX() > 0 ? VTMouseButton::WheelRight
-                                                                 : VTMouseButton::WheelLeft,
-                                        position);
-            session.resetScrollX();
+            session.sendMousePressEvent(modifiers, horizontalScrollEvent, currentMousePixelPosition);
         }
 
-        if (std::abs(session.getScrollY()) > unbox<int>(session.terminal().cellPixelSize().height))
+        auto const verticalScrollEvent =
+            linesScroll.as<int>() > 0 ? VTMouseButton::WheelUp : VTMouseButton::WheelDown;
+
+        for (int i = 0; i < std::abs(linesScroll.as<int>()); ++i)
         {
-            session.sendMousePressEvent(modifiers,
-                                        session.getScrollY() > 0 ? VTMouseButton::WheelUp
-                                                                 : VTMouseButton::WheelDown,
-                                        position);
-            session.resetScrollY();
+            session.sendMousePressEvent(modifiers, verticalScrollEvent, currentMousePixelPosition);
         }
     }
 
@@ -387,10 +379,12 @@ bool sendKeyEvent(QKeyEvent* event, vtbackend::KeyboardEventType eventType, Term
         return true;
     }
 
-#if defined(__apple__)
-    if (0x20 <= key && key < 0x80 && (modifiers.alt() && session.profile().optionKeyAsAlt.value()))
+#if defined(__APPLE__)
+    if (0x20 <= key && key < 0x80
+        && (modifiers.test(Modifier::Alt) && session.profile().optionKeyAsAlt.value()))
     {
-        auto const ch = static_cast<char32_t>(modifiers.shift() ? std::toupper(key) : std::tolower(key));
+        bool const shiftPressed = modifiers.test(Modifier::Shift) ^ modifiers.test(Modifier::CapsLock);
+        auto const ch = static_cast<char32_t>(shiftPressed ? std::toupper(key) : std::tolower(key));
         session.sendCharEvent(ch, physicalKey, modifiers, eventType, now);
         event->accept();
         return true;
@@ -407,7 +401,7 @@ bool sendKeyEvent(QKeyEvent* event, vtbackend::KeyboardEventType eventType, Term
     if (!event->text().isEmpty())
     {
         auto const codepoints = event->text().toUcs4();
-        assert(codepoints.size() == 1);
+        assert(codepoints.size() >= 1);
 #if defined(__APPLE__)
         // On macOS the Alt-modifier does not seem to be passed to the terminal apps
         // but rather remapped to whatever macOS is mapping them to.
@@ -427,39 +421,45 @@ bool sendKeyEvent(QKeyEvent* event, vtbackend::KeyboardEventType eventType, Term
 
 void sendWheelEvent(QWheelEvent* event, TerminalSession& session)
 {
+    if (event->pixelDelta().isNull() && event->angleDelta().isNull())
+        return;
+
     using vtbackend::Modifier;
 
     auto const modifiers = makeModifiers(event->modifiers());
 
+    auto const pixelPosition =
+        makeMousePixelPosition(event, session.profile().margins.value(), session.display()->contentScale());
+
     // NOTE: Qt is playing some weird games with the mouse wheel events, i.e. if Alt is pressed
     //       it will send horizontal wheel events instead of vertical ones. We need to compensate
     //       for that here.
 
-    auto const scaledPixelDelta = session.display()->contentScale() * event->pixelDelta();
-    auto const pixelDelta = (modifiers & Modifier::Alt) ? transposed(scaledPixelDelta) : scaledPixelDelta;
+    auto const pixelDelta = [&]() -> crispy::point {
+        if (event->pixelDelta().isNull())
+            return { .x = 0, .y = 0 };
 
-    auto const numDegrees =
-        ((modifiers & Modifier::Alt) ? transposed(event->angleDelta()) : event->angleDelta()) / 8;
+        auto const scaledPixelDelta = session.display()->contentScale() * event->pixelDelta();
+        auto const x = scaledPixelDelta.x();
+        auto const y = scaledPixelDelta.y();
+        if (modifiers & Modifier::Alt)
+            return { .x = y, .y = x };
+        else
+            return { .x = x, .y = y };
+    }();
 
-    auto const pixelPosition =
-        makeMousePixelPosition(event, session.profile().margins.value(), session.display()->contentScale());
+    auto const angleDelta = [&]() -> crispy::point {
+        if (event->angleDelta().isNull())
+            return { .x = 0, .y = 0 };
 
-    if (!pixelDelta.isNull())
-    {
-        sendWheelEventForDelta(pixelDelta, pixelPosition, modifiers, session);
-        event->accept();
-    }
-    else if (!numDegrees.isNull())
-    {
-        auto const numSteps = numDegrees / 15;
-        auto const cellSize = session.terminal().cellPixelSize();
-        auto const scaledDelta = QPoint {
-            numSteps.x() * unbox<int>(cellSize.width),
-            numSteps.y() * unbox<int>(cellSize.height),
-        };
-        sendWheelEventForDelta(scaledDelta, pixelPosition, modifiers, session);
-        event->accept();
-    }
+        auto const numDegrees =
+            ((modifiers & Modifier::Alt) ? transposed(event->angleDelta()) : event->angleDelta());
+
+        return { .x = numDegrees.x(), .y = numDegrees.y() };
+    }();
+
+    sendWheelEvent(pixelDelta, angleDelta, pixelPosition, modifiers, session);
+    event->accept();
 }
 
 void sendMousePressEvent(QMouseEvent* event, TerminalSession& session)
@@ -491,11 +491,7 @@ void sendMouseMoveEvent(QMouseEvent* event, TerminalSession& session)
 
 void sendMouseMoveEvent(QHoverEvent* event, TerminalSession& session)
 {
-#if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
     auto const position = event->position().toPoint();
-#else
-    auto const position = event->pos();
-#endif
     session.sendMouseMoveEvent(
         makeModifiers(event->modifiers()),
         makeMouseCellLocation(position.x(), position.y(), session),
