@@ -2,7 +2,6 @@
 #pragma once
 
 #include <format>
-#include <ostream>
 
 namespace crispy
 {
@@ -11,24 +10,51 @@ struct [[nodiscard]] point
 {
     int x {};
     int y {};
+
+    constexpr bool operator!() const noexcept { return x == 0 && y == 0; }
+    constexpr operator bool() const noexcept { return x != 0 || y != 0; }
 };
 
 template <typename T>
 constexpr inline T Zero {};
 template <>
-constexpr inline point Zero<point> = point { 0, 0 };
+constexpr inline point Zero<point> = point { .x = 0, .y = 0 };
 
 constexpr point operator*(point p, double s) noexcept
 {
     return point {
-        static_cast<int>(static_cast<double>(p.x) * s),
-        static_cast<int>(static_cast<double>(p.y) * s),
+        .x = static_cast<int>(static_cast<double>(p.x) * s),
+        .y = static_cast<int>(static_cast<double>(p.y) * s),
+    };
+}
+
+constexpr point operator/(point p, double s) noexcept
+{
+    return point {
+        .x = static_cast<int>(static_cast<double>(p.x) / s),
+        .y = static_cast<int>(static_cast<double>(p.y) / s),
+    };
+}
+
+constexpr point operator*(point a, point b) noexcept
+{
+    return point {
+        .x = a.x * b.x,
+        .y = a.y * b.y,
+    };
+}
+
+constexpr point operator/(point a, point b) noexcept
+{
+    return point {
+        .x = a.x / b.x,
+        .y = a.y / b.y,
     };
 }
 
 constexpr point operator+(point a, point b) noexcept
 {
-    return point { a.x + b.x, a.y + b.y };
+    return point { .x = a.x + b.x, .y = a.y + b.y };
 }
 
 constexpr point& operator+=(point& a, point b) noexcept
@@ -38,6 +64,13 @@ constexpr point& operator+=(point& a, point b) noexcept
     return a;
 }
 
+constexpr point& operator-=(point& a, point b) noexcept
+{
+    a.x -= b.x;
+    a.y -= b.y;
+    return a;
+}
+
 constexpr void swap(point& a, point& b) noexcept
 {
     point const c = a;
