@@ -3,9 +3,6 @@
 #include <crispy/defines.h>
 #include <crispy/escape.h>
 
-#include <range/v3/view/iota.hpp>
-#include <range/v3/view/transform.hpp>
-
 #include <algorithm>
 #include <filesystem>
 #include <format>
@@ -13,9 +10,11 @@
 #include <functional>
 #include <mutex>
 #include <optional>
+#include <ranges>
 #include <sstream>
 #include <string>
 #include <string_view>
+#include <type_traits>
 #include <unordered_map>
 #include <vector>
 
@@ -31,16 +30,124 @@ namespace views
     template <typename T>
     auto as()
     {
-        return ranges::views::transform([](auto in) { return T(in); });
+        return std::views::transform([](auto in) { return T(in); });
     }
 
     template <typename T>
     auto iota_as(int n)
     {
-        return ranges::views::ints(0, n) | as<T>();
+        return std::views::iota(0, n) | as<T>();
     }
+
+    namespace detail
+    {
+        template <typename Range>
+        std::string join_with_impl(Range const& range, std::string_view separator)
+        {
+            std::string result;
+            auto it = std::begin(range);
+            auto const end = std::end(range);
+
+            if (it != end)
+            {
+                result += *it;
+                ++it;
+            }
+
+            for (; it != end; ++it)
+            {
+                result += separator;
+                result += *it;
+            }
+            return result;
+        }
+
+        struct join_with_fn
+        {
+            std::string_view separator;
+
+            template <typename R>
+            friend auto operator|(R&& r, join_with_fn const& self)
+            {
+                return join_with_impl(std::forward<R>(r), self.separator);
+            }
+        };
+    } // namespace detail
+
+    inline auto join_with(std::string_view sep)
+    {
+        return detail::join_with_fn { sep };
+    }
+
+    template <typename Range>
+    auto join_with(Range&& range, std::string_view sep)
+    {
+        return detail::join_with_impl(std::forward<Range>(range), sep);
+    }
+    template <typename Range>
+    struct enumerate_view_sentinel
+    {
+        using sentinel = std::ranges::sentinel_t<Range>;
+        sentinel end;
+
+        friend constexpr bool operator==(std::ranges::iterator_t<Range> const& it,
+                                         enumerate_view_sentinel const& s)
+        {
+            return it == s.end;
+        }
+    };
+
+    template <typename Range>
+    struct enumerate_view
+    {
+        Range range;
+
+        struct iterator
+        {
+            using range_iterator = std::ranges::iterator_t<Range>;
+            using range_reference = std::ranges::range_reference_t<Range>;
+            using difference_type = std::ptrdiff_t;
+            using value_type = std::pair<size_t, std::ranges::range_value_t<Range>>;
+
+            size_t index;
+            range_iterator current;
+
+            constexpr auto operator*() const
+            {
+                return std::pair<size_t, range_reference> { index, *current };
+            }
+
+            constexpr iterator& operator++()
+            {
+                ++index;
+                ++current;
+                return *this;
+            }
+
+            constexpr bool operator!=(iterator const& other) const { return current != other.current; }
+            constexpr bool operator==(iterator const& other) const { return current == other.current; }
+            constexpr bool operator!=(std::ranges::sentinel_t<Range> const& s) const { return current != s; }
+            constexpr bool operator==(std::ranges::sentinel_t<Range> const& s) const { return current == s; }
+        };
+
+        constexpr auto begin() { return iterator { 0, std::begin(range) }; }
+        constexpr auto end() { return std::end(range); }
+    };
+
+    struct enumerate_fn
+    {
+        template <typename Range>
+        constexpr auto operator()(Range&& range) const
+        {
+            return enumerate_view<Range> { std::forward<Range>(range) };
+        }
+    };
+
+    constexpr inline enumerate_fn enumerate; // NOLINT(readability-identifier-naming)
 } // namespace views
 
+using views::join_with;
+
 constexpr std::string_view trimRight(std::string_view value) noexcept
 {
     while (!value.empty())
@@ -117,27 +224,29 @@ std::string joinHumanReadableQuoted(std::vector<T> const& list, U sep = ", ")
 }
 
 template <typename T, typename Callback>
-constexpr inline bool split(std::basic_string_view<T> text, T delimiter, Callback&& callback)
+constexpr bool split(std::basic_string_view<T> text,
+                     T delimiter,
+                     Callback const& callback) noexcept(noexcept(callback(std::basic_string_view<T> {})))
 {
     size_t a = 0;
     size_t b = 0;
     while ((b = text.find(delimiter, a)) != std::basic_string_view<T>::npos)
     {
-        if (!(std::forward<Callback>(callback)(text.substr(a, b - a))))
+        if (!(callback(text.substr(a, b - a))))
             return false;
 
         a = b + 1;
     }
 
     if (a < text.size())
-        return std::forward<Callback>(callback)(text.substr(a));
+        return callback(text.substr(a));
 
     return true;
 }
 
 template <typename T>
-constexpr inline auto split(std::basic_string_view<T> text,
-                            T delimiter) -> std::vector<std::basic_string_view<T>>
+constexpr inline auto split(std::basic_string_view<T> text, T delimiter)
+    -> std::vector<std::basic_string_view<T>>
 {
     std::vector<std::basic_string_view<T>> output {};
     split(text, delimiter, [&](auto value) {
@@ -271,7 +380,7 @@ constexpr std::optional<T> to_integer(std::basic_string_view<C> text) noexcept
 }
 
 template <std::size_t Base = 10, typename T = unsigned, typename C>
-constexpr std::optional<T> to_integer(std::basic_string<C> text) noexcept
+constexpr std::optional<T> to_integer(std::basic_string<C> const& text) noexcept
 {
     return to_integer<Base, T, C>(std::basic_string_view<C>(text));
 }
@@ -334,6 +443,60 @@ inline std::optional<unsigned> fromHexDigit(char value)
     return std::nullopt;
 }
 
+inline std::string unescapeURL(std::string_view input)
+{
+    std::string result;
+    result.reserve(input.size());
+    for (size_t i = 0; i < input.size(); ++i)
+    {
+        if (input[i] == '%' && i + 2 < input.size())
+        {
+            auto const h1 = crispy::fromHexDigit(input[i + 1]);
+            auto const h2 = crispy::fromHexDigit(input[i + 2]);
+            if (h1.has_value() && h2.has_value())
+            {
+                result.push_back(static_cast<char>((h1.value() << 4) | h2.value()));
+                i += 2;
+                continue;
+            }
+        }
+        result.push_back(input[i]);
+    }
+    return result;
+}
+
+struct for_each_key_value_params
+{
+    std::string_view text;
+    char entryDelimiter;
+    char assignmentDelimiter;
+
+    template <typename Callback>
+        requires std::is_invocable_v<Callback, std::string_view, std::string_view>
+    void operator()(Callback const& callback) noexcept(noexcept(callback(std::string_view {},
+                                                                         std::string_view {})))
+    {
+        split(text, entryDelimiter, [&](std::string_view keyValuePair) {
+            auto const assignmentSeparator = keyValuePair.find(assignmentDelimiter);
+            if (assignmentSeparator != std::string_view::npos)
+                callback(keyValuePair.substr(0, assignmentSeparator),
+                         keyValuePair.substr(assignmentSeparator + 1));
+            else if (!keyValuePair.empty())
+                callback(keyValuePair, std::string_view {});
+            return true;
+        });
+    }
+};
+
+template <typename Callback>
+    requires std::is_invocable_v<Callback, std::string_view, std::string_view>
+constexpr void for_each_key_value(for_each_key_value_params params,
+                                  Callback&& callback) noexcept(noexcept(callback(std::string_view {},
+                                                                                  std::string_view {})))
+{
+    params(std::forward<Callback>(callback));
+}
+
 template <typename T>
 std::optional<std::basic_string<T>> fromHexString(std::basic_string_view<T> hexString)
 {
@@ -560,4 +723,7 @@ struct overloaded: Ts...
 template <class... Ts>
 overloaded(Ts...) -> overloaded<Ts...>;
 
+template <typename T, typename... Ts>
+concept one_of = (std::same_as<T, Ts> || ...);
+
 } // namespace crispy
