@@ -5,11 +5,10 @@
 #include <crispy/assert.h>
 #include <crispy/utils.h>
 
-#include <range/v3/view/drop.hpp>
-#include <range/v3/view/iota.hpp>
-
 #include <fontconfig/fontconfig.h>
 
+#include <algorithm>
+#include <ranges>
 #include <string_view>
 #include <variant>
 
@@ -149,10 +148,7 @@ struct fontconfig_locator::private_tag
     }
 };
 
-fontconfig_locator::fontconfig_locator():
-    _d { new private_tag(), [](private_tag* p) {
-            delete p;
-        } }
+fontconfig_locator::fontconfig_locator(): _d { new private_tag(), [](private_tag* p) { delete p; } }
 {
 }
 
@@ -252,7 +248,10 @@ font_source_list fontconfig_locator::locate(font_description const& description)
         if (FcPatternGetInteger(font, FC_SLANT, 0, &integerValue) == FcResultMatch)
             slant = fcToFontSlant(integerValue);
 
-        output.emplace_back(font_path { string { (char const*) (file) }, ttcIndex, weight, slant });
+        output.emplace_back(font_path { .value = string { (char const*) (file) },
+                                        .collectionIndex = ttcIndex,
+                                        .weight = weight,
+                                        .slant = slant });
         locatorLog()("Font {} (ttc index {}, weight {}, slant {}, spacing {}) in chain: {}",
                      output.size(),
                      ttcIndex,
@@ -272,7 +271,7 @@ font_source_list fontconfig_locator::locate(font_description const& description)
                        // find font in the fallback list and add it
                        for (auto&& fallbackFont: list.fallbackFonts)
                        {
-                           for (auto i: ranges::views::ints(1, fs->nfont))
+                           for (auto i: std::views::iota(1, fs->nfont))
                            {
                                FcPattern* font = fs->fonts[i];
 
@@ -281,6 +280,7 @@ font_source_list fontconfig_locator::locate(font_description const& description)
 
                                // remove spaces from the fonts names
                                auto fallbackFontNoSpaces = fallbackFont;
+                               // NOLINTBEGIN
                                fallbackFontNoSpaces.erase(
                                    std::remove(fallbackFontNoSpaces.begin(), fallbackFontNoSpaces.end(), ' '),
                                    fallbackFontNoSpaces.end());
@@ -288,6 +288,7 @@ font_source_list fontconfig_locator::locate(font_description const& description)
                                familyNoSpaces.erase(
                                    std::remove(familyNoSpaces.begin(), familyNoSpaces.end(), ' '),
                                    familyNoSpaces.end());
+                               // NOLINTEND
                                if (fallbackFontNoSpaces == familyNoSpaces)
                                {
                                    addFont(font);
@@ -297,7 +298,7 @@ font_source_list fontconfig_locator::locate(font_description const& description)
                        }
                    },
                    [&](std::monostate) {
-                       for (auto i: ranges::views::ints(1, fs->nfont))
+                       for (auto i: std::views::iota(1, fs->nfont))
                            addFont(fs->fonts[i]);
                    },
                },
@@ -388,7 +389,7 @@ font_source_list fontconfig_locator::all()
             continue;
 
         locatorLog()("font({}, {}, {})", fcWeightStr(weight), fcSlantStr(slant), (char*) family);
-        output.emplace_back(font_path { (char const*) filename });
+        output.emplace_back(font_path { .value = (char const*) filename });
     }
 
     FcObjectSetDestroy(os);
