@@ -51,15 +51,15 @@ struct Def
 };
 
 // {{{ variable names
-constexpr auto inline AutoLeftMargin = Def { Code { "am" }, "am" };
-constexpr auto inline CanChange = Def { Code { "cc" }, "ccc" };
-constexpr auto inline EatNewlineGlitch = Def { Code { "xn" }, "xenl" };
+constexpr auto inline AutoLeftMargin = Def { .code = Code { "am" }, .name = "am" };
+constexpr auto inline CanChange = Def { .code = Code { "cc" }, .name = "ccc" };
+constexpr auto inline EatNewlineGlitch = Def { .code = Code { "xn" }, .name = "xenl" };
 // TODO ... (all the rest that is at least needed by us)
 // }}}
 
 namespace literals
 {
-    constexpr Code operator"" _tcap(char const* code, size_t)
+    constexpr Code operator""_tcap(char const* code, size_t)
     {
         return Code { uint16_t(code[0] << 8 | code[1]) };
     }
