@@ -13,10 +13,6 @@
 
 #include <libunicode/convert.h>
 
-#include <range/v3/algorithm/copy.hpp>
-#include <range/v3/iterator/insert_iterators.hpp>
-#include <range/v3/view/iota.hpp>
-
 #include <gsl/span>
 #include <gsl/span_ext>
 
@@ -164,7 +160,8 @@ struct LogicalLine
                     std::u32string_view const remainingTextToMatch(searchText.data() + result,
                                                                    searchText.size() - result);
                     if (matchTextAt(remainingTextToMatch, ColumnOffset(0), line + 1, isCaseSensitive))
-                        return CellLocation { i, ColumnOffset(static_cast<int>(lineLength - result)) };
+                        return CellLocation { .line = i,
+                                              .column = ColumnOffset(static_cast<int>(lineLength - result)) };
                 }
                 startPosition = ColumnOffset(0);
                 ++i;
@@ -183,8 +180,8 @@ struct LogicalLine
                 if (line + 1 != lines.end()
                     && (line + 1)->get().matchTextAtWithSensetivityMode(
                         remainingText, ColumnOffset(0), isCaseSensitive))
-                    return CellLocation { i,
-                                          ColumnOffset::cast_from(
+                    return CellLocation { .line = i,
+                                          .column = ColumnOffset::cast_from(
                                               static_cast<int>(unbox<size_t>(line->get().size())
                                                                - result->partialMatchLength)) };
             }
@@ -758,7 +755,7 @@ class Grid
     [[nodiscard]] bool cellEmptyOrContainsOneOf(CellLocation position,
                                                 std::u32string_view delimiters) const noexcept;
 
-    // Lineary extracts the text of a given grid cell range.
+    // Linearly extracts the text of a given grid cell range.
     [[nodiscard]] std::u32string extractText(CellLocationRange range) const noexcept;
 
     // Conditionally extends the cell location forward if the grid cell at the given location holds a wide
@@ -782,7 +779,7 @@ class Grid
         if (line.isTrivialBuffer())
         {
             if (line.empty())
-                return CellLocation { lineOffset, ColumnOffset(0) };
+                return CellLocation { .line = lineOffset, .column = ColumnOffset(0) };
 
             auto const& trivial = line.trivialBuffer();
             auto const columnOffset = ColumnOffset::cast_from(trivial.usedColumns - 1);
@@ -793,7 +790,7 @@ class Grid
         auto columnOffset = ColumnOffset::cast_from(_pageSize.columns - 1);
         while (columnOffset > ColumnOffset(0) && inflatedLine[unbox<size_t>(columnOffset)].empty())
             --columnOffset;
-        return CellLocation { lineOffset, columnOffset };
+        return CellLocation { .line = lineOffset, .column = columnOffset };
     }
 
     [[nodiscard]] uint8_t cellWidthAt(CellLocation position) const noexcept
@@ -887,11 +884,11 @@ template <typename RendererT>
             auto const cellFlags = line.trivialBuffer().textAttributes.flags;
             hints.containsBlinkingCells = hints.containsBlinkingCells || (cellFlags & CellFlag::Blinking)
                                           || (cellFlags & CellFlag::RapidBlinking);
-            render.renderTrivialLine(line.trivialBuffer(), y);
+            render.renderTrivialLine(line.trivialBuffer(), y, line.flags());
         }
         else
         {
-            render.startLine(y);
+            render.startLine(y, line.flags());
             for (Cell const& cell: line.cells())
             {
                 hints.containsBlinkingCells = hints.containsBlinkingCells
