@@ -49,8 +49,6 @@ bool StandardKeyboardInputGenerator::generateChar(char32_t characterEvent,
     if (eventType == KeyboardEventType::Release)
         return false;
 
-    char const chr = static_cast<char>(characterEvent);
-
     // See section "Alt and Meta Keys" in ctlseqs.txt from xterm.
     if (modifiers == Modifier::Alt)
         // NB: There are other modes in xterm to send Alt+Key options or even send ESC on Meta key instead.
@@ -82,22 +80,57 @@ bool StandardKeyboardInputGenerator::generateChar(char32_t characterEvent,
         return true;
     }
 
-    if (modifiers == Modifier::Control && characterEvent == L' ')
+    // See: DEC STD-070 in section 6.16 (Control Codes and Keystrokes), page 6-170
+    //
+    // https://vt100.net/mirror/antonio/std070_c_video_systems_reference_manual.pdf
+    if (modifiers.without(Modifier::Shift) == Modifier::Control)
     {
-        append('\x00');
-        return true;
-    }
-
-    if (modifiers == Modifier::Control && crispy::ascending('A', chr, 'Z'))
-    {
-        append(static_cast<char>(chr - 'A' + 1));
-        return true;
-    }
-
-    if (modifiers == Modifier::Control && characterEvent >= '[' && characterEvent <= '_')
-    {
-        append(static_cast<char>(chr - 'A' + 1)); // remaining C0 characters 0x1B .. 0x1F
-        return true;
+        // clang-format off
+        switch (characterEvent)
+        {
+            case ' ':
+            case '2':
+                append('\x00');
+                return true;
+            case 'A': case 'B': case 'C': case 'D': case 'E': case 'F':
+            case 'G': case 'H': case 'I': case 'J': case 'K': case 'L':
+            case 'M': case 'N': case 'O': case 'P': case 'Q': case 'R':
+            case 'S': case 'T': case 'U': case 'V': case 'W': case 'X':
+            case 'Y': case 'Z':
+                append(static_cast<char>(characterEvent - 'A' + 1));
+                return true;
+            case '3':
+            case '[':
+                append('\x1B');
+                return true;
+            case '4':
+            case '\\':
+                append('\x1C');
+                return true;
+            case '5':
+            case ']':
+                append('\x1D');
+                return true;
+            case '6':
+            case '~':
+            case '^':
+                append('\x1E');
+                return true;
+            case '7':
+            case '?':
+            case '_':
+                append('\x1F');
+                return true;
+            case '8':
+                append('\x7F');
+                return true;
+            case '\x09': // TAB
+                append('\x09');
+                return true;
+            default:
+            break;
+        }
+        // clang-format on
     }
 
     if (modifiers.without(Modifier::Alt).none() || modifiers == Modifier::Shift)
@@ -115,10 +148,20 @@ bool StandardKeyboardInputGenerator::generateChar(char32_t characterEvent,
     return true;
 }
 
+std::string StandardKeyboardInputGenerator::selectNumpad(Modifiers modifiers,
+                                                         FunctionKeyMapping mapping) const
+{
+    if (modifiers.contains(Modifier::NumLock))
+        return select(modifiers, { .std = mapping.std, .mods = mapping.std, .appKeypad = mapping.std });
+
+    return select(modifiers.without(Modifier::NumLock), mapping);
+}
+
 std::string StandardKeyboardInputGenerator::select(Modifiers modifiers, FunctionKeyMapping mapping) const
 {
-    if (modifiers && !mapping.mods.empty())
-        return crispy::replace(mapping.mods, "{}"sv, makeVirtualTerminalParam(modifiers));
+    if (modifiers.without(Modifier::NumLock) && !mapping.mods.empty())
+        return crispy::replace(
+            mapping.mods, "{}"sv, makeVirtualTerminalParam(modifiers.without(Modifier::NumLock)));
 
     auto const prefix = modifiers.contains(Modifier::Alt) ? "\033" : ""s;
 
@@ -192,23 +235,23 @@ bool StandardKeyboardInputGenerator::generateKey(Key key, Modifiers modifiers, K
         case Key::PageDown: append(select(modifiers, { .std = CSI "6~", .mods = CSI "6;{}~", .appKeypad = CSI "6~" })); break;
         case Key::Insert: append(select(modifiers, { .std = CSI "2~", .mods = CSI "2;{}~" })); break;
         case Key::Delete: append(select(modifiers, { .std = CSI "3~", .mods = CSI "3;{}~" })); break;
-        case Key::Numpad_Enter:    append(select(modifiers, { .std = "\r", .appKeypad = SS3 "M" })); break;
-        case Key::Numpad_Multiply: append(select(modifiers, { .std = "*",  .appKeypad = SS3 "j" })); break;
-        case Key::Numpad_Add:      append(select(modifiers, { .std = "+",  .appKeypad = SS3 "k" })); break;
-        case Key::Numpad_Subtract: append(select(modifiers, { .std = "-",  .appKeypad = SS3 "m" })); break;
-        case Key::Numpad_Decimal:  append(select(modifiers, { .std = ".",  .appKeypad = CSI "3~" })); break;
-        case Key::Numpad_Divide:   append(select(modifiers, { .std = "/",  .appKeypad = SS3 "o" })); break;
-        case Key::Numpad_0:        append(select(modifiers, { .std = "0",  .appKeypad = CSI "2~" })); break;
-        case Key::Numpad_1:        append(select(modifiers, { .std = "1",  .appKeypad = SS3 "F" })); break;
-        case Key::Numpad_2:        append(select(modifiers, { .std = "2",  .appKeypad = CSI "B" })); break;
-        case Key::Numpad_3:        append(select(modifiers, { .std = "3",  .appKeypad = CSI "6~" })); break;
-        case Key::Numpad_4:        append(select(modifiers, { .std = "4",  .appKeypad = CSI "D" })); break;
-        case Key::Numpad_5:        append(select(modifiers, { .std = "5",  .appKeypad = CSI "E" })); break;
-        case Key::Numpad_6:        append(select(modifiers, { .std = "6",  .appKeypad = CSI "C" })); break;
-        case Key::Numpad_7:        append(select(modifiers, { .std = "7",  .appKeypad = SS3 "H" })); break;
-        case Key::Numpad_8:        append(select(modifiers, { .std = "8",  .appKeypad = CSI "A" })); break;
-        case Key::Numpad_9:        append(select(modifiers, { .std = "9",  .appKeypad = CSI "5~" })); break;
-        case Key::Numpad_Equal:    append(select(modifiers, { .std = "=",  .appKeypad = SS3 "X" })); break;
+        case Key::Numpad_Enter:    append(selectNumpad(modifiers, { .std = "\r", .appKeypad = SS3 "M" })); break;
+        case Key::Numpad_Multiply: append(selectNumpad(modifiers, { .std = "*",  .appKeypad = SS3 "j" })); break;
+        case Key::Numpad_Add:      append(selectNumpad(modifiers, { .std = "+",  .appKeypad = SS3 "k" })); break;
+        case Key::Numpad_Subtract: append(selectNumpad(modifiers, { .std = "-",  .appKeypad = SS3 "m" })); break;
+        case Key::Numpad_Decimal:  append(selectNumpad(modifiers, { .std = ".",  .appKeypad = CSI "3~" })); break;
+        case Key::Numpad_Divide:   append(selectNumpad(modifiers, { .std = "/",  .appKeypad = SS3 "o" })); break;
+        case Key::Numpad_0:        append(selectNumpad(modifiers, { .std = "0",  .appKeypad = CSI "2~" })); break;
+        case Key::Numpad_1:        append(selectNumpad(modifiers, { .std = "1",  .appKeypad = SS3 "F" })); break;
+        case Key::Numpad_2:        append(selectNumpad(modifiers, { .std = "2",  .appKeypad = CSI "B" })); break;
+        case Key::Numpad_3:        append(selectNumpad(modifiers, { .std = "3",  .appKeypad = CSI "6~" })); break;
+        case Key::Numpad_4:        append(selectNumpad(modifiers, { .std = "4",  .appKeypad = CSI "D" })); break;
+        case Key::Numpad_5:        append(selectNumpad(modifiers, { .std = "5",  .appKeypad = CSI "E" })); break;
+        case Key::Numpad_6:        append(selectNumpad(modifiers, { .std = "6",  .appKeypad = CSI "C" })); break;
+        case Key::Numpad_7:        append(selectNumpad(modifiers, { .std = "7",  .appKeypad = SS3 "H" })); break;
+        case Key::Numpad_8:        append(selectNumpad(modifiers, { .std = "8",  .appKeypad = CSI "A" })); break;
+        case Key::Numpad_9:        append(selectNumpad(modifiers, { .std = "9",  .appKeypad = CSI "5~" })); break;
+        case Key::Numpad_Equal:    append(selectNumpad(modifiers, { .std = "=",  .appKeypad = SS3 "X" })); break;
         // {{{ unsupported keys in legacy input protocol
         case Key::MediaPlay:
         case Key::MediaStop:
@@ -836,7 +879,7 @@ bool InputGenerator::mouseTransportX10(uint8_t button, uint8_t modifier, CellLoc
     constexpr uint8_t SkipCount = 0x20; // TODO std::numeric_limits<ControlCode>::max();
     constexpr uint8_t MaxCoordValue = std::numeric_limits<uint8_t>::max() - SkipCount;
 
-    if (*pos.line < MaxCoordValue && *pos.column < MaxCoordValue)
+    if (std::cmp_less(unbox(pos.line), MaxCoordValue) && std::cmp_less(unbox(pos.column), MaxCoordValue))
     {
         auto const buttonValue = static_cast<uint8_t>(SkipCount + static_cast<uint8_t>(button | modifier));
         auto const line = static_cast<uint8_t>(SkipCount + *pos.line + 1);
