@@ -14,6 +14,7 @@
 #include <set>
 #include <string>
 #include <string_view>
+#include <utility>
 
 namespace vtbackend
 {
@@ -278,14 +279,20 @@ class StandardKeyboardInputGenerator: public KeyboardInputGenerator
     }
 
   protected:
+    /// Maps a function key to its corresponding VT sequences based on different modes and modifiers.
     struct FunctionKeyMapping
     {
+        /// Standard VT sequence.
         std::string_view std {};
+        /// VT sequence when modifiers are pressed.
         std::string_view mods {};
+        /// VT sequence when Application Cursor Keys mode is enabled.
         std::string_view appCursor {};
+        /// VT sequence when Application Keypad mode is enabled.
         std::string_view appKeypad {};
     };
 
+    [[nodiscard]] std::string selectNumpad(Modifiers modifier, FunctionKeyMapping mapping) const;
     [[nodiscard]] std::string select(Modifiers modifier, FunctionKeyMapping mapping) const;
     void append(char ch) { _pendingSequence += ch; }
     void append(std::string_view sequence) { _pendingSequence += sequence; }
@@ -470,7 +477,7 @@ class InputGenerator
     void consume(int n)
     {
         _consumedBytes += n;
-        if (_consumedBytes == static_cast<int>(_pendingSequence.size()))
+        if (std::cmp_equal(_consumedBytes, _pendingSequence.size()))
         {
             _consumedBytes = 0;
             _pendingSequence.clear();
