@@ -39,12 +39,12 @@ namespace
     RGBColorPair makeRGBColorPair(RGBColorPair actualColors,
                                   CellRGBColorAndAlphaPair configuredColor) noexcept
     {
-        return RGBColorPair { mix(makeRGBColor(actualColors, configuredColor.foreground),
-                                  actualColors.foreground,
-                                  configuredColor.foregroundAlpha),
-                              mix(makeRGBColor(actualColors, configuredColor.background),
-                                  actualColors.background,
-                                  configuredColor.backgroundAlpha) }
+        return RGBColorPair { .foreground = mix(makeRGBColor(actualColors, configuredColor.foreground),
+                                                actualColors.foreground,
+                                                configuredColor.foregroundAlpha),
+                              .background = mix(makeRGBColor(actualColors, configuredColor.background),
+                                                actualColors.background,
+                                                configuredColor.backgroundAlpha) }
             .distinct();
     }
 
@@ -85,15 +85,16 @@ namespace
             return selectionColors;
 
         if (!selected)
-            return RGBColorPair { makeRGBColor(sgrColors, colorPalette.cursor.textOverrideColor),
-                                  makeRGBColor(sgrColors, colorPalette.cursor.color) }
+            return RGBColorPair { .foreground =
+                                      makeRGBColor(sgrColors, colorPalette.cursor.textOverrideColor),
+                                  .background = makeRGBColor(sgrColors, colorPalette.cursor.color) }
                 .distinct();
 
         Require(isCursor && selected);
 
         auto cursorColor =
-            RGBColorPair { makeRGBColor(selectionColors, colorPalette.cursor.textOverrideColor),
-                           makeRGBColor(selectionColors, colorPalette.cursor.color) };
+            RGBColorPair { .foreground = makeRGBColor(selectionColors, colorPalette.cursor.textOverrideColor),
+                           .background = makeRGBColor(selectionColors, colorPalette.cursor.color) };
 
         return mix(cursorColor, selectionColors, 0.25f).distinct();
     }
@@ -137,13 +138,13 @@ optional<RenderCursor> RenderBufferBuilder<Cell>::renderCursor() const
     auto const shape = _terminal->focused() ? _terminal->cursorShape() : InactiveCursorShape;
 
     auto const cursorScreenPosition =
-        CellLocation { _baseLine + _cursorPosition->line
-                           + boxed_cast<LineOffset>(_terminal->viewport().scrollOffset()),
-                       _cursorPosition->column };
+        CellLocation { .line = _baseLine + _cursorPosition->line
+                               + boxed_cast<LineOffset>(_terminal->viewport().scrollOffset()),
+                       .column = _cursorPosition->column };
 
     auto const cellWidth = _terminal->currentScreen().cellWidthAt(*_cursorPosition);
 
-    return RenderCursor { cursorScreenPosition, shape, cellWidth };
+    return RenderCursor { .position = cursorScreenPosition, .shape = shape, .width = cellWidth };
 }
 
 template <CellConcept Cell>
@@ -151,6 +152,7 @@ RenderCell RenderBufferBuilder<Cell>::makeRenderCellExplicit(ColorPalette const&
                                                              u32string graphemeCluster,
                                                              ColumnCount width,
                                                              CellFlags flags,
+                                                             LineFlags lineFlags,
                                                              RGBColor fg,
                                                              RGBColor bg,
                                                              Color ul,
@@ -162,6 +164,7 @@ RenderCell RenderBufferBuilder<Cell>::makeRenderCellExplicit(ColorPalette const&
     renderCell.attributes.foregroundColor = fg;
     renderCell.attributes.decorationColor = CellUtil::makeUnderlineColor(colorPalette, fg, ul, flags);
     renderCell.attributes.flags = flags;
+    renderCell.attributes.lineFlags = lineFlags;
     renderCell.position.line = line;
     renderCell.position.column = column;
     renderCell.width = unbox<uint8_t>(width);
@@ -173,6 +176,7 @@ template <CellConcept Cell>
 RenderCell RenderBufferBuilder<Cell>::makeRenderCellExplicit(ColorPalette const& colorPalette,
                                                              char32_t codepoint,
                                                              CellFlags flags,
+                                                             LineFlags lineFlags,
                                                              RGBColor fg,
                                                              RGBColor bg,
                                                              Color ul,
@@ -184,6 +188,7 @@ RenderCell RenderBufferBuilder<Cell>::makeRenderCellExplicit(ColorPalette const&
     renderCell.attributes.foregroundColor = fg;
     renderCell.attributes.decorationColor = CellUtil::makeUnderlineColor(colorPalette, fg, ul, flags);
     renderCell.attributes.flags = flags;
+    renderCell.attributes.lineFlags = lineFlags;
     renderCell.position.line = line;
     renderCell.position.column = column;
     renderCell.width = 1;
@@ -196,6 +201,7 @@ template <CellConcept Cell>
 RenderCell RenderBufferBuilder<Cell>::makeRenderCell(ColorPalette const& colorPalette,
                                                      HyperlinkStorage const& hyperlinks,
                                                      Cell const& screenCell,
+                                                     LineFlags lineFlags,
                                                      RGBColor fg,
                                                      RGBColor bg,
                                                      LineOffset line,
@@ -206,6 +212,7 @@ RenderCell RenderBufferBuilder<Cell>::makeRenderCell(ColorPalette const& colorPa
     renderCell.attributes.foregroundColor = fg;
     renderCell.attributes.decorationColor = CellUtil::makeUnderlineColor(colorPalette, fg, screenCell);
     renderCell.attributes.flags = screenCell.flags();
+    renderCell.attributes.lineFlags = lineFlags;
     renderCell.position.line = line;
     renderCell.position.column = column;
     renderCell.width = screenCell.width();
@@ -250,9 +257,10 @@ RGBColorPair RenderBufferBuilder<Cell>::makeColorsForCell(CellLocation gridPosit
     // clang-format on
 
     auto const selected =
-        _includeSelection && _terminal->isSelected(CellLocation { gridPosition.line, gridPosition.column });
+        _includeSelection
+        && _terminal->isSelected(CellLocation { .line = gridPosition.line, .column = gridPosition.column });
     auto const highlighted =
-        _terminal->isHighlighted(CellLocation { gridPosition.line, gridPosition.column });
+        _terminal->isHighlighted(CellLocation { .line = gridPosition.line, .column = gridPosition.column });
     auto const blink = _terminal->blinkState();
     auto const rapidBlink = _terminal->rapidBlinkState();
 
@@ -283,6 +291,7 @@ RenderAttributes RenderBufferBuilder<Cell>::createRenderAttributes(
     renderAttributes.decorationColor = CellUtil::makeUnderlineColor(
         _terminal->colorPalette(), fg, graphicsAttributes.underlineColor, graphicsAttributes.flags);
     renderAttributes.flags = graphicsAttributes.flags;
+    renderAttributes.lineFlags = _currentLineFlags;
     return renderAttributes;
 }
 
@@ -290,7 +299,7 @@ template <CellConcept Cell>
 RenderLine RenderBufferBuilder<Cell>::createRenderLine(TrivialLineBuffer const& lineBuffer,
                                                        LineOffset lineOffset) const
 {
-    auto const pos = CellLocation { lineOffset, ColumnOffset(0) };
+    auto const pos = CellLocation { .line = lineOffset, .column = ColumnOffset(0) };
     auto const gridPosition = _terminal->viewport().translateScreenToGridCoordinate(pos);
     auto renderLine = RenderLine {};
     renderLine.lineOffset = lineOffset;
@@ -299,6 +308,7 @@ RenderLine RenderBufferBuilder<Cell>::createRenderLine(TrivialLineBuffer const&
     renderLine.text = lineBuffer.text.view();
     renderLine.textAttributes = createRenderAttributes(gridPosition, lineBuffer.textAttributes);
     renderLine.fillAttributes = createRenderAttributes(gridPosition, lineBuffer.fillAttributes);
+    renderLine.flags = _currentLineFlags;
 
     return renderLine;
 }
@@ -320,18 +330,22 @@ bool RenderBufferBuilder<Cell>::gridLineContainsCursor(LineOffset lineOffset) co
 }
 
 template <CellConcept Cell>
-void RenderBufferBuilder<Cell>::renderTrivialLine(TrivialLineBuffer const& lineBuffer, LineOffset lineOffset)
+void RenderBufferBuilder<Cell>::renderTrivialLine(TrivialLineBuffer const& lineBuffer,
+                                                  LineOffset lineOffset,
+                                                  LineFlags flags)
 {
     // if (lineBuffer.text.size())
-    //     std::cout << std::format("Rendering trivial line {:2} 0..{}/{} ({} bytes): \"{}\"\n",
+    //     std::cout << std::format("Rendering trivial line {:2} 0..{}/{} ({} bytes): \"{}\" (flags: {})\n",
     //                lineOffset.value,
     //                lineBuffer.usedColumns,
     //                lineBuffer.displayWidth,
     //                lineBuffer.text.size(),
-    //                lineBuffer.text.view());
+    //                lineBuffer.text.view(),
+    //                flags);
 
     // No need to call isCursorLine(lineOffset) because lines containing a cursor are always inflated.
     _useCursorlineColoring = false;
+    _currentLineFlags = flags;
 
     auto const frontIndex = _output->cells.size();
 
@@ -361,7 +375,7 @@ void RenderBufferBuilder<Cell>::renderTrivialLine(TrivialLineBuffer const& lineB
 
     // render text
     _searchPatternOffset = 0;
-    renderUtf8Text(CellLocation { lineOffset, ColumnOffset(0) },
+    renderUtf8Text(CellLocation { .line = lineOffset, .column = ColumnOffset(0) },
                    lineBuffer.textAttributes,
                    lineBuffer.text.view(),
                    true);
@@ -369,18 +383,20 @@ void RenderBufferBuilder<Cell>::renderTrivialLine(TrivialLineBuffer const& lineB
     // {{{ fill the remaining empty cells
     for (auto columnOffset = textMargin; columnOffset < pageColumnsEnd; ++columnOffset)
     {
-        auto const pos = CellLocation { lineOffset, columnOffset };
+        auto const pos = CellLocation { .line = lineOffset, .column = columnOffset };
         auto const gridPosition = _terminal->viewport().translateScreenToGridCoordinate(pos);
         auto renderAttributes = createRenderAttributes(gridPosition, lineBuffer.fillAttributes);
+        auto const scale = _currentLineFlags.test(LineFlag::DoubleWidth) ? 2 : 1;
 
         _output->cells.emplace_back(makeRenderCellExplicit(_terminal->colorPalette(),
                                                            char32_t { 0 },
                                                            lineBuffer.fillAttributes.flags,
+                                                           _currentLineFlags,
                                                            renderAttributes.foregroundColor,
                                                            renderAttributes.backgroundColor,
                                                            lineBuffer.fillAttributes.underlineColor,
                                                            _baseLine + lineOffset,
-                                                           columnOffset));
+                                                           columnOffset * scale));
     }
     // }}}
 
@@ -431,8 +447,8 @@ void RenderBufferBuilder<Cell>::matchSearchPattern(T const& cellText)
 
     auto const isFocusedMatch =
         CellLocationRange {
-            _output->cells[offsetIntoFront].position,
-            _output->cells.back().position,
+            .first = _output->cells[offsetIntoFront].position,
+            .second = _output->cells.back().position,
         }
             .contains(
                 _terminal->viewport().translateGridToScreenCoordinate(_terminal->normalModeCursorPosition()));
@@ -458,8 +474,8 @@ void RenderBufferBuilder<Cell>::matchSearchPattern(T const& cellText)
     for (size_t i = offsetIntoFront; i < _output->cells.size(); ++i)
     {
         auto& cellAttributes = _output->cells[i].attributes;
-        auto const actualColors =
-            RGBColorPair { cellAttributes.foregroundColor, cellAttributes.backgroundColor };
+        auto const actualColors = RGBColorPair { .foreground = cellAttributes.foregroundColor,
+                                                 .background = cellAttributes.backgroundColor };
         auto const searchMatchColors = makeRGBColorPair(actualColors, highlightColors);
 
         cellAttributes.backgroundColor = searchMatchColors.background;
@@ -469,9 +485,10 @@ void RenderBufferBuilder<Cell>::matchSearchPattern(T const& cellText)
 }
 
 template <CellConcept Cell>
-void RenderBufferBuilder<Cell>::startLine(LineOffset line) noexcept
+void RenderBufferBuilder<Cell>::startLine(LineOffset line, LineFlags flags) noexcept
 {
     _lineNr = line;
+    _currentLineFlags = flags;
     _prevWidth = 0;
     _prevHasCursor = false;
 
@@ -484,7 +501,8 @@ bool RenderBufferBuilder<Cell>::isCursorLine(LineOffset line) const noexcept
     return _terminal->inputHandler().mode() != ViMode::Insert && _cursorPosition
            && line
                   == _terminal->viewport()
-                         .translateGridToScreenCoordinate(CellLocation { _cursorPosition->line, {} })
+                         .translateGridToScreenCoordinate(
+                             CellLocation { .line = _cursorPosition->line, .column = {} })
                          .line;
 }
 
@@ -521,16 +539,19 @@ ColumnCount RenderBufferBuilder<Cell>::renderUtf8Text(CellLocation screenPositio
         //            unicode::convert_to<char>(u32string_view(graphemeCluster)).size(),
         //            unicode::convert_to<char>(u32string_view(graphemeCluster)));
 
-        _output->cells.emplace_back(
-            makeRenderCellExplicit(_terminal->colorPalette(),
-                                   graphemeCluster,
-                                   width,
-                                   textAttributes.flags,
-                                   fg,
-                                   bg,
-                                   textAttributes.underlineColor,
-                                   _baseLine + screenPosition.line,
-                                   screenPosition.column + ColumnOffset::cast_from(columnCountRendered)));
+        auto const scale = _currentLineFlags.test(LineFlag::DoubleWidth) ? 2 : 1;
+
+        _output->cells.emplace_back(makeRenderCellExplicit(
+            _terminal->colorPalette(),
+            graphemeCluster,
+            width,
+            textAttributes.flags,
+            _currentLineFlags,
+            fg,
+            bg,
+            textAttributes.underlineColor,
+            _baseLine + screenPosition.line,
+            (screenPosition.column + ColumnOffset::cast_from(columnCountRendered)) * scale));
 
         // Span filling cells for preciding wide glyphs to get the background color properly painted.
         for (auto i = ColumnCount(1); i < width; ++i)
@@ -540,11 +561,12 @@ ColumnCount RenderBufferBuilder<Cell>::renderUtf8Text(CellLocation screenPositio
                 U" ", // {}
                 ColumnCount(1),
                 textAttributes.flags,
+                _currentLineFlags,
                 fg,
                 bg,
                 textAttributes.underlineColor,
                 _baseLine + screenPosition.line,
-                screenPosition.column + ColumnOffset::cast_from(columnCountRendered + i)));
+                (screenPosition.column + ColumnOffset::cast_from(columnCountRendered + i)) * scale));
         }
 
         columnCountRendered += ColumnCount::cast_from(width);
@@ -596,20 +618,28 @@ bool RenderBufferBuilder<Cell>::tryRenderInputMethodEditor(CellLocation screenPo
 template <CellConcept Cell>
 void RenderBufferBuilder<Cell>::renderCell(Cell const& screenCell, LineOffset line, ColumnOffset column)
 {
-    auto const screenPosition = CellLocation { line, column };
+    auto const screenPosition = CellLocation { .line = line, .column = column };
     auto const gridPosition = _terminal->viewport().translateScreenToGridCoordinate(screenPosition);
 
     if (tryRenderInputMethodEditor(screenPosition, gridPosition))
         return;
 
-    auto /*const*/ [fg, bg] = makeColorsForCell(
+    auto const [fg, bg] = makeColorsForCell(
         gridPosition, screenCell.flags(), screenCell.foregroundColor(), screenCell.backgroundColor());
 
     _prevWidth = screenCell.width();
     _prevHasCursor = _cursorPosition && gridPosition == *_cursorPosition;
 
-    _output->cells.emplace_back(makeRenderCell(
-        _terminal->colorPalette(), _terminal->hyperlinks(), screenCell, fg, bg, _baseLine + line, column));
+    auto const displayColumn = _currentLineFlags.test(LineFlag::DoubleWidth) ? column * 2 : column;
+
+    _output->cells.emplace_back(makeRenderCell(_terminal->colorPalette(),
+                                               _terminal->hyperlinks(),
+                                               screenCell,
+                                               _currentLineFlags,
+                                               fg,
+                                               bg,
+                                               _baseLine + line,
+                                               displayColumn));
 
     if (column == ColumnOffset(0))
         _output->cells.back().groupStart = true;
