@@ -155,6 +155,8 @@ class Screen final: public ScreenBase, public capabilities::StaticDatabase
     using StaticDatabase::numericCapability;
     [[nodiscard]] unsigned numericCapability(capabilities::Code cap) const override;
 
+    [[nodiscard]] LineFlags lineFlags(LineOffset line) const noexcept { return _grid.lineAt(line).flags(); }
+
     // {{{ SequenceHandler overrides
     void writeText(char32_t codepoint) override;
     void writeText(std::string_view text, size_t cellCount) override;
@@ -198,6 +200,7 @@ class Screen final: public ScreenBase, public capabilities::StaticDatabase
     void clearToBeginOfScreen();
     void clearToEndOfScreen();
     void clearScreen();
+    void setMark();
 
     // DECSEL
     void selectiveEraseToBeginOfLine();
@@ -255,7 +258,6 @@ class Screen final: public ScreenBase, public capabilities::StaticDatabase
     void index();        // IND
     void reverseIndex(); // RI
 
-    void setMark();
     void setScrollSpeed(int speed);      // DECSSCLS
     void deviceStatusReport();           // DSR
     void reportCursorPosition();         // CPR
@@ -605,6 +607,8 @@ class Screen final: public ScreenBase, public capabilities::StaticDatabase
   private:
     void writeTextInternal(char32_t codepoint);
 
+    void configureCurrentLineSize(LineFlags enabled);
+
     /// Attempts to emplace the given character sequence into the current cursor position, assuming
     /// that the current line is either empty or trivial and the input character sequence is contiguous.
     ///
@@ -638,6 +642,8 @@ class Screen final: public ScreenBase, public capabilities::StaticDatabase
     [[nodiscard]] std::unique_ptr<ParserExtension> hookDECRQSS(Sequence const& seq);
     [[nodiscard]] std::unique_ptr<ParserExtension> hookXTGETTCAP(Sequence const& seq);
 
+    void processShellIntegration(Sequence const& seq);
+
     gsl::not_null<Terminal*> _terminal;
     gsl::not_null<Settings*> _settings;
     gsl::not_null<Margin*> _margin;
