@@ -128,7 +128,7 @@ class SequenceParameterBuilder
 
     void nextParameter()
     {
-        if (_currentParameter != _parameters->_values.end())
+        if (std::next(_currentParameter) != _parameters->_values.end())
         {
             ++_currentParameter;
             *_currentParameter = 0;
@@ -138,7 +138,7 @@ class SequenceParameterBuilder
 
     void nextSubParameter()
     {
-        if (_currentParameter != _parameters->_values.end())
+        if (std::next(_currentParameter) != _parameters->_values.end())
         {
             ++_currentParameter;
             *_currentParameter = 0;
@@ -148,7 +148,11 @@ class SequenceParameterBuilder
 
     constexpr void multiplyBy10AndAdd(uint8_t value) noexcept
     {
-        *_currentParameter = static_cast<uint16_t>(*_currentParameter * 10 + value);
+        unsigned const newValue = (*_currentParameter * 10) + value;
+        if (newValue > 0xFFFF)
+            *_currentParameter = 0xFFFF;
+        else
+            *_currentParameter = static_cast<uint16_t>(newValue);
     }
 
     constexpr void apply(uint16_t value) noexcept
@@ -194,7 +198,9 @@ class SequenceParameterBuilder
 class Sequence
 {
   public:
-    size_t constexpr static MaxOscLength = 512; // NOLINT(readability-identifier-naming)
+    // Make maximum size 50 kB since we need to support adding to the clipboard
+    // and the clipboard can contain large amounts of text.
+    size_t constexpr static MaxOscLength = 1024 * 50; // NOLINT(readability-identifier-naming)
 
     using Parameter = uint16_t;
     using Intermediaries = std::string;
@@ -266,18 +272,22 @@ class Sequence
         switch (_category)
         {
             case FunctionCategory::OSC:
-                return FunctionSelector {
-                    _category, 0, static_cast<int>(parameterCount() ? param(0) : 0), 0, 0
-                };
+                return FunctionSelector { .category = _category,
+                                          .leader = 0,
+                                          .argc = static_cast<int>(parameterCount() ? param(0) : 0),
+                                          .intermediate = 0,
+                                          .finalSymbol = 0 };
             default: {
                 // Only support CSI sequences with 0 or 1 intermediate characters.
                 char const intermediate = _intermediateCharacters.size() == 1
                                               ? static_cast<char>(_intermediateCharacters[0])
                                               : char {};
 
-                return FunctionSelector {
-                    _category, _leaderSymbol, static_cast<int>(parameterCount()), intermediate, _finalChar
-                };
+                return FunctionSelector { .category = _category,
+                                          .leader = _leaderSymbol,
+                                          .argc = static_cast<int>(parameterCount()),
+                                          .intermediate = intermediate,
+                                          .finalSymbol = _finalChar };
             }
         }
     }
