@@ -8,13 +8,11 @@
 
 #include <libunicode/convert.h>
 
-#include <range/v3/range/conversion.hpp>
-#include <range/v3/view/join.hpp>
-#include <range/v3/view/transform.hpp>
-
 #include <chrono>
 #include <cstdio>
 #include <format>
+#include <optional>
+#include <ranges>
 
 using namespace std::string_view_literals;
 
@@ -156,6 +154,7 @@ std::optional<StatusLineDefinitions::Item> makeStatusLineItem(
             styles,
             activeColor,
             activeBackground,
+            std::nullopt, // separator
         };
     }
 
@@ -309,9 +308,16 @@ struct VTSerializer
         auto const cellText = vt.currentScreen().cellTextAt(currentMousePosition);
         auto const cellText32 = unicode::convert_to<char32_t>(std::string_view(cellText));
 
-        return ranges::views::transform(
-                   cellText32, [](char32_t ch) { return std::format("U+{:04X}", static_cast<uint32_t>(ch)); })
-               | ranges::views::join(" ") | ranges::to<std::string>;
+        std::string result;
+        bool first = true;
+        for (char32_t ch: cellText32)
+        {
+            if (!first)
+                result += " ";
+            result += std::format("U+{:04X}", static_cast<uint32_t>(ch));
+            first = false;
+        }
+        return result;
     }
 
     std::string visit(StatusLineDefinitions::CellTextUtf8 const&)
@@ -404,6 +410,9 @@ struct VTSerializer
             return std::format("Search: {}█",
                                unicode::convert_to<char>(std::u32string_view(vt.search().pattern)));
 
+        if (vt.inputHandler().isEditingPrompt())
+            return std::format("{}{}█", vt.prompt().prompt, vt.prompt().text);
+
         return {};
     }
 
@@ -443,10 +452,10 @@ struct VTSerializer
         auto const tabsInfo = vt.guiTabsInfoForStatusLine();
 
         std::string fragment;
-        for (const auto position: std::views::iota(1u, tabsInfo.tabCount + 1))
+        for (const auto position: std::views::iota(1u, tabsInfo.tabs.size() + 1))
         {
             if (!fragment.empty())
-                fragment += ' ';
+                fragment += tabs.separator.value_or("|");
 
             auto const isActivePosition = position == tabsInfo.activeTabPosition;
             auto const activePositionStylized =
@@ -459,7 +468,10 @@ struct VTSerializer
                 fragment += makeBackgroundColor(tabs.activeBackground);
             }
 
-            fragment += std::to_string(position);
+            if (tabsInfo.tabs[position - 1].name)
+                fragment += tabsInfo.tabs[position - 1].name.value();
+            else
+                fragment += std::to_string(position);
 
             if (activePositionStylized)
                 fragment += SGRRESTORE();
