@@ -157,14 +157,13 @@ Terminal::Terminal(Events& eventListener,
     _effectiveImageCanvasSize { _settings.maxImageSize },
     _sixelColorPalette { std::make_shared<SixelColorPalette>(_maxSixelColorRegisters,
                                                              _maxSixelColorRegisters) },
-    _imagePool { [this](Image const* image) {
-        discardImage(*image);
-    } },
+    _imagePool { [this](Image const* image) { discardImage(*image); } },
     _hyperlinks { .cache = HyperlinkCache { 1024 } },
     _sequenceBuilder { ModeDependantSequenceHandler { *this }, TerminalInstructionCounter { *this } },
     _parser { std::ref(_sequenceBuilder) },
     _viCommands { *this },
-    _inputHandler { _viCommands, ViMode::Insert }
+    _inputHandler { _viCommands, ViMode::Insert },
+    _shellIntegration { std::make_unique<NullShellIntegration>() }
 {
     _savedColorPalettes.reserve(MaxColorPaletteSaveStackSize);
 
@@ -681,7 +680,7 @@ Handled Terminal::sendMousePressEvent(Modifiers modifiers,
         && _inputGenerator.generateMousePress(
             modifiers, button, _currentMousePosition, pixelPosition, uiHandledHint);
 
-    // TODO: Ctrl+(Left)Click's should still be catched by the terminal iff there's a hyperlink
+    // TODO: Ctrl+(Left)Click's should still be caught by the terminal iff there's a hyperlink
     // under the current position
     flushInput();
     return Handled { eventHandledByApp && !isModeEnabled(DECMode::MousePassiveTracking) };
@@ -1567,6 +1566,16 @@ std::string const& Terminal::windowTitle() const noexcept
     return _windowTitle;
 }
 
+void Terminal::requestTabName()
+{
+    inputHandler().setTabName([&](std::string name) { _tabName = std::move(name); });
+}
+
+std::optional<std::string> Terminal::tabName() const noexcept
+{
+    return _tabName;
+}
+
 void Terminal::saveWindowTitle()
 {
     _savedWindowTitles.push(_windowTitle);
@@ -1612,12 +1621,6 @@ void Terminal::setMode(AnsiMode mode, bool enable)
 // NOLINTNEXTLINE(readability-function-cognitive-complexity)
 void Terminal::setMode(DECMode mode, bool enable)
 {
-    if (!isValidDECMode(static_cast<unsigned int>(mode)))
-    {
-        errorLog()("Attempt to {} invalid DEC mode: {}", mode, enable ? "set" : "reset");
-        return;
-    }
-
     if (_modes.frozen(mode))
     {
         if (_modes.enabled(mode) != enable)
@@ -2195,6 +2198,16 @@ bool Terminal::setNewSearchTerm(std::u32string text, bool initiatedByDoubleClick
     return true;
 }
 
+void Terminal::setPrompt(std::string prompt)
+{
+    _prompt.prompt = std::move(prompt);
+}
+
+void Terminal::setPromptText(std::string text)
+{
+    _prompt.text = std::move(text);
+}
+
 optional<CellLocation> Terminal::searchReverse(u32string text, CellLocation searchPosition)
 {
     if (!setNewSearchTerm(std::move(text), false))
