@@ -12,6 +12,7 @@
 #include <vtbackend/Sequence.h>
 #include <vtbackend/SequenceBuilder.h>
 #include <vtbackend/Settings.h>
+#include <vtbackend/ShellIntegration.h>
 #include <vtbackend/StatusLineBuilder.h>
 #include <vtbackend/ViCommands.h>
 #include <vtbackend/ViInputHandler.h>
@@ -42,6 +43,7 @@
 #include <mutex>
 #include <stack>
 #include <string_view>
+#include <utility>
 
 namespace vtbackend
 {
@@ -84,14 +86,11 @@ class Modes
 
     [[nodiscard]] bool frozen(DECMode mode) const noexcept
     {
-        assert(isValidDECMode(static_cast<unsigned int>(mode)));
         return _decFrozen.test(static_cast<size_t>(mode));
     }
 
     void freeze(DECMode mode)
     {
-        assert(isValidDECMode(static_cast<unsigned int>(mode)));
-
         if (mode == DECMode::BatchedRendering)
         {
             errorLog()("Attempt to freeze batched rendering. Ignoring.");
@@ -104,7 +103,6 @@ class Modes
 
     void unfreeze(DECMode mode)
     {
-        assert(isValidDECMode(static_cast<unsigned int>(mode)));
         _decFrozen.set(static_cast<size_t>(mode), false);
         terminalLog()("Unfreezing permanently-{} DEC mode {}.", mode, enabled(mode));
     }
@@ -143,6 +141,12 @@ struct Search
     bool initiatedByDoubleClick = false;
 };
 
+struct Prompt
+{
+    std::string prompt;
+    std::string text;
+};
+
 // Mandates what execution mode the terminal will take to process VT sequences.
 //
 enum class ExecutionMode : uint8_t
@@ -209,7 +213,13 @@ class TraceHandler: public SequenceHandler
 
 struct TabsInfo
 {
-    size_t tabCount = 1;
+    struct Tab
+    {
+        std::optional<std::string> name;
+        Color color;
+    };
+
+    std::vector<Tab> tabs;
     size_t activeTabPosition = 1;
 };
 
@@ -246,6 +256,7 @@ class Terminal
         virtual void requestWindowResize(Width, Height) {}
         virtual void requestShowHostWritableStatusLine() {}
         virtual void setWindowTitle(std::string_view /*title*/) {}
+        virtual void setTabName(std::string_view /*title*/) {}
         virtual void setTerminalProfile(std::string const& /*configProfileName*/) {}
         virtual void discardImage(Image const&) {}
         virtual void inputModeChanged(ViMode /*mode*/) {}
@@ -276,6 +287,7 @@ class Terminal
         void requestWindowResize(Width, Height) override {}
         void requestShowHostWritableStatusLine() override {}
         void setWindowTitle(std::string_view /*title*/) override {}
+        void setTabName(std::string_view /*title*/) override {}
         void setTerminalProfile(std::string const& /*configProfileName*/) override {}
         void discardImage(Image const&) override {}
         void inputModeChanged(ViMode /*mode*/) override {}
@@ -357,7 +369,7 @@ class Terminal
 
     [[nodiscard]] CellLocation clampToScreen(CellLocation coord) const noexcept
     {
-        return { clampedLine(coord.line), clampedColumn(coord.column) };
+        return { .line = clampedLine(coord.line), .column = clampedColumn(coord.column) };
     }
 
     // Tests if given coordinate is within the visible screen area.
@@ -551,6 +563,13 @@ class Terminal
     void popColorPalette(size_t slot);
     void reportColorPaletteStack();
 
+    [[nodiscard]] ShellIntegration& shellIntegration() noexcept { return *_shellIntegration; }
+    [[nodiscard]] ShellIntegration const& shellIntegration() const noexcept { return *_shellIntegration; }
+    void setShellIntegration(std::unique_ptr<ShellIntegration> newShellIntegration)
+    {
+        _shellIntegration = std::move(newShellIntegration);
+    }
+
     [[nodiscard]] ScreenBase& currentScreen() noexcept { return *_currentScreen; }
     [[nodiscard]] ScreenBase const& currentScreen() const noexcept { return *_currentScreen; }
 
@@ -797,10 +816,14 @@ class Terminal
     void setMouseTransport(MouseTransport transport);
     void setMouseWheelMode(InputGenerator::MouseWheelMode mode);
     void setWindowTitle(std::string_view title);
+    void setTabName(std::string_view title);
     [[nodiscard]] std::string const& windowTitle() const noexcept;
+    [[nodiscard]] std::optional<std::string> tabName() const noexcept;
     [[nodiscard]] bool focused() const noexcept { return _focused; }
     [[nodiscard]] Search& search() noexcept { return _search; }
     [[nodiscard]] Search const& search() const noexcept { return _search; }
+    [[nodiscard]] Prompt& prompt() noexcept { return _prompt; }
+    [[nodiscard]] Prompt const& prompt() const noexcept { return _prompt; }
     void saveWindowTitle();
     void restoreWindowTitle();
     void setTerminalProfile(std::string const& configProfileName);
@@ -882,6 +905,9 @@ class Terminal
     bool setNewSearchTerm(std::u32string text, bool initiatedByDoubleClick);
     void clearSearch();
 
+    void setPrompt(std::string prompt);
+    void setPromptText(std::string text);
+
     // Tests if the grid cell at the given location does contain a word delimiter.
     [[nodiscard]] bool wordDelimited(CellLocation position) const noexcept;
     [[nodiscard]] bool wordDelimited(CellLocation position,
@@ -966,7 +992,10 @@ class Terminal
     void resetStatusLineDefinition();
 
     TabsInfo guiTabsInfoForStatusLine() const noexcept { return _guiTabInfoForStatusLine; }
-    void setGuiTabInfoForStatusLine(TabsInfo info) { _guiTabInfoForStatusLine = info; }
+    void setGuiTabInfoForStatusLine(TabsInfo&& info) { _guiTabInfoForStatusLine = std::move(info); }
+
+    TabsNamingMode getTabsNamingMode() const noexcept { return _settings.tabNamingMode; }
+    void requestTabName();
 
   private:
     void mainLoop();
@@ -1048,7 +1077,7 @@ class Terminal
     std::unique_ptr<vtpty::Pty> _pty;
     // }}}
 
-    // {{{ mouse related state (helpers for detecting double/tripple clicks)
+    // {{{ mouse related state (helpers for detecting double/triple clicks)
     std::chrono::steady_clock::time_point _lastClick {};
     unsigned int _speedClicks = 0;
     vtbackend::CellLocation _currentMousePosition {}; // current mouse position
@@ -1065,8 +1094,8 @@ class Terminal
         bool state = false;
         std::chrono::milliseconds interval {};
     };
-    mutable BlinkerState _slowBlinker { false, std::chrono::milliseconds { 500 } };
-    mutable BlinkerState _rapidBlinker { false, std::chrono::milliseconds { 300 } };
+    mutable BlinkerState _slowBlinker { .state = false, .interval = std::chrono::milliseconds { 500 } };
+    mutable BlinkerState _rapidBlinker { .state = false, .interval = std::chrono::milliseconds { 300 } };
     mutable std::chrono::steady_clock::time_point _lastBlink;
     mutable std::chrono::steady_clock::time_point _lastRapidBlink;
     // }}}
@@ -1080,7 +1109,9 @@ class Terminal
     Viewport _viewport;
     StatusLineDefinition _indicatorStatusLineDefinition;
 
+    // {{{ tabs info
     TabsInfo _guiTabInfoForStatusLine;
+    // }}}
 
     // {{{ selection states
     std::unique_ptr<Selection> _selection;
@@ -1147,6 +1178,7 @@ class Terminal
     ActiveStatusDisplay _activeStatusDisplay = ActiveStatusDisplay::Main;
 
     Search _search;
+    Prompt _prompt;
 
     CursorDisplay _cursorDisplay = CursorDisplay::Steady;
     CursorShape _cursorShape = CursorShape::Block;
@@ -1165,6 +1197,8 @@ class Terminal
     std::string _windowTitle {};
     std::stack<std::string> _savedWindowTitles {};
 
+    std::optional<std::string> _tabName {};
+
     struct ModeDependantSequenceHandler
     {
         Terminal& terminal;
@@ -1205,6 +1239,8 @@ class Terminal
 
     ViCommands _viCommands;
     ViInputHandler _inputHandler;
+
+    std::unique_ptr<ShellIntegration> _shellIntegration;
 };
 
 } // namespace vtbackend
