@@ -126,9 +126,9 @@ namespace
         for (auto const& pair: Pairs)
         {
             if (input == pair.first)
-                return { { pair.second, true } };
+                return { { static_cast<char>(pair.second), true } };
             if (input == pair.second)
-                return { { pair.first, false } };
+                return { { static_cast<char>(pair.first), false } };
         }
 
         return std::nullopt;
@@ -192,6 +192,28 @@ void ViCommands::searchCancel()
     _terminal->screenUpdated();
 }
 
+void ViCommands::promptStart(std::string const& query)
+{
+    _terminal->setPrompt(query);
+    _terminal->screenUpdated();
+}
+
+void ViCommands::promptDone()
+{
+    _terminal->screenUpdated();
+}
+
+void ViCommands::promptCancel()
+{
+    _terminal->screenUpdated();
+}
+
+void ViCommands::updatePromptText(std::string const& text)
+{
+    _terminal->setPromptText(text);
+    _terminal->screenUpdated();
+}
+
 bool ViCommands::jumpToNextMatch(unsigned count)
 {
     for (unsigned i = 0; i < count; ++i)
@@ -230,9 +252,7 @@ void ViCommands::updateSearchTerm(std::u32string const& text)
 
 void ViCommands::modeChanged(ViMode mode)
 {
-    auto _ = crispy::finally { [this, mode]() {
-        _lastMode = mode;
-    } };
+    auto _ = crispy::finally { [this, mode]() { _lastMode = mode; } };
 
     inputLog()("mode changed to {}\n", mode);
 
@@ -327,10 +347,8 @@ void ViCommands::executeYank(ViMotion motion, unsigned count)
             break;
         }
         default: {
-            auto const [from, to] = translateToCellRange(motion, count);
-            // motion is inclusive but for yank we want to exclude the last cell which is the first cell of
-            // the next word
-            executeYank(from, { .line = to.line, .column = to.column - 1 });
+            auto const [from, to] = translateToCellRange(motion, count).ordered();
+            executeYank(from, to);
         }
         break;
     }
@@ -1085,6 +1103,11 @@ CellLocation ViCommands::translateToCellLocationAndRecord(ViMotion motion, unsig
         case ViMotion::JumpToLastJumpPoint: return _jumpHistory.jumpToLast(cursorPosition);
         case ViMotion::JumpToMarkBackward: return _jumpHistory.jumpToMarkBackward(cursorPosition);
         case ViMotion::JumpToMarkForward: return _jumpHistory.jumpToMarkForward(cursorPosition);
+        case ViMotion::CenterCursor: {
+            _terminal->viewport().makeVisibleWithinSafeArea(unbox<LineOffset>(cursorPosition.line),
+                                                            LineCount(_terminal->pageSize().lines / 2));
+            return cursorPosition;
+        }
     }
     crispy::unreachable();
 }
