@@ -100,11 +100,15 @@ enum class ViMotion : uint8_t
     ToCharLeft,            // F {char}
     RepeatCharMove,        // ;
     RepeatCharMoveReverse, // ,
+    JumpToLastJumpPoint,   // '' or `` (jump to last jump)
+    JumpToMarkBackward,    // <C-O>
+    JumpToMarkForward,     // <C-I>
+    CenterCursor,          // zz
 };
 
 enum class ViOperator : uint8_t
 {
-    MoveCursor,
+    MoveCursor = 0,
     Yank = 'y',
     Paste = 'p',
     PasteStripped = 'P',
@@ -140,7 +144,7 @@ struct RectangularHighlight { CellLocation from; CellLocation to; };
 
 using HighlightRange = std::variant<LinearHighlight, RectangularHighlight>;
 
-enum class SearchEditMode : uint8_t
+enum class PromptMode : uint8_t
 {
     Disabled,
     Enabled,
@@ -173,6 +177,11 @@ class ViInputHandler: public InputHandler
         virtual void searchCancel() = 0;
         virtual void updateSearchTerm(std::u32string const& text) = 0;
 
+        virtual void promptStart(std::string const& query) = 0;
+        virtual void promptDone() = 0;
+        virtual void promptCancel() = 0;
+        virtual void updatePromptText(std::string const& text) = 0;
+
         virtual void scrollViewport(ScrollOffset delta) = 0;
 
         // Starts searching for the word under the cursor position in reverse order.
@@ -208,14 +217,31 @@ class ViInputHandler: public InputHandler
         crispy::unreachable();
     }
 
-    [[nodiscard]] bool isEditingSearch() const noexcept
-    {
-        return _searchEditMode != SearchEditMode::Disabled;
-    }
+    [[nodiscard]] bool isEditingSearch() const noexcept { return _searchEditMode != PromptMode::Disabled; }
+    [[nodiscard]] bool isEditingPrompt() const noexcept { return _promptEditMode != PromptMode::Disabled; }
 
     void startSearchExternally();
 
+    void setTabName(auto&& callback)
+    {
+        _promptText.clear();
+        _executor->promptStart("Tab name: ");
+
+        if (_viMode != ViMode::Insert)
+            _promptEditMode = PromptMode::Enabled;
+        else
+        {
+            _promptEditMode = PromptMode::ExternallyEnabled;
+            setMode(ViMode::Normal);
+            // ^^^ So that we can see the statusline (which contains the search edit field),
+            // AND it's weird to be in insert mode while typing in the prompt term anyways.
+        }
+
+        _setTabNameCallback = callback;
+    }
+
     void setSearchModeSwitch(bool enabled);
+    void clearSearch();
 
   private:
     enum class ModeSelect : uint8_t
@@ -245,14 +271,17 @@ class ViInputHandler: public InputHandler
     bool parseCount(char32_t ch, Modifiers modifiers);
     bool parseTextObject(char32_t ch, Modifiers modifiers);
     Handled handleSearchEditor(char32_t ch, Modifiers modifiers);
+    Handled handlePromptEditor(char32_t ch, Modifiers modifiers);
     Handled handleModeSwitches(char32_t ch, Modifiers modifiers);
     void startSearch();
 
     ViMode _viMode = ViMode::Normal;
 
-    SearchEditMode _searchEditMode = SearchEditMode::Disabled;
+    PromptMode _searchEditMode = PromptMode::Disabled;
+    PromptMode _promptEditMode = PromptMode::Disabled;
     bool _searchExternallyActivated = false;
     std::u32string _searchTerm;
+    std::string _promptText;
 
     std::string _pendingInput;
     CommandHandlerMap _normalMode;
@@ -260,6 +289,7 @@ class ViInputHandler: public InputHandler
     unsigned _count = 0;
     char32_t _lastChar = 0;
     gsl::not_null<Executor*> _executor;
+    std::optional<std::function<void(std::string)>> _setTabNameCallback { std::nullopt };
 };
 
 } // namespace vtbackend
@@ -370,12 +400,16 @@ struct std::formatter<vtbackend::ViMotion>: formatter<std::string_view>
             case ViMotion::ToCharLeft: name = "ToCharLeft"; break;
             case ViMotion::RepeatCharMove: name = "RepeatCharMove"; break;
             case ViMotion::RepeatCharMoveReverse: name = "RepeatCharMoveReverse"; break;
+            case ViMotion::JumpToLastJumpPoint: name = "JumpToLastJumpPoint"; break;
+            case ViMotion::JumpToMarkBackward: name = "JumpToMarkUp"; break;
+            case ViMotion::JumpToMarkForward: name = "JumpToMarkDown"; break;
             case ViMotion::GlobalCurlyCloseUp: name = "GlobalCurlyCloseUp"; break;
             case ViMotion::GlobalCurlyCloseDown: name = "GlobalCurlyCloseDown"; break;
             case ViMotion::GlobalCurlyOpenUp: name = "GlobalCurlyOpenUp"; break;
             case ViMotion::GlobalCurlyOpenDown: name = "GlobalCurlyOpenDown"; break;
             case ViMotion::LineMarkUp: name = "LineMarkUp"; break;
             case ViMotion::LineMarkDown: name = "LineMarkDown"; break;
+            case ViMotion::CenterCursor: name = "CenterCursor"; break;
         }
         return formatter<string_view>::format(name, ctx);
     }
