@@ -6,6 +6,7 @@
 
 #include <cstdint>
 #include <limits>
+#include <optional>
 #include <ostream>
 #include <string>
 #include <variant>
@@ -152,33 +153,36 @@ inline std::ostream& operator<<(std::ostream& os, CellLocation coord)
 
 inline CellLocation operator+(CellLocation a, CellLocation b) noexcept
 {
-    return { a.line + b.line, a.column + b.column };
+    return { .line = a.line + b.line, .column = a.column + b.column };
 }
 
 constexpr CellLocation operator+(CellLocation c, LineOffset y) noexcept
 {
-    return CellLocation { c.line + y, c.column };
+    return CellLocation { .line = c.line + y, .column = c.column };
 }
 
 constexpr CellLocation operator-(CellLocation c, LineOffset y) noexcept
 {
-    return CellLocation { c.line - y, c.column };
+    return CellLocation { .line = c.line - y, .column = c.column };
 }
 
 constexpr CellLocation operator+(CellLocation c, ColumnOffset x) noexcept
 {
-    return CellLocation { c.line, c.column + x };
+    return CellLocation { .line = c.line, .column = c.column + x };
 }
 
 constexpr CellLocation operator-(CellLocation c, ColumnOffset x) noexcept
 {
-    return CellLocation { c.line, c.column - x };
+    return CellLocation { .line = c.line, .column = c.column - x };
 }
+
 // Constructs a top-left and bottom-right coordinate-pair from given input.
 constexpr std::pair<CellLocation, CellLocation> orderedPoints(CellLocation a, CellLocation b) noexcept
 {
-    auto const topLeft = CellLocation { std::min(a.line, b.line), std::min(a.column, b.column) };
-    auto const bottomRight = CellLocation { std::max(a.line, b.line), std::max(a.column, b.column) };
+    auto const topLeft =
+        CellLocation { .line = std::min(a.line, b.line), .column = std::min(a.column, b.column) };
+    auto const bottomRight =
+        CellLocation { .line = std::max(a.line, b.line), .column = std::max(a.column, b.column) };
     return std::pair { topLeft, bottomRight };
 }
 
@@ -194,6 +198,12 @@ struct CellLocationRange
     CellLocation first;
     CellLocation second;
 
+    [[nodiscard]] CellLocationRange ordered() const noexcept
+    {
+        auto [a, b] = orderedPoints(first, second);
+        return CellLocationRange { .first = a, .second = b };
+    }
+
     [[nodiscard]] bool contains(CellLocation location) const noexcept
     {
         switch (abs(unbox(first.line) - unbox(second.line)))
@@ -276,10 +286,10 @@ struct Rect
 
     [[nodiscard]] Rect clampTo(PageSize size) const noexcept
     {
-        return Rect { top,
-                      left,
-                      std::min(bottom, Bottom::cast_from(size.lines)),
-                      std::min(right, Right::cast_from(size.columns)) };
+        return Rect { .top = top,
+                      .left = left,
+                      .bottom = std::min(bottom, Bottom::cast_from(size.lines)),
+                      .right = std::min(right, Right::cast_from(size.columns)) };
     }
 };
 
@@ -295,11 +305,11 @@ struct PageMargin
 
 constexpr Range horizontal(PageMargin m) noexcept
 {
-    return Range { From { *m.top }, To { *m.bottom } };
+    return Range { .from = From { *m.top }, .to = To { *m.bottom } };
 }
 constexpr Range vertical(PageMargin m) noexcept
 {
-    return Range { From { *m.left }, To { *m.right } };
+    return Range { .from = From { *m.left }, .to = To { *m.right } };
 }
 
 // }}}
@@ -365,7 +375,8 @@ struct GridSize
 
         constexpr Offset makeOffset(int offset) noexcept
         {
-            return Offset { LineOffset(offset / unbox(_width)), ColumnOffset(offset % unbox(_width)) };
+            return Offset { .line = LineOffset(offset / unbox(_width)),
+                            .column = ColumnOffset(offset % unbox(_width)) };
         }
     };
 
@@ -378,7 +389,7 @@ struct GridSize
 
 constexpr CellLocation operator+(CellLocation a, GridSize::Offset b) noexcept
 {
-    return CellLocation { a.line + b.line, a.column + b.column };
+    return CellLocation { .line = a.line + b.line, .column = a.column + b.column };
 }
 
 constexpr GridSize::iterator begin(GridSize const& s) noexcept
@@ -464,7 +475,7 @@ enum class ScreenType : uint8_t
 };
 
 // TODO: Maybe make boxed.h into its own C++ github repo?
-// TODO: Differenciate Line/Column types for DECOM enabled/disabled coordinates?
+// TODO: Differentiate Line/Column types for DECOM enabled/disabled coordinates?
 //
 // Line, Column                 : respects DECOM if enabled (a.k.a. logical column)
 // PhysicalLine, PhysicalColumn : always relative to origin (top left)
@@ -568,20 +579,20 @@ enum class AnsiMode : uint8_t
 
 enum class DECMode : std::uint16_t
 {
-    UseApplicationCursorKeys,
-    DesignateCharsetUSASCII,
-    Columns132,
-    SmoothScroll,
-    ReverseVideo,
+    UseApplicationCursorKeys = 0,
+    DesignateCharsetUSASCII = 1,
+    Columns132 = 2,
+    SmoothScroll = 3,
+    ReverseVideo = 4,
 
-    MouseProtocolX10,
-    MouseProtocolNormalTracking,
-    MouseProtocolHighlightTracking,
-    MouseProtocolButtonTracking,
-    MouseProtocolAnyEventTracking,
+    MouseProtocolX10 = 5,
+    MouseProtocolNormalTracking = 6,
+    MouseProtocolHighlightTracking = 7,
+    MouseProtocolButtonTracking = 8,
+    MouseProtocolAnyEventTracking = 9,
 
-    SaveCursor,
-    ExtendedAltScreen,
+    SaveCursor = 10,
+    ExtendedAltScreen = 11,
 
     /**
      * DECOM - Origin Mode.
@@ -600,7 +611,7 @@ enum class DECMode : std::uint16_t
      * The starting point for line numbers is independent of the margins.
      * The cursor can move outside of the margins.
      */
-    Origin,
+    Origin = 12,
 
     /**
      * DECAWM - Autowrap Mode.
@@ -613,22 +624,22 @@ enum class DECMode : std::uint16_t
      *
      * Any text on the page scrolls up if the cursor is at the end of the scrolling region.
      */
-    AutoWrap,
+    AutoWrap = 13,
 
-    PrinterExtend,
-    LeftRightMargin,
+    PrinterExtend = 14,
+    LeftRightMargin = 15,
 
-    ShowToolbar,
-    BlinkingCursor,
-    VisibleCursor, // DECTCEM
-    ShowScrollbar,
-    AllowColumns80to132, // ?40
-    DebugLogging,        // ?46,
-    UseAlternateScreen,
-    BracketedPaste,
-    FocusTracking,            // 1004
-    NoSixelScrolling,         // ?80
-    UsePrivateColorRegisters, // ?1070
+    ShowToolbar = 16,
+    BlinkingCursor = 17,
+    VisibleCursor = 18, // DECTCEM
+    ShowScrollbar = 19,
+    AllowColumns80to132 = 20, // ?40
+    DebugLogging = 21,        // ?46,
+    UseAlternateScreen = 22,
+    BracketedPaste = 23,
+    FocusTracking = 24,            // 1004
+    NoSixelScrolling = 25,         // ?80
+    UsePrivateColorRegisters = 26, // ?1070
 
     // {{{ Mouse related flags
     /// extend mouse protocl encoding
@@ -739,7 +750,7 @@ constexpr bool isValidAnsiMode(unsigned int mode) noexcept
 std::string to_string(AnsiMode mode);
 std::string to_string(DECMode mode);
 
-constexpr unsigned toDECModeNum(DECMode m)
+constexpr unsigned toDECModeNum(DECMode m) noexcept
 {
     switch (m)
     {
@@ -786,53 +797,66 @@ constexpr unsigned toDECModeNum(DECMode m)
     return static_cast<unsigned>(m);
 }
 
+constexpr std::optional<DECMode> fromDECModeNum(unsigned int modeNum) noexcept
+{
+    switch (modeNum)
+    {
+        case 1: return DECMode::UseApplicationCursorKeys;
+        case 2: return DECMode::DesignateCharsetUSASCII;
+        case 3: return DECMode::Columns132;
+        case 4: return DECMode::SmoothScroll;
+        case 5: return DECMode::ReverseVideo;
+        case 6: return DECMode::Origin;
+        case 7: return DECMode::AutoWrap;
+        // TODO: Ps = 8  -> Auto-repeat Keys (DECARM), VT100.
+        case 9: return DECMode::MouseProtocolX10;
+        case 10: return DECMode::ShowToolbar;
+        case 12: return DECMode::BlinkingCursor;
+        case 19: return DECMode::PrinterExtend;
+        case 25: return DECMode::VisibleCursor;
+        case 30: return DECMode::ShowScrollbar;
+        // TODO: Ps = 3 5  -> Enable font-shifting functions (rxvt).
+        // IGNORE? Ps = 3 8  -> Enter Tektronix Mode (DECTEK), VT240, xterm.
+        case 40: return DECMode::AllowColumns80to132;
+        // IGNORE: Ps = 4 1  -> more(1) fix (see curses resource).
+        // TODO: Ps = 4 2  -> Enable National Replacement Character sets (DECNRCM), VT220.
+        // TODO: Ps = 4 4  -> Turn On Margin Bell, xterm.
+        // TODO: Ps = 4 5  -> Reverse-wraparound Mode, xterm.
+        case 46: return DECMode::DebugLogging;
+        case 47: return DECMode::UseAlternateScreen;
+        // TODO: Ps = 6 6  -> Application keypad (DECNKM), VT320.
+        // TODO: Ps = 6 7  -> Backarrow key sends backspace (DECBKM), VT340, VT420.  This sets the
+        // backarrowKey resource to "true".
+        case 69: return DECMode::LeftRightMargin;
+        case 80: return DECMode::NoSixelScrolling;
+        case 1000: return DECMode::MouseProtocolNormalTracking;
+        case 1001: return DECMode::MouseProtocolHighlightTracking;
+        case 1002: return DECMode::MouseProtocolButtonTracking;
+        case 1003: return DECMode::MouseProtocolAnyEventTracking;
+        case 1004: return DECMode::FocusTracking;
+        case 1005: return DECMode::MouseExtended;
+        case 1006: return DECMode::MouseSGR;
+        case 1007: return DECMode::MouseAlternateScroll;
+        case 1015: return DECMode::MouseURXVT;
+        case 1016: return DECMode::MouseSGRPixels;
+        case 1048: return DECMode::SaveCursor;
+        case 1049: return DECMode::ExtendedAltScreen;
+        case 1070: return DECMode::UsePrivateColorRegisters;
+        case 2004: return DECMode::BracketedPaste;
+        case 2026: return DECMode::BatchedRendering;
+        case 2027: return DECMode::Unicode;
+        case 2028: return DECMode::TextReflow;
+        case 2029: return DECMode::MousePassiveTracking;
+        case 2030: return DECMode::ReportGridCellSelection;
+        case 2031: return DECMode::ReportColorPaletteUpdated;
+        case 8452: return DECMode::SixelCursorNextToGraphic;
+        default: return std::nullopt;
+    }
+}
+
 constexpr bool isValidDECMode(unsigned int mode) noexcept
 {
-    switch (static_cast<DECMode>(mode))
-    {
-        case DECMode::UseApplicationCursorKeys:
-        case DECMode::DesignateCharsetUSASCII:
-        case DECMode::Columns132:
-        case DECMode::SmoothScroll:
-        case DECMode::ReverseVideo:
-        case DECMode::MouseProtocolX10:
-        case DECMode::MouseProtocolNormalTracking:
-        case DECMode::MouseProtocolHighlightTracking:
-        case DECMode::MouseProtocolButtonTracking:
-        case DECMode::MouseProtocolAnyEventTracking:
-        case DECMode::SaveCursor:
-        case DECMode::ExtendedAltScreen:
-        case DECMode::Origin:
-        case DECMode::AutoWrap:
-        case DECMode::PrinterExtend:
-        case DECMode::LeftRightMargin:
-        case DECMode::ShowToolbar:
-        case DECMode::BlinkingCursor:
-        case DECMode::VisibleCursor:
-        case DECMode::ShowScrollbar:
-        case DECMode::AllowColumns80to132:
-        case DECMode::DebugLogging:
-        case DECMode::UseAlternateScreen:
-        case DECMode::BracketedPaste:
-        case DECMode::FocusTracking:
-        case DECMode::NoSixelScrolling:
-        case DECMode::UsePrivateColorRegisters:
-        case DECMode::MouseExtended:
-        case DECMode::MouseSGR:
-        case DECMode::MouseURXVT:
-        case DECMode::MouseSGRPixels:
-        case DECMode::MouseAlternateScroll:
-        case DECMode::MousePassiveTracking:
-        case DECMode::ReportGridCellSelection:
-        case DECMode::ReportColorPaletteUpdated:
-        case DECMode::BatchedRendering:
-        case DECMode::Unicode:
-        case DECMode::TextReflow:
-        case DECMode::SixelCursorNextToGraphic:
-            //.
-            return true;
-    }
-    return false;
+    return fromDECModeNum(mode).has_value();
 }
 
 constexpr DynamicColorName getChangeDynamicColorCommand(unsigned value)
