@@ -17,6 +17,7 @@
 #include <stdexcept>
 #include <string>
 #include <thread>
+#include <utility>
 
 #if defined(__APPLE__) || defined(__OpenBSD__)
     #include <util.h>
@@ -60,13 +61,13 @@ namespace
     UnixPty::PtyHandles createUnixPty(PageSize const& windowSize, optional<ImageSize> pixels)
     {
         // See https://code.woboq.org/userspace/glibc/login/forkpty.c.html
-        assert(*windowSize.lines <= numeric_limits<unsigned short>::max());
-        assert(*windowSize.columns <= numeric_limits<unsigned short>::max());
+        assert(std::cmp_less_equal(unbox(windowSize.lines), numeric_limits<unsigned short>::max()));
+        assert(std::cmp_less_equal(unbox(windowSize.columns), numeric_limits<unsigned short>::max()));
 
-        winsize const ws { unbox<unsigned short>(windowSize.lines),
-                           unbox<unsigned short>(windowSize.columns),
-                           unbox<unsigned short>(pixels.value_or(ImageSize {}).width),
-                           unbox<unsigned short>(pixels.value_or(ImageSize {}).height) };
+        winsize const ws { .ws_row = unbox<unsigned short>(windowSize.lines),
+                           .ws_col = unbox<unsigned short>(windowSize.columns),
+                           .ws_xpixel = unbox<unsigned short>(pixels.value_or(ImageSize {}).width),
+                           .ws_ypixel = unbox<unsigned short>(pixels.value_or(ImageSize {}).height) };
 
 #if defined(__APPLE__)
         auto* wsa = const_cast<winsize*>(&ws);
@@ -82,7 +83,8 @@ namespace
 
         ptyLog()("PTY opened. master={}, slave={}", masterFd, slaveFd);
 
-        return { PtyMasterHandle::cast_from(masterFd), PtySlaveHandle::cast_from(slaveFd) };
+        return { .master = PtyMasterHandle::cast_from(masterFd),
+                 .slave = PtySlaveHandle::cast_from(slaveFd) };
     }
 
 #if defined(__linux__) && defined(UTEMPTER)
@@ -327,7 +329,7 @@ int UnixPty::write(std::string_view data)
         if (rv < 0)
             // errorlog()("PTY write failed: {}", strerror(errno));
             ptyOutLog()("PTY write of {} bytes failed. {}\n", size, strerror(errno));
-        else if (0 <= rv && static_cast<size_t>(rv) < size)
+        else if (0 <= rv && std::cmp_less(rv, size))
             // clang-format off
             ptyOutLog()("Partial write. {} bytes written and {} bytes left.",
                         rv,
@@ -335,7 +337,7 @@ int UnixPty::write(std::string_view data)
         // clang-format on
     }
 
-    if (0 <= rv && static_cast<size_t>(rv) < size)
+    if (0 <= rv && std::cmp_less(rv, size))
     {
         util::setFileBlocking(_masterFd, true);
         auto const rv2 = ::write(_masterFd, buf + rv, size - rv);
