@@ -17,7 +17,6 @@ namespace
     struct From { int value; };
     struct To { int value; };
     struct BaseOffset { int value; };
-    enum class Orientation: uint8_t { Horizontal, Vertical };
     // clang-format on
 
     Pixmap& segment_line(Pixmap& pixmap, Orientation orientation, BaseOffset base, From from, To to)
@@ -29,13 +28,13 @@ namespace
         switch (orientation)
         {
             case Orientation::Horizontal:
-                for (auto const y: ranges::views::iota(base.value - 1, base.value + 1))
-                    for (auto const x: ranges::views::iota(from.value, to.value))
+                for (auto const y: std::views::iota(base.value - 1, base.value + 1))
+                    for (auto const x: std::views::iota(from.value, to.value))
                         pixmap.paint(x, y);
                 break;
             case Orientation::Vertical:
-                for (auto const y: ranges::views::iota(from.value, to.value))
-                    for (auto const x: ranges::views::iota(base.value - 1, base.value + 1))
+                for (auto const y: std::views::iota(from.value, to.value))
+                    for (auto const x: std::views::iota(base.value - 1, base.value + 1))
                         pixmap.paint(x, y);
                 break;
         }
@@ -61,15 +60,15 @@ Pixmap& Pixmap::line(Ratio rFrom, Ratio rTo)
     if (from.x != to.x)
     {
         auto const f = linearEq(from, to);
-        for (auto const x: ranges::views::iota(0, unbox<int>(size.width)))
+        for (auto const x: std::views::iota(0, unbox<int>(size.width)))
             if (auto const y = f(x); from.y <= y && y <= to.y)
-                for (auto const i: ranges::views::iota(-z, z))
+                for (auto const i: std::views::iota(-z, z))
                     paint(x, y + i);
     }
     else
     {
-        for (auto const y: ranges::views::iota(from.y, to.y))
-            for (auto const i: ranges::views::iota(-z, z))
+        for (auto const y: std::views::iota(from.y, to.y))
+            for (auto const i: std::views::iota(-z, z))
                 paint(from.x, y + i);
     }
     return *this;
@@ -86,14 +85,14 @@ Pixmap& Pixmap::halfFilledCircleLeft()
             paint(xi, yf, 0xFF);
     };
     auto const putAbove = [&](int x, int y) {
-        putpixel(x, y - h / 2);
+        putpixel(x, y - (h / 2));
     };
     auto const putBelow = [&](int x, int y) {
-        putpixel(x, y + h / 2);
+        putpixel(x, y + (h / 2));
     };
-    auto const radius = crispy::point { w, h / 2 };
-    drawEllipseArc(putAbove, size, radius, Arc::BottomLeft);
-    drawEllipseArc(putBelow, size, radius, Arc::TopLeft);
+    auto const radius = crispy::point { .x = w, .y = h / 2 };
+    drawEllipseArc(putAbove, size, radius, Arc::UR);
+    drawEllipseArc(putBelow, size, radius, Arc::BR);
     return *this;
 }
 
@@ -107,14 +106,14 @@ Pixmap& Pixmap::halfFilledCircleRight()
             paint(x, y, 0xFF);
     };
     auto const putAbove = [&](int x, int y) {
-        putpixel(x, y - h / 2);
+        putpixel(x, y - (h / 2));
     };
     auto const putBelow = [&](int x, int y) {
-        putpixel(x, y + h / 2);
+        putpixel(x, y + (h / 2));
     };
-    auto const radius = crispy::point { w, h / 2 };
-    drawEllipseArc(putAbove, size, radius, Arc::BottomRight);
-    drawEllipseArc(putBelow, size, radius, Arc::TopRight);
+    auto const radius = crispy::point { .x = w, .y = h / 2 };
+    drawEllipseArc(putAbove, size, radius, Arc::UL);
+    drawEllipseArc(putBelow, size, radius, Arc::BL);
     return *this;
 }
 
@@ -134,8 +133,8 @@ Pixmap& Pixmap::segment_bar(int which)
     auto const r = unbox<int>(size.width) - z;
 
     auto const t = static_cast<int>(ceil(unbox<double>(size.height) * (1 / 8_th))); // Z;
-    auto const b = unbox<int>(size.height) - baseLine - z / 2;
-    auto const m = t + (b - t) / 2;
+    auto const b = unbox<int>(size.height) - baseLine - (z / 2);
+    auto const m = t + ((b - t) / 2);
 
     // clang-format off
     switch (which)
