@@ -7,12 +7,11 @@
 
 #include <crispy/point.h>
 
-#include <range/v3/view/iota.hpp>
-
 #include <algorithm>
 #include <cstdint>
 #include <format>
 #include <functional>
+#include <ranges>
 
 namespace vtrasterizer
 {
@@ -22,7 +21,7 @@ struct Ratio1
 {
     double value;
 };
-constexpr Ratio1 operator"" _th(unsigned long long ratio)
+constexpr Ratio1 operator""_th(unsigned long long ratio)
 {
     return Ratio1 { static_cast<double>(ratio) };
 }
@@ -40,28 +39,33 @@ struct Ratio
 
 // clang-format off
 struct RatioBlock { Ratio from {}; Ratio to {}; };
-constexpr RatioBlock lower(double r) noexcept { return RatioBlock { { 0, 1 - r },     { 1, 1 } }; }
-constexpr RatioBlock upper(double r) noexcept { return RatioBlock { { 0, 0 },         { 1, r } }; }
-constexpr RatioBlock left(double r) noexcept  { return RatioBlock { { 0, 0 },         { r, 1 } }; }
-constexpr RatioBlock right(double r) noexcept { return RatioBlock { { 1.f - r, 0.f }, { 1.f, 1.f } }; }
+constexpr RatioBlock lower(double r) noexcept { return RatioBlock { .from={ .x=0, .y=1 - r },     .to={ .x=1, .y=1 } }; }
+constexpr RatioBlock upper(double r) noexcept { return RatioBlock { .from={ .x=0, .y=0 },         .to={ .x=1, .y=r } }; }
+constexpr RatioBlock left(double r) noexcept  { return RatioBlock { .from={ .x=0, .y=0 },         .to={ .x=r, .y=1 } }; }
+constexpr RatioBlock right(double r) noexcept { return RatioBlock { .from={ .x=1.f - r, .y=0.f }, .to={ .x=1.f, .y=1.f } }; }
 // clang-format on
 
 constexpr crispy::point operator*(vtbackend::ImageSize a, Ratio b) noexcept
 {
-    return crispy::point { static_cast<int>(a.width.as<double>() * b.x),
-                           static_cast<int>(a.height.as<double>() * b.y) };
+    return crispy::point { .x = static_cast<int>(a.width.as<double>() * b.x),
+                           .y = static_cast<int>(a.height.as<double>() * b.y) };
 }
 
 constexpr auto linearEq(crispy::point p1, crispy::point p2) noexcept
 {
     // Require(p2.x != p1.x);
     auto const m = double(p2.y - p1.y) / double(p2.x - p1.x);
-    auto const n = double(p1.y) - m * double(p1.x);
+    auto const n = double(p1.y) - (m * double(p1.x));
     return [m, n](int x) -> int {
-        return int(double(m) * double(x) + n);
+        return int((double(m) * double(x)) + n);
     };
 }
 
+enum class Orientation : uint8_t
+{
+    Horizontal,
+    Vertical
+};
 enum class Dir : uint8_t
 {
     Top,
@@ -74,14 +78,13 @@ enum class Inverted : uint8_t
     No,
     Yes
 };
-
-enum Arc : uint8_t
+enum class Arc : uint8_t
 {
     NoArc,
-    TopLeft,
-    TopRight,
-    BottomRight,
-    BottomLeft
+    BR,
+    BL,
+    UL,
+    UR,
 };
 
 template <typename F>
@@ -92,10 +95,10 @@ auto makeDraw4WaySymmetric(Arc arc, vtbackend::ImageSize size, F putpixel)
         auto const h = unbox<int>(size.height);
         switch (arc)
         {
-            case Arc::TopLeft: putpixel(w - x, y); break;
-            case Arc::TopRight: putpixel(x, y); break;
-            case Arc::BottomLeft: putpixel(w - x, h - y); break;
-            case Arc::BottomRight: putpixel(x, h - y); break;
+            case Arc::BR: putpixel(w - x, y); break;
+            case Arc::BL: putpixel(x, y); break;
+            case Arc::UR: putpixel(w - x, h - y); break;
+            case Arc::UL: putpixel(x, h - y); break;
             case Arc::NoArc: break;
         }
     };
@@ -235,9 +238,10 @@ template <std::size_t SupersamplingFactor = 1>
 inline Pixmap blockElement(vtbackend::ImageSize size)
 {
     auto const superSize = size * SupersamplingFactor;
-    return Pixmap { atlas::Buffer(superSize.width.as<size_t>() * superSize.height.as<size_t>(), 0x00),
-                    superSize,
-                    size };
+    return Pixmap { .buffer =
+                        atlas::Buffer(superSize.width.as<size_t>() * superSize.height.as<size_t>(), 0x00),
+                    .size = superSize,
+                    .downsampledSize = size };
 }
 
 template <size_t N, typename F>
@@ -257,7 +261,7 @@ inline void Pixmap::paint(int x, int y, uint8_t value)
         return;
     if (!(0 <= x && x < w))
         return;
-    buffer.at(static_cast<unsigned>((h - y) * w + x)) = value;
+    buffer.at(static_cast<unsigned>(((h - y) * w) + x)) = value;
 }
 
 inline void Pixmap::paintOver(int x, int y, uint8_t intensity)
@@ -268,7 +272,7 @@ inline void Pixmap::paintOver(int x, int y, uint8_t intensity)
         return;
     if (!(0 <= x && x < w))
         return;
-    auto& target = buffer.at(static_cast<unsigned>((h - y) * w + x));
+    auto& target = buffer.at(static_cast<unsigned>(((h - y) * w) + x));
     target = static_cast<uint8_t>(std::min(static_cast<int>(target) + static_cast<int>(intensity), 255));
 }
 
@@ -282,8 +286,8 @@ inline void Pixmap::paintOverThick(int x, int y, uint8_t intensity, int sx, int
 template <typename F>
 Pixmap& Pixmap::fill(F const& filler)
 {
-    for (auto const y: ::ranges::views::iota(0, unbox<int>(size.height)))
-        for (auto const x: ::ranges::views::iota(0, unbox<int>(size.width)))
+    for (auto const y: std::views::iota(0, unbox<int>(size.height)))
+        for (auto const x: std::views::iota(0, unbox<int>(size.width)))
             paint(x, y, static_cast<uint8_t>(filler(x, y)));
     return *this;
 }
@@ -308,10 +312,10 @@ struct std::formatter<vtrasterizer::Arc>: std::formatter<string_view>
         switch (value)
         {
             case Arc::NoArc: name = "NoArc"; break;
-            case Arc::TopLeft: name = "TopLeft"; break;
-            case Arc::TopRight: name = "TopRight"; break;
-            case Arc::BottomLeft: name = "BottomLeft"; break;
-            case Arc::BottomRight: name = "BottomRight"; break;
+            case Arc::BR: name = "BottomToRight"; break;
+            case Arc::BL: name = "BottomToLeft"; break;
+            case Arc::UR: name = "UpToRight"; break;
+            case Arc::UL: name = "UpToLeft"; break;
         }
         return formatter<string_view>::format(name, ctx);
     }
