@@ -32,7 +32,7 @@
 
 ### How emoji are rendered
 
-    U+1F600 is the standard smily, a single grapheme cluster.
+    U+1F600 is the standard smiley, a single grapheme cluster.
     It has an east asian width of 2.
     Text shaping yields 1 glyph position with x-advance being twice as large (2 grid cells).
     The glyph renders with overlarge width.
@@ -124,27 +124,8 @@ Making use of reserved glyph slots
 #include <crispy/StrongLRUHashtable.h>
 #include <crispy/point.h>
 
-#include <range/v3/view/iota.hpp>
-
-#if defined(_WIN32)
-    #include <text_shaper/directwrite_locator.h>
-#endif
-
-#if defined(__APPLE__)
-    #include <text_shaper/coretext_locator.h>
-#endif
-
-#include <crispy/algorithm.h>
-#include <crispy/assert.h>
-#include <crispy/range.h>
-
-#include <libunicode/convert.h>
-#include <libunicode/utf8_grapheme_segmenter.h>
-
-#include <range/v3/algorithm/copy.hpp>
-#include <range/v3/view/enumerate.hpp>
-
 #include <algorithm>
+#include <ranges>
 
 using crispy::point;
 using crispy::strong_hash;
@@ -344,7 +325,7 @@ void TextRenderer::restrictToTileSize(TextureAtlas::TileCreateData& tileCreateDa
                         tileCreateData.metadata.targetSize,
                         targetSize);
 
-    for (auto const rowIndex: ranges::views::iota(uintptr_t { 0 }, unbox<uintptr_t>(targetSize.height)))
+    for (auto const rowIndex: std::views::iota(uintptr_t { 0 }, unbox<uintptr_t>(targetSize.height)))
     {
         uint8_t* targetRow = slicedBitmap.data() + (rowIndex * targetPitch);
         uint8_t const* sourceRow = tileCreateData.bitmap.data() + (rowIndex * sourcePitch);
@@ -370,7 +351,7 @@ void TextRenderer::initializeDirectMapping()
     _directMappedGlyphKeyToTileIndex.clear();
     _directMappedGlyphKeyToTileIndex.resize(LastReservedChar + 1);
 
-    for (char32_t codepoint = FirstReservedChar; codepoint <= LastReservedChar; ++codepoint)
+    for (char32_t const codepoint: std::views::iota(FirstReservedChar, LastReservedChar + 1))
     {
         if (optional<text::glyph_position> gposOpt = _textShaper.shape(_fonts.regular, codepoint))
         {
@@ -434,7 +415,8 @@ void TextRenderer::renderLine(vtbackend::RenderLine const& renderLine)
     _textClusterGrouper.renderLine(renderLine.text,
                                    renderLine.lineOffset,
                                    renderLine.textAttributes.foregroundColor,
-                                   makeTextStyle(renderLine.textAttributes.flags));
+                                   makeTextStyle(renderLine.textAttributes.flags),
+                                   renderLine.flags);
 }
 
 void TextRenderer::renderCell(vtbackend::RenderCell const& cell)
@@ -451,21 +433,14 @@ void TextRenderer::renderCell(vtbackend::RenderCell const& cell)
 
     _textClusterGrouper.renderCell(cell.position,
                                    cell.codepoints,
+                                   cell.attributes.foregroundColor,
                                    makeTextStyle(cell.attributes.flags),
-                                   cell.attributes.foregroundColor);
+                                   cell.attributes.lineFlags);
 
     if (cell.groupEnd)
         _textClusterGrouper.forceGroupEnd();
 }
 
-void TextRenderer::renderCell(vtbackend::CellLocation position,
-                              std::u32string_view graphemeCluster,
-                              TextStyle textStyle,
-                              vtbackend::RGBColor foregroundColor)
-{
-    _textClusterGrouper.renderCell(position, graphemeCluster, textStyle, foregroundColor);
-}
-
 void TextRenderer::endFrame()
 {
     _textClusterGrouper.endFrame();
@@ -535,74 +510,169 @@ void TextRenderer::renderRasterizedGlyph(crispy::point pen,
 
 bool TextRenderer::renderBoxDrawingCell(vtbackend::CellLocation position,
                                         char32_t codepoint,
-                                        vtbackend::RGBColor foregroundColor)
+                                        vtbackend::RGBColor foregroundColor,
+                                        vtbackend::LineFlags flags)
 {
     if (_fontDescriptions.builtinBoxDrawing)
-        return _boxDrawingRenderer.render(position.line, position.column, codepoint, foregroundColor);
+        return _boxDrawingRenderer.render(position.line, position.column, codepoint, flags, foregroundColor);
 
     return false;
 }
 
+crispy::point adjustPenForLineFlags(vtbackend::LineFlags lineFlags,
+                                    GridMetrics const& gridMetrics,
+                                    int glyphHeight,
+                                    int glyphBearingY,
+                                    crispy::point pen) noexcept
+{
+    using vtbackend::LineFlag;
+
+    if (lineFlags.test(LineFlag::DoubleHeightTop))
+    {
+        auto const lineHeight = unbox<int>(gridMetrics.cellSize.height);
+        pen.y += lineHeight - glyphBearingY;
+    }
+    else if (lineFlags.test(LineFlag::DoubleHeightBottom))
+    {
+        pen.y += glyphHeight - glyphBearingY;
+    }
+
+    return pen;
+}
+
+Renderable::AtlasTileAttributes adjustTileAttributesForLineFlags(
+    vtbackend::LineFlags lineFlags, Renderable::AtlasTileAttributes const& originalAttributes)
+{
+    using vtbackend::LineFlag;
+    using vtbackend::Width;
+
+    auto attributesCopy = originalAttributes;
+
+    auto const currentWidth = unbox(attributesCopy.metadata.targetSize.width);
+
+    if (lineFlags.test(LineFlag::DoubleHeightTop))
+    {
+        attributesCopy.metadata.targetSize.width = Width::cast_from(currentWidth * 2);
+        attributesCopy.metadata.x.value *= 2;
+        attributesCopy.metadata.normalizedLocation.height /= 2.0f;
+    }
+    else if (lineFlags.test(LineFlag::DoubleHeightBottom))
+    {
+        attributesCopy.metadata.targetSize.width = Width::cast_from(currentWidth * 2);
+        attributesCopy.metadata.x.value *= 2;
+        attributesCopy.metadata.normalizedLocation.y +=
+            attributesCopy.metadata.normalizedLocation.height / 2.0f;
+        attributesCopy.metadata.normalizedLocation.height /= 2.0f;
+    }
+    else if (lineFlags.test(LineFlag::DoubleWidth))
+    {
+        attributesCopy.metadata.targetSize.width = Width::cast_from(currentWidth * 2);
+        attributesCopy.metadata.x.value *= 2;
+    }
+
+    return attributesCopy;
+}
+
 void TextRenderer::renderTextGroup(std::u32string_view codepoints,
                                    gsl::span<unsigned> clusters,
                                    vtbackend::CellLocation initialPenPosition,
                                    TextStyle style,
-                                   vtbackend::RGBColor color)
+                                   vtbackend::RGBColor color,
+                                   vtbackend::LineFlags lineFlags)
 {
     if (codepoints.empty())
         return;
 
     _textRendererEvents.onBeforeRenderingText();
-    auto _ = crispy::finally { [&]() noexcept {
-        _textRendererEvents.onAfterRenderingText();
-    } };
+    auto _ = crispy::finally { [&]() noexcept { _textRendererEvents.onAfterRenderingText(); } };
 
     auto const hash = hashTextAndStyle(codepoints, style);
     text::shape_result const& glyphPositions =
         getOrCreateCachedGlyphPositions(hash, codepoints, clusters, style);
     crispy::point pen = _gridMetrics.mapBottomLeft(initialPenPosition);
 
+    using vtbackend::LineFlag;
+
+    auto const advanceScale = lineFlags.test(LineFlag::DoubleWidth) ? 2 : 1;
+
     for (auto const& glyphPosition: glyphPositions)
     {
         if (auto const* attributes = ensureRasterizedIfDirectMapped(glyphPosition.glyph))
         {
-            auto const pen1 = applyGlyphPositionToPen(pen, *attributes, glyphPosition);
-            renderRasterizedGlyph(pen1, color, *attributes);
-            pen.x += unbox<decltype(pen.x)>(_gridMetrics.cellSize.width);
+            auto const attributesCopy = adjustTileAttributesForLineFlags(lineFlags, *attributes);
+            auto pen1 = applyGlyphPositionToPen(pen, attributesCopy, glyphPosition);
+            pen1 = adjustPenForLineFlags(lineFlags,
+                                         _gridMetrics,
+                                         unbox<int>(attributes->bitmapSize.height),
+                                         attributes->metadata.y.value,
+                                         pen1);
+
+            renderRasterizedGlyph(pen1, color, attributesCopy);
+            pen.x += static_cast<decltype(pen.x)>(unbox(_gridMetrics.cellSize.width) * advanceScale);
             continue;
         }
 
         auto const hash = hashGlyphKeyAndPresentation(glyphPosition.glyph, glyphPosition.presentation);
-
         AtlasTileAttributes const* attributes =
             getOrCreateRasterizedMetadata(hash, glyphPosition.glyph, glyphPosition.presentation);
 
         if (attributes)
         {
-            auto const pen1 = applyGlyphPositionToPen(pen, *attributes, glyphPosition);
-            renderRasterizedGlyph(pen1, color, *attributes);
+            auto const attributesCopy = adjustTileAttributesForLineFlags(lineFlags, *attributes);
+            auto pen1 = applyGlyphPositionToPen(pen, attributesCopy, glyphPosition);
+            pen1 = adjustPenForLineFlags(lineFlags,
+                                         _gridMetrics,
+                                         unbox<int>(attributes->bitmapSize.height),
+                                         attributes->metadata.y.value,
+                                         pen1);
+
+            renderRasterizedGlyph(pen1, color, attributesCopy);
 
             auto xOffset = unbox(textureAtlas().tileSize().width);
             while (AtlasTileAttributes const* subAttribs = textureAtlas().try_get(hash * xOffset))
             {
-                renderTile(atlas::RenderTile::X { pen1.x + int(xOffset) },
+                auto subAttribsCopy = *subAttribs;
+                auto const subWidth = unbox(subAttribsCopy.metadata.targetSize.width)
+                                          ? unbox(subAttribsCopy.metadata.targetSize.width)
+                                          : unbox(subAttribsCopy.bitmapSize.width);
+
+                if (lineFlags.test(LineFlag::DoubleHeightTop))
+                {
+                    subAttribsCopy.metadata.targetSize.width = vtbackend::Width::cast_from(subWidth * 2);
+                    subAttribsCopy.metadata.x.value *= 2;
+                    subAttribsCopy.metadata.normalizedLocation.height /= 2.0f;
+                }
+                else if (lineFlags.test(LineFlag::DoubleHeightBottom))
+                {
+                    subAttribsCopy.metadata.targetSize.width = vtbackend::Width::cast_from(subWidth * 2);
+                    subAttribsCopy.metadata.x.value *= 2;
+                    subAttribsCopy.metadata.normalizedLocation.y +=
+                        subAttribsCopy.metadata.normalizedLocation.height / 2.0f;
+                    subAttribsCopy.metadata.normalizedLocation.height /= 2.0f;
+                }
+                else if (lineFlags.test(LineFlag::DoubleWidth))
+                {
+                    subAttribsCopy.metadata.targetSize.width = vtbackend::Width::cast_from(subWidth * 2);
+                    subAttribsCopy.metadata.x.value *= 2;
+                }
+
+                auto const drawXOffset =
+                    lineFlags.test(LineFlag::DoubleWidth) ? (int(xOffset) * 2) : int(xOffset);
+
+                renderTile(atlas::RenderTile::X { pen1.x + drawXOffset },
                            atlas::RenderTile::Y { pen1.y },
                            color,
-                           *subAttribs);
+                           subAttribsCopy);
                 xOffset += unbox(textureAtlas().tileSize().width);
             }
         }
 
         if (glyphPosition.advance.x)
         {
-
             auto numberOfCellsToAdvance =
                 std::rint(glyphPosition.advance.x / unbox<double>(_gridMetrics.cellSize.width));
-            // Only advance horizontally, as we're (guess what) a terminal. :-)
-            // Only advance in fixed-width steps.
-            // Only advance if there harfbuzz told us to.
-            pen.x +=
-                static_cast<decltype(pen.x)>(numberOfCellsToAdvance * (unbox(_gridMetrics.cellSize.width)));
+            pen.x += static_cast<decltype(pen.x)>(numberOfCellsToAdvance
+                                                  * (unbox(_gridMetrics.cellSize.width)) * advanceScale);
         }
     }
 }
@@ -770,16 +840,21 @@ auto TextRenderer::createRasterizedGlyph(atlas::TileLocation tileLocation,
     // or 0 if not overflowing.
     auto const yOverflow = max(0, yMax - _gridMetrics.cellSize.height.as<int>());
 
-    // {{{ crop underflow if yMin < 0 and yMax > 0
-    // If the rasterized glyph is underflowing below the grid cell's minimum (0),
-    // and overlaps with grid cell, then cut off at grid cell's bottom.
-    // For underscore glyphs, ymin can be lower than 0 but ymax is zero, so we need to check for yMax > 0.
-    if (yMin < 0 && yMax > 0)
+    // {{{ crop underflow if glyph is larger than cell
+    if (yMax - yMin > _gridMetrics.cellSize.height.as<int>())
     {
         auto const rowCount = (unsigned) -yMin;
         Require(rowCount <= unbox(glyph.bitmapSize.height));
         auto const pixelCount =
             rowCount * unbox<size_t>(glyph.bitmapSize.width) * text::pixel_size(glyph.format);
+        if (static_cast<size_t>(pixelCount) > glyph.bitmap.size())
+        {
+            errorLog()("TextRenderer: Glyph bitmap size: {}, dimensions: {}, pixelCount: {}",
+                       glyph.bitmap.size(),
+                       glyph.bitmapSize,
+                       pixelCount);
+            return nullopt;
+        }
         Require(0 < pixelCount && static_cast<size_t>(pixelCount) <= glyph.bitmap.size());
         rasterizerLog()("Cropping {} underflowing bitmap rows.", rowCount);
         glyph.bitmapSize.height += vtbackend::Height::cast_from(yMin);
@@ -883,12 +958,14 @@ text::shape_result TextRenderer::shapeTextRun(unicode::run_segmenter::range cons
         // clang-format on
 
         msg.append(" (");
-        for (auto const [i, codepoint]: ranges::views::enumerate(codepoints))
+        size_t i = 0;
+        for (auto const codepoint: codepoints)
         {
             if (i)
                 msg.append(" ");
             auto const cluster = clusters[i];
             msg.append("U+{:04X}/{}", unsigned(codepoint), cluster);
+            ++i;
         }
         msg.append(")\n");
 
