@@ -133,9 +133,9 @@ struct ConfigureAtlas
 struct UploadTile
 {
     TileLocation location;
-    Buffer bitmap; // texture data to be uploaded
     vtbackend::ImageSize bitmapSize;
     Format bitmapFormat;
+    Buffer bitmap;        // texture data to be uploaded
     int rowAlignment = 1; // byte-alignment per row
 };
 
@@ -423,25 +423,25 @@ constexpr auto sliced(vtbackend::Width tileWidth, uint32_t offsetX, vtbackend::I
 
         [[nodiscard]] constexpr iterator begin() noexcept
         {
-            return iterator { tileWidth,
-                              TileSliceIndex {
-                                  0,               // index
-                                  offsetX,         // begin
-                                  unbox(tileWidth) // end
+            return iterator { .tileWidth = tileWidth,
+                              .value = TileSliceIndex {
+                                  .sliceIndex = 0,         // index
+                                  .beginX = offsetX,       // begin
+                                  .endX = unbox(tileWidth) // end
                               } };
         }
 
         [[nodiscard]] constexpr iterator end() noexcept
         {
-            return iterator { tileWidth,
-                              TileSliceIndex {
-                                  0,               // index (irrelevant, undefind)
-                                  offsetForEndX(), // begin
-                                  offsetForEndX()  // end
+            return iterator { .tileWidth = tileWidth,
+                              .value = TileSliceIndex {
+                                  .sliceIndex = 0,           // index (irrelevant, undefind)
+                                  .beginX = offsetForEndX(), // begin
+                                  .endX = offsetForEndX()    // end
                               } };
         }
     };
-    return Container { tileWidth, offsetX, bitmapSize };
+    return Container { .tileWidth = tileWidth, .offsetX = offsetX, .bitmapSize = bitmapSize };
 }
 
 // {{{ implementation
@@ -495,7 +495,7 @@ TextureAtlas<Metadata>::TextureAtlas(AtlasBackend& backend, AtlasProperties atla
                                // minus the number of reserved tiles for direct-mapping, and
                                // minus one for the LRU-sentinel entry (which is why entryIndex
                                // is between 1 and capacity inclusive)
-                               _tilesInX * _tilesInY - _atlasProperties.directMappingCount - 1 },
+                               (_tilesInX * _tilesInY) - _atlasProperties.directMappingCount - 1 },
         "LRU cache for texture atlas") },
     _tileLocations { static_cast<size_t>(_tilesInX * _tilesInY) }
 {
@@ -541,8 +541,8 @@ constexpr bool TextureAtlas<Metadata>::contains(crispy::strong_hash const& id) c
 
 template <typename Metadata>
 template <typename CreateTileDataFn>
-auto TextureAtlas<Metadata>::constructTile(CreateTileDataFn createTileData,
-                                           uint32_t entryIndex) -> std::optional<TileAttributes<Metadata>>
+auto TextureAtlas<Metadata>::constructTile(CreateTileDataFn createTileData, uint32_t entryIndex)
+    -> std::optional<TileAttributes<Metadata>>
 {
     Require(1 <= entryIndex && entryIndex <= _tileCache->capacity());
     auto const tileIndex = _atlasProperties.directMappingCount + entryIndex;
@@ -556,12 +556,13 @@ auto TextureAtlas<Metadata>::constructTile(CreateTileDataFn createTileData,
 
     TileCreateData& tileCreateData = *tileCreateDataOpt;
 
-    auto tileUpload = UploadTile {};
-    tileUpload.location = tileLocation;
-    tileUpload.bitmapSize = tileCreateData.bitmapSize;
-    tileUpload.bitmapFormat = tileCreateData.bitmapFormat;
-    tileUpload.bitmap = std::move(tileCreateData.bitmap);
-    _backend.uploadTile(std::move(tileUpload));
+    _backend.uploadTile(UploadTile {
+        .location = tileLocation,
+        .bitmapSize = tileCreateData.bitmapSize,
+        .bitmapFormat = tileCreateData.bitmapFormat,
+        .bitmap = std::move(tileCreateData.bitmap),
+        .rowAlignment = 1,
+    });
 
     auto instance = TileAttributes<Metadata> {};
     instance.location = tileLocation;
